#include "3dmess.h"

DEFINE_RESPONSE_TABLE1(dreidmess, dreidchild)
	EV_WM_SIZE,
	EV_WM_RBUTTONDBLCLK,
	EV_WM_LBUTTONUP,
	EV_COMMAND(CM_PMCLIPBOARD, CmPMClipboard),
	EV_COMMAND(CM_PMAKTUALISIEREN, CmPAktualisieren),
	EV_COMMAND(CM_PEXPORT, CmPExport),
END_RESPONSE_TABLE;

dreidmess::dreidmess(TMDIClient& parent, mystring& titel,
							MesswerteListe* ListeX,
							MesswerteListe* ListeY, MesswerteListe* ListeZ,
							mystring& NameX,mystring& NameY,mystring& NameZ)
							: dreidchild(parent) {

	// Namen der Variablen bernehmen
	_Var1=NameX;
	_Var2=NameZ;
	_stellgroesse_name=NameY;
	neue_Messwerte();
	bitmap = 0;
	SetCaption(titel.c_str());
}


dreidmess::dreidmess(TMDIClient& parent, FController* Controller, bool Fehler) :
	dreidchild(parent) {

	// Liste der Eingabevariablen holen
	_EingabeVarListe=Controller->EingabeVarListe_holen();
	TEingabeVarlisteIter Iter(_EingabeVarListe);

	// die ersten beiden Eingabevariablen holen
	Var1=Iter.Current();
	Iter++;
	Var2=Iter.Current();

	// Ausgabevariable holen
	if (!Fehler)
		Var3=&Controller->_AusgabeVar;
	else
		Var3=&Controller->_FehlerVar;


	// die Zeiger auf die Listen der Mewerte, die von den Variablen
	// verwaltet werden, werden geholt
	_pListeX=Var1->Messwerte_holen();
	_pListeY=Var3->Messwerte_holen();
	_pListeZ=Var2->Messwerte_holen();

	// Namen der Variablen initialisieren
	_Var1=Var1->Name();
	_Var2=Var2->Name();

	if (Fehler)
		_stellgroesse_name=Controller->FehlerVar_holen();
	else
		_stellgroesse_name=Controller->AusgabeVar_holen();

	neue_Messwerte();

	bitmap = 0;

	mystring temp;
	temp = "Values 3D-" + Var3->Name();
	SetCaption(temp.c_str());

}
void dreidmess::neue_Messwerte() {

	// neue Mewerte kopieren
	_ListeX=*_pListeX;
	_ListeY=*_pListeY;
	_ListeZ=*_pListeZ;

	// Minimum und Maximum jeder Liste ermitteln
	minmax(*_pListeX,xmin,xmax);
	minmax(*_pListeY,ymin,ymax);
	minmax(*_pListeZ,zmin,zmax);

	_nichtleer=_ListeX.GetItemsInContainer() > 1;
	_neuzeichnen=true;
	Konstante_berechnen();
}
void dreidmess::minmax(MesswerteListe& Liste,FTyp& Min,FTyp& Max) {

	// ermittelt das Minimum und das Maximum einer gegebenen Liste und
	// sorgt bei konstaten Mewerten fr einen sinnvollen Ersatz

	MesswerteIter Iter(Liste);
	Min=Max=Iter.Current();

	long int zaehler(Liste.GetItemsInContainer());

	while (Iter && (zaehler>0)) {

		if (Iter.Current() > Max)
			Max=Iter.Current();
		if (Iter.Current() < Min)
			Min=Iter.Current();

		--zaehler;
		Iter++;
	}

	// Mewerte konstant ?

	if (Min==Max) {
			if (Min==0) {

				Max=1;

				Min=-1;

			}

			else {

				FTyp dy;

				if (Min<0)

					dy=-Min;

				else

					dy=Min;


				Min-=dy;

				Max+=dy;

			}

		}
}
void dreidmess::SetupWindow() {
	dreidchild::SetupWindow();

	SetIcon(GetApplication(),ICON_MESSWERTE);

	Konstante_berechnen();

	// geeignete Gre des Fensters whlen
	TRect parentRect;
	Parent->GetWindowRect(parentRect);

	int breite;

	if (parentRect.Width() < parentRect.Height())
		breite=parentRect.Width();
	else
		breite=parentRect.Height();

	breite /= 3;

	MoveWindow(60,60,breite,breite);

}
// Paint------------------------------------------------------------------------
void dreidmess::Paint(TDC& paintdc, BOOL, TRect& ) {

	FTyp xe1,ye1,xe2,ye2;

	TRect groesse=GetClientRect();
	TClientDC mydc(*this);

	// nur nichtleere Listen knnen gezeichnet werden
	if (!_nichtleer) {
		mydc.SelectObject(*_Schrift);
		mydc.SetTextAlign(TA_CENTER | TA_BASELINE);
		mydc.TextOut(groesse.left+groesse.Width()/2,
						groesse.bottom-groesse.Height()/2,
						"No values, yet.",23);
		return;
	}

	// hat sich der Name einer Variablen gendert ?
	bool Aenderung=(_Var1!=Var1->Name()) || (_Var2!=Var2->Name()) ||
									(_stellgroesse_name!=Var3->Name());

	if (Aenderung) {
		// falls ja, neue Mewerte
		neue_Messwerte();
		_Var1=Var1->Name();
		_Var2=Var2->Name();
		_stellgroesse_name=Var3->Name();

		mystring temp;
		temp = "Values 3D-" + Var3->Name();
		SetCaption(temp.c_str());
	}



	if (!_neuzeichnen && bitmap) {

		// falls die Mewerte noch aktuell sind und die Darstellung im Speicher
		// als Bitmap noch vorhanden ist, wird diese nur neu in das Fenster
		// kopiert

		memDC=new TMemoryDC(mydc);
		memDC->SelectObject(*bitmap);
		paintdc.BitBlt(0,0, bitmap->Width(), bitmap->Height(),
							*memDC,0,0,SRCCOPY);
		delete memDC;
	}
	else {
		// sonst neu skalieren, neu zeichnen
		neu_skalieren(groesse.Width(),groesse.Height());
		_neuzeichnen=false;

		// Wartebox zeichnen
		TWarteBox warte(this,DIALOG_WARTE,"drawing...",TColor::LtGreen);
		warte.Create();
		warte.Show(SW_SHOW);

		Parent->SetCursor(0,IDC_WAIT);

		// eventuell vorhandene Bitmap lschen, da diese nicht mehr aktuell ist
		if (bitmap) {
			delete bitmap;
			bitmap =0;
		}

		// neue Bitmap anlegen
		bitmap = new TBitmap(mydc, groesse.Width(),groesse.Height(),true);
		memDC=   new TMemoryDC(mydc);
		memDC->SelectObject(*bitmap);


		memDC->SelectStockObject(WHITE_PEN);
		memDC->SelectStockObject(WHITE_BRUSH);
		memDC->Rectangle(groesse);
		memDC->SelectStockObject(BLACK_PEN);

		// Achsen ("hinten") malen
		male_Achsen(*memDC);

		// 2D-Darstellung der Mewerte auf dem Boden ("Schatten") zeichnen,
		// dabei ist y = ymin

		memDC->SelectObject(TPen(TColor::Gray));
		MesswerteIter IterX1(_ListeX),IterZ1(_ListeZ);

		FTyp Anzahl(_ListeX.GetItemsInContainer());
		FTyp count=1;
		FTyp xe1,ye1;

		FTyp lastx,lastz,thisx,thisz;

		// Sonderfall: 1. Mewerttupel
		if (IterX1 && IterZ1) {
			lastx=IterX1.Current();
			lastz=IterZ1.Current();

			projiziere(IterX1.Current(),ymin,IterZ1.Current(),
							xe1,ye1);

			memDC->MoveTo(xe1,ye1);
			IterX1++;
			IterZ1++;

		}

		// weitere Mewerttupel
		while (IterX1 && IterZ1) {

			projiziere(IterX1.Current(),ymin,IterZ1.Current(),
							xe1,ye1);

			// durch Linie verbinden
			memDC->LineTo(xe1,ye1);

			projiziere(IterX1.Current(),ymin,IterZ1.Current(),
							xe1,ye1);
			memDC->MoveTo(xe1,ye1);

			// fr einen neuen Mewert ein Quadrat zeichnen
			memDC->Rectangle(xe1-2,ye1-2,xe1+2,ye1+2);
			lastx=IterX1.Current();
			lastz=IterZ1.Current();

			IterX1++;
			IterZ1++;
			++count;

			// Wartebox aktualisieren

			if (!(((int)count) % 20))
				warte.SetValue((count)/Anzahl*50);
		}

		// 3D-Darstellung der Mewerte
		memDC->SelectObject(TPen(TColor::LtRed));
		memDC->SelectObject(TBrush(TColor::Black));

		MesswerteIter IterX(_ListeX),IterY(_ListeY),IterZ(_ListeZ);

		// Sonderfall 1. Mewerttripel
		count = 1;
		if (IterX && IterY && IterZ) {
			projiziere(IterX.Current(),IterY.Current(),IterZ.Current(),
							xe1,ye1);
			memDC->MoveTo(xe1,ye1);
			IterX++;
			IterY++;
			IterZ++;
		}

		// weitere Mewerttripel
		while (IterX && IterY && IterZ) {

			projiziere(IterX.Current(),IterY.Current(),IterZ.Current(),
							xe1,ye1);
			// verbinden
			memDC->LineTo(xe1,ye1);
			// Rechteck (s.o.)
			memDC->Rectangle(xe1-2,ye1-2,xe1+2,ye1+2);

			IterX++;
			IterY++;
			IterZ++;
			++count;

			// Wartebox aktualisieren
			if (!(((int)count) % 20))
				warte.SetValue(50+count/Anzahl*50);
		}

		// restliche Achsen zeichnen ("vorne")
		male_restliche_Achsen(*memDC);

		// Darstellung aus der Bitmap in das Fenster kopieren
		mydc.BitBlt(0,0, bitmap->Width(), bitmap->Height(),
					*memDC,groesse.left,groesse.top,SRCCOPY);

		delete memDC;

		Parent->SetCursor(0,IDC_ARROW);
	}
}

void dreidmess::EvSize(UINT sizeType, TSize& Size) {
	// wenn das Fenster in der Gre verndert wird, neuzeichnen
	_neuzeichnen=true;
	dreidchild::EvSize(sizeType,Size);
	Invalidate(false);
}

void dreidmess::EvRButtonDblClk(UINT, TPoint&) {
	CmPAktualisieren();
}

void dreidmess::CmPAktualisieren() {
	// fr eine Aktualisierung mssen zunchst neue Mewerte beschaft werden
	neue_Messwerte();
	// dann neuzeichnen
	Invalidate(false);
}
dreidmess::~dreidmess() {

	// falls Bitmap verwendet, diese freigeben
	if (bitmap)
		delete (bitmap);
}

void dreidmess::EvLButtonUp(UINT, TPoint& point) {
	TPopupMenu PopupMenu;
	TRect rect;
	GetWindowRect(rect);

	// Men erstellen
	PopupMenu.AppendMenu(MF_STRING, CM_PMCLIPBOARD, "Copy into clipboard");
	PopupMenu.AppendMenu(MF_STRING, CM_PEXPORT, "Export data to file...");
	PopupMenu.AppendMenu(MF_SEPARATOR);
	PopupMenu.AppendMenu(MF_STRING, CM_PMAKTUALISIEREN, "Update");

	// Men anzeigen
	PopupMenu.TrackPopupMenu(TPM_LEFTBUTTON,
										point.x+rect.left, point.y+rect.top,
										0,HWindow,0);
}
void dreidmess::CmPMClipboard() {

	// kopiert die Darstellung in die Zwischenablage

	Invalidate(false);
	TClientDC clientDC(*this);
	TMemoryDC memDC(clientDC);
	TRect groesse=GetClientRect();

	// neue Bitmap erstellen
	TBitmap* bitmap=new TBitmap(clientDC,groesse.Width(),groesse.Height(),FALSE);
	memDC.SelectObject(*bitmap);
	memDC.BitBlt(0,0,groesse.Width(),groesse.Height(),clientDC,0,0,SRCCOPY);

	// in das Clipboard kopieren
	TClipboard &clipboard = OpenClipboard();
	clipboard.EmptyClipboard();
	bitmap->ToClipboard(clipboard);
	clipboard.CloseClipboard();

	delete bitmap;
}

void dreidmess::CmPExport() {

	// exportiert die Mewerttripel in eine Datei (ASCII)

	// Speicherdialog erstellen
	TOpenSaveDialog::TData fileData(OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT,
			"ASCII Files(*.dat)|*.dat",0,0,"dat");
	  strcpy(fileData.FileName,mystring(_stellgroesse_name+".dat").c_str());
		TFileSaveDialog *dialog= new TFileSaveDialog(this,fileData);
		int result= dialog->Execute();
		delete dialog;

		if (result==IDOK) {

			Parent->SetCursor(0,IDC_WAIT);

			// Dateinamen ermitteln
			ofstream of(fileData.FileName);

			// alle Mewerttripel ausgeben
			MesswerteIter IterX(_ListeX),IterY(_ListeY),IterZ(_ListeZ);

			while (IterX && IterY && IterZ) {
				of << IterX.Current() << " ";
				of << IterZ.Current() << " ";
				of << IterY.Current() << endl;

				IterX++;
				IterY++;
				IterZ++;
			}

			of.close();
			Parent->SetCursor(0,IDC_ARROW);
		}
}
