#include "contldlg.h"

DEFINE_RESPONSE_TABLE1(TControlDialog, TCenterDialog)
	EV_CHILD_NOTIFY(IDOK, BN_CLICKED, HandleOK),
	EV_CHILD_NOTIFY(IDCANCEL, BN_CLICKED, HandleAbbruch),
END_RESPONSE_TABLE;

TControlDialog::TControlDialog(TWindow* parent, MetaController* Meta) :
	TCenterDialog (parent,DIALOG_CONTROL),
	_Meta(Meta),
	_EpochenBereich(0,32000),

	// Edit-Fenster konstruieren
	_EditEpochen(this,IDC_EPOCHEN,16),
	_EditProzent(this,IDC_PROZENT,8),
	_EditRegelnEpochen(this,IDC_REGELEPOCHEN,16),
	_EditRegelnEpochen2(this,IDC_REGELEPOCHEN2,16),
	_EditDelta(this,IDC_DELTA,16),
	_EditDeltaPlus(this,IDC_DELTAPLUS,16),
	_EditDeltaMinus(this,IDC_DELTAMINUS,16),

	// restliche Dialogelemente initialisieren
	_Verfahren(this,IDC_VERFAHREN,30),
	_RegelLernVerfahren(this,IDC_REGELLERNVERF,30),
	_Aktion(this,IDC_AKTION,30),
	_CheckTrace(this,IDC_TRACE),
	_CheckKeep(this,IDC_KEEP)
	{
		// Parameter vom Metacontroller besorgen
		_Parameter=Meta->Parameter_holen();
}

void TControlDialog::SetupWindow() {
	char buf[16];

	TCenterDialog::SetupWindow();

	// Zyklen in das Fenster schreiben
	wsprintf(buf,"%d",_Parameter._Epochen);
	_EditEpochen.SetText((char *)buf);

	// Schrittweiten in die Fenster schreiben
	sprintf(buf,"%.4f",_Parameter._delta);
	_EditDelta.SetText((char *)buf);
	sprintf(buf,"%.4f",_Parameter._delta_plus);
	_EditDeltaPlus.SetText((char *)buf);
	sprintf(buf,"%.4f",_Parameter._delta_minus);
	_EditDeltaMinus.SetText((char *)buf);

	// Auswahlmglichkeiten fr die Adaption der Fuzzy-Mengen angeben
	_Verfahren.AddString("Fuzzy-Backpropagation");
	_Verfahren.AddString("Fuzzy-SuperSAB");
	_Verfahren.AddString("Fuzzy-Delta-Bar-Delta");

	_Verfahren.AddString("Fuzzy-Backpropagation & move conclusion");
	_Verfahren.AddString("Fuzzy-SuperSAB & move conclusion");
	_Verfahren.AddString("Fuzzy-Delta-Bar-Delta & move conclusion");

	// aktuell verwendetes Verfahren in die Auswahl schreiben
	switch (_Parameter._Verfahren)  {
		case (fbackprop) : {
			_Verfahren.SetText("Fuzzy-Backpropagation");
			break;
		}
		case (fssab) : {
			_Verfahren.SetText("Fuzzy-SuperSAB");
			break;
		}
		case (fdbd) : {
			_Verfahren.SetText("Fuzzy-Delta-Bar-Delta");
			break;
		}

		case (fbackpropv) : {
			_Verfahren.SetText("Fuzzy-Backpropagation & move conclusion");
			break;
		}
		case (fssabv) : {
			_Verfahren.SetText("Fuzzy-SuperSAB & move conclusion");
			break;
		}
		case (fdbdv) : {
			_Verfahren.SetText("Fuzzy-Delta-Bar-Delta & move conclusion");
			break;
		}
	}

	// Auswahlmglichkeiten fr die Aktion angeben
	_Aktion.AddString("No adaption");
	_Aktion.AddString("Fuzzy-set adaption");
	_Aktion.AddString("Control rule learning");

	// aktuelle Aktion in die Auswahl schreiben
	switch (_Parameter._Lernmodus)  {
		case (simulieren) : {
			_Aktion.SetText("No adaption");
			break;
		}
		case (Fuzzy_adaptieren) : {
			_Aktion.SetText("Fuzzy-set adaption");
			break;
		}
		case (Regeln_lernen) : {
			_Aktion.SetText("Control rule learning");
			break;
		}
	}
	// sollen die Vernderungen an den Fuzzy-Mengen beim Lernen angezeigt werden?
	if (_Parameter._trace)
		_CheckTrace.Check();

	// sollen bereits definiert Regeln beim Regellernen erhalten werden ?
	if (!_Parameter._alte_Regeln_verwerfen)
		_CheckKeep.Check();

	// initialisiert Mindestaktivierung der Regeln beim Verfahren von Nauck
	sprintf(buf,"%.2f",_Parameter._MindestRegelAktivierung*100);
	_EditProzent.SetText((char *)buf);

	// Zyklen fr die einzelnen Phasen beim Regellernen
	wsprintf(buf,"%d",_Parameter._RegelnEpochen);
	_EditRegelnEpochen.SetText((char *)buf);
	wsprintf(buf,"%d",_Parameter._RegelnEpochen2);
	_EditRegelnEpochen2.SetText((char *)buf);

	// Auswahlmglichkeiten fr die Regellernverfahren angeben
	_RegelLernVerfahren.AddString("Nauck");
	_RegelLernVerfahren.AddString("Stellmachs Bottom-Up");

	// aktuell verwendetes Regellernverfahren in die Auswahl schreiben
	switch (_Parameter._RegelLernVerfahren)  {
		case (Stellmach) : {
			_RegelLernVerfahren.SetText("Stellmachs Bottom-Up");
			break;
		}
		case (Nauck) : {
			_RegelLernVerfahren.SetText("Nauck");
			break;
		}
	}
}

void TControlDialog::HandleOK() {

	// holt die neuen Parameter aus der Dialogbox und nimmt die
	// entsprechenden nderungen vor

	char Wert[50];

	//--------------- Parameter fr die Adaption der Fuzzy-Mengen ---------------

	// Zyklen holen, Bereich berprfen, speichern
	_EditEpochen.GetText((char *)&Wert,16);
	if (!_EpochenBereich.IsValid((char *)&Wert) ) {
		MessageBox("Epochs > 0 !","Range-Error",MB_OK);
		return;
	}
	_Parameter._Epochen=atoi((char *)&Wert);

	// Schrittweite holen, Bereich berprfen, speichern
	_EditDelta.GetText((char *)&Wert,16);
	if (atof((char *)&Wert)< 0) {
		MessageBox("Delta > 0 !","Range-Error",MB_OK);
		return;
	}
	_Parameter._delta=atof((char *)&Wert);

	// Schrittweite+ holen, Bereich berprfen, speichern
	_EditDeltaPlus.GetText((char *)&Wert,16);
	if (atof((char *)&Wert)< 0) {
		MessageBox("Delta+ > 0 !","Range-Error",MB_OK);
		return;
	}
	_Parameter._delta_plus=atof((char *)&Wert);

	// Schrittweite- holen, Bereich berprfen, speichern
	_EditDeltaMinus.GetText((char *)&Wert,16);
	if (atof((char *)&Wert)< 0) {
		MessageBox("Delta- > 0 !","Range-Error",MB_OK);
		return;
	}
	_Parameter._delta_minus=atof((char *)&Wert);

	// Verfahren fr die Adaption der Fuzzy-Mengen holen
	_Verfahren.GetText((char *)&Wert,50);
	mystring Eingabe(Wert);
	if (Eingabe=="Fuzzy-Backpropagation") {
			_Parameter._Verfahren=fbackprop;
	}
	if (Eingabe=="Fuzzy-SuperSAB") {
			_Parameter._Verfahren=fssab;
	}
	if (Eingabe=="Fuzzy-Delta-Bar-Delta") {
			_Parameter._Verfahren=fdbd;
	}
	if (Eingabe=="Fuzzy-Backpropagation & move conclusion") {
			_Parameter._Verfahren=fbackpropv;
	}
	if (Eingabe=="Fuzzy-SuperSAB & move conclusion") {
			_Parameter._Verfahren=fssabv;
	}
	if (Eingabe=="Fuzzy-Delta-Bar-Delta & move conclusion") {
			_Parameter._Verfahren=fdbdv;
	}
	// tracen ?
	_Parameter._trace=_CheckTrace.GetCheck();

	//--------------------- Parameter fr das Regellernen -----------------------

	// Verfahren fr das Regellernen holen
	_RegelLernVerfahren.GetText((char *)&Wert,30);
	Eingabe=Wert;
	if (Eingabe=="Nauck") {
			_Parameter._RegelLernVerfahren=Nauck;
	}
	if (Eingabe=="Stellmachs Bottom-Up") {
			_Parameter._RegelLernVerfahren=Stellmach;
	}

	// Zyklen fr Phase I holen, Bereich berprfen, speichern
	_EditRegelnEpochen.GetText((char *)&Wert,16);
	if (!_EpochenBereich.IsValid((char *)&Wert) ) {
		MessageBox("Epochs > 0 !","Range-Error",MB_OK);
		return;
	}
	_Parameter._RegelnEpochen=atoi((char *)&Wert);

	// Zyklen fr Phase II holen, Bereich berprfen, speichern
	_EditRegelnEpochen2.GetText((char *)&Wert,16);

	if (!_EpochenBereich.IsValid((char *)&Wert) ) {
		MessageBox("Epochs > 0 !","Range-Error",MB_OK);
		return;
	}

	_Parameter._RegelnEpochen2=atoi((char *)&Wert);

	// Mindestaktivierung holen, Bereich berprfen, speichern
	_EditProzent.GetText((char *)&Wert,16);
	FTyp Prozent=atof(Wert);
	if ((Prozent < 0) || (Prozent > 100)) {
		MessageBox("0 <= percent <= 100 !","Range-Error",MB_OK);
		return;
	}
	_Parameter._MindestRegelAktivierung=Prozent/100;

	// existierende Regeln erhalten ?
	_Parameter._alte_Regeln_verwerfen=!_CheckKeep.GetCheck();

	//-------------------------- Aktion holen -----------------------------------
	_Aktion.GetText((char *)&Wert,30);
	mystring Eingabe2(Wert);

	if (Eingabe2=="No adaption") {
			_Parameter._Lernmodus=simulieren;
	}
	if (Eingabe2=="Fuzzy-set adaption") {
			_Parameter._Lernmodus=Fuzzy_adaptieren;
	}
	if (Eingabe2=="Control rule learning") {
			_Parameter._Lernmodus=Regeln_lernen;
	}

	// falls bisher ok, Gltigkeit der Parameter signalisieren und
	// Dialog schlieen
	_Abbruch=false;
	TCenterDialog::CloseWindow();
}
void TControlDialog::HandleAbbruch() {

	// Abbruch signalisieren
	_Abbruch=true;
	TCenterDialog::CloseWindow();
}
