#include "EingabeV.h"

EingabeVariable::EingabeVariable(): Variable() {
	_Eingabewert=1e99;
}

EingabeVariable::EingabeVariable(mystring Name, FTyp& min, FTyp& max):
	 Variable(Name, min, max)  {
	_Name=Name;
	_Eingabewert=1e99;
}

FTyp EingabeVariable::Treffergrad(mystring& Name) const {

	// liefert den Treffergrad der Fuzzy-Menge mit dem Namen "Name"
	// bei dem aktuellen Eingabewert

	AssociationType assoc(&Name,0);
	return(_FuzzySets.Find(assoc)->Value()->y_wert(_Eingabewert));
}


void EingabeVariable::setze_Eingabewert(FTyp& x) {


	// setzt den aktuellen Eingabewert auf "x",

	// und berechnet die neuen Aktivierungen der Fuzzy-Mengen, falls

	// sich der Eingabewet gendert hat


	if (x !=_Eingabewert) {

		_Eingabewert=x;

		IteratorType Iter(_FuzzySets);
		while (Iter) {
			Iter.Current().Value()->berechne_Aktivierung(x);
			Iter++;
		}
	}
}

int EingabeVariable::operator==(const EingabeVariable& mc) const
{
	// vergleicht den Namen zweier Eingabevariabler
	return (_Name==mc._Name);
}

FsSet* EingabeVariable::hol_bestes_FuzzySet() {

	// liefert diejenige Fuzzy-Menge, die fr den momentanen Eingabewert
	// eine maximale Aktivierung aufweist (fr Stellmachs Regellernverf.)

	IteratorType Iter(_FuzzySets);
	FTyp temp,max=0;
	FsSet* bestes=0;
	while (Iter) {
		temp=Iter.Current().Value()->Aktivierung();
		if ((Iter.Current().Value()->_fuer_Regel_verfuegbar) && (temp > max)) {
			max = temp;
			bestes=(FsSet*) Iter.Current().Value();
		}
		Iter++;
	}
	return bestes;
}

void EingabeVariable::alle_FuzzySets_fuer_Regeln_erlauben() {

	// macht alle Fuzzy-Mengen der Variablen fr das Regellernen verfgbar

	IteratorType Iter(_FuzzySets);
	while (Iter) {
		((FsSet*)Iter.Current().Value())->_fuer_Regel_verfuegbar=true;
		Iter++;
	}
}

