#include "fsint.h"

FsInt::FsInt() {
  _links_offen=_rechts_offen=0;
  _b=_m=_x1=_x2=0;
  _SFunktion=false;
}

void FsInt::setze_Intervall(FTyp x1,FTyp x2, int links_offen, int rechts_offen) {

	// Initialisierung der Eckpunkte und Berechnung der Konstanten

	_x1=x1;
	_x2=x2;
	_m= 1/ ( x2- x1);
	_b = - _m * _x1;

	_links_offen=links_offen;
	_rechts_offen=rechts_offen;
}
void FsInt::skalieren (FTyp& Faktor) {

	// die Breite des Fuzzy-Intervalls wird adaptiert, falls die Randbedingungen
	// (minv,maxv) nicht verletzt werden

	if (Faktor > 0) {
		// belohnen ...
		FTyp dx = Faktor * (_x2 -_x1);

		if (_x2>_x1) {
			if ((_x1 - dx) < _minv)
				_x1 =_minv;
			else
				_x1 -= dx;
		}
		else {
			if ((_x1 - dx) > _maxv)
				_x1=_maxv;
			else
				_x1-=dx;
		}
	}
	else {
		// bestrafen...
		FTyp dx = Faktor * (_x2 -_x1);
		if (_x2>_x1) {
			if (((_x1 - dx)+ 0.05) < _x2)
				_x1 -= dx;
		}
		else {
			if ((_x1 - dx) - 0.05 > _x2)
				_x1-=dx;
		}
	}

	// nach der Skalierung mssen die Konstanten aktualisiert werden
	_m= 1/ ( _x2- _x1);
	_b = - _m * _x1;
}

void FsInt::malen(TDC& dc,mystring& Name, TRect& viereck,FTyp& min,FTyp& max, TColor Farbe, bool istFehler) {

	// das Fuzzy-Intervall wird in ein Fenster gezeichnet

	// Fuzzy-Intervalle, die von der Fehler-Regelbasis verwendet werden,
	// werden gestrichelt dargestellt

	if (istFehler)
		dc.SelectObject(TPen(Farbe,1,PS_DOT));
	else
		dc.SelectObject(TPen(Farbe,1,PS_SOLID));

	dc.SetTextColor(Farbe);

	// Projektion der Eckpunkte
	FTyp dx=max-min;
	FTyp sx=(viereck.right - viereck.left)/dx;

	FTyp x1=viereck.left+(_x1-min)*sx;
	FTyp x2=viereck.left+(_x2-min)*sx;

	FTyp y;

	if (_SFunktion && (x1!=x2)) {

		// Fuzzy-Intervall ist vom Typ S-Funktion
		y=y_wert((x1-viereck.left)/sx+min);
		y=viereck.bottom-y*(viereck.bottom-viereck.top);
		dc.MoveTo(x1,(int)y);

		// Intervall zeichnen
		if (x1 < x2) {
			for (int x=x1; x <=x2; ++x) {
				y=y_wert((x-viereck.left)/sx+min);
				y=viereck.bottom-y*(viereck.bottom-viereck.top);
				dc.LineTo(x,(int)y);
			}
		}
		else {
			for (int x=x1; x >=x2; --x) {
				y=y_wert((x-viereck.left)/sx+min);
				y=viereck.bottom-y*(viereck.bottom-viereck.top);
				dc.LineTo(x,(int)y);

			}
		}

	}
	else {
		// Fuzzy-Intervall ist linear

		dc.MoveTo(x1,viereck.bottom);

		dc.LineTo(x2,viereck.top);

	}
	// Eckpunkte mit Rechtecken markieren
	dc.Rectangle(x1-3,viereck.bottom-6, x1+3,viereck.bottom);
	dc.Rectangle(x2-3,viereck.top+6, x2+3,viereck.top);

	// Name des Fuzzy-Intervalls ausgeben
	dc.TextOut(x2+5,viereck.top-6,Name.c_str());
}

void FsInt::verschieben(FTyp delta_x) {

	// beide Eckpunkte werden um delta_x verschoben
	_x1 += delta_x;
	_x2 += delta_x;
	_b = - _m * _x1;
}

void FsInt::verschiebe_links(FTyp delta_x) {

	// verschiebe den linken Eckpunkt um delta_x
	if (_x1<_x2)
		_x1 += delta_x;
	else
		_x2 += delta_x;

	_m= 1/ ( _x2- _x1);
	_b = - _m * _x1;
}

void FsInt::verschiebe_rechts(FTyp delta_x) {

	// verschiebe den rechten Eckpunkt um delta_x
	if (_x1<_x2)
		_x2 += delta_x;
	else
		_x1 += delta_x;

	_m= 1/ ( _x2- _x1);
	_b = - _m * _x1;
}

bool FsInt::verschiebbar(FTyp delta_x,FTyp min, FTyp max) {

	// ist das Fuzzy-Intervall unter Einhaltung der Restriktionen
	// um delta_x verschiebbar ?

	if (_x1<_x2) {
		return ((_x1 + delta_x >= min) && ( _x2 + delta_x <= max));
	}
	else {
		return ((_x2 + delta_x >= min) && (_x1 + delta_x <= max));
	}
}

bool FsInt::links_verschiebbar(FTyp delta_x,FTyp min, FTyp max) {

	// ist der linke Eckpunkt unter Einhaltung der Restriktionen
	// um delta_x verschiebbar ?

	if (_x1<_x2) {
		return ((_x1 + delta_x >= min) && (_x1 + delta_x < _x2));
	}
	else {
		return ((_x2 + delta_x >= min ) && (_x2 + delta_x < _x1 ));
	}
}

bool FsInt::rechts_verschiebbar(FTyp delta_x,FTyp min, FTyp max) {

	// ist der rechte Eckpunkt unter Einhaltung der Restriktionen
	// um delta_x verschiebbar ?
	if (_x1<_x2) {
		return ((_x2 + delta_x <= max) && ((_x2 + delta_x) > _x1));
	}
	else {
		return ((_x1 + delta_x <= max ) && ((_x1 + delta_x) > _x2));
	}
}
bool FsInt::steigend() {
	return(_x1 < _x2);
}
bool FsInt::enthaelt_Teilmenge_von(FTyp x1,FTyp x2) {

	// ermittelt, ob sich das Fuzzy-Intervall mit dem Intervall [x1,x2]
	// berschneidet,
	// wird zum Neuzeichen im Editor verwendet

	if (_x1 < _x2)
		return ((x1 >= _x1 && x1<=_x2) || (x2 >= _x1 && x2<=_x2));
	else
		return ((x1 >= _x2 && x1<=_x1) || (x2 >= _x2 && x2<=_x1));
}

ostream& operator << (ostream& os, FsInt& einFsInt) {
	os << einFsInt._x1 << " " << einFsInt._x2 << " "
		<< einFsInt._minv << " " << einFsInt._maxv << " "
		<< einFsInt._links_offen << " " << einFsInt._rechts_offen << " "
		<< einFsInt._SFunktion << endl;

	return os;
}

istream& operator >> (istream& is, FsInt& einFsInt) {


	is >> einFsInt._x1 >> einFsInt._x2
		>> einFsInt._minv >> einFsInt._maxv
		>> einFsInt._links_offen >> einFsInt._rechts_offen;
	int dummy;
	is >> dummy ;
	einFsInt._SFunktion=dummy;

	// Konstante werden nicht mitgespeichert, mssen also neu berechnet werden
	einFsInt._m= 1/ (einFsInt._x2- einFsInt._x1);
	einFsInt._b = - einFsInt._m * einFsInt._x1;

	return is;
}


FTyp FsInt::x_wert(FTyp& y) const {

	// invertiert das Fuzzy-Intervall

	if (_x2 == _x1)
		 return _x1;

	 if (_SFunktion) {

		// Fuzzy-Intervall ist vom Typ S-Funktion
		if (y>0.5) {
			FTyp c = 2/ ((_x2 - _x1) * (_x2 - _x1));

			if (_x1 < _x2)
				return - 0.5 * (-2 *c *_x2 + 2 * sqrt(fabs(c - c * y)))/c;
			else
				return - 0.5 * (-2 *c *_x2 - 2 * sqrt(fabs(c - c * y)))/c;
		}
		else {
			FTyp c= 2/((_x2 - _x1) * (_x2 - _x1));

			if (_x1 < _x2)
				return 0.5 * (2*c*_x1 + 2 * sqrt(fabs(c * y)))/c;
			else
				return 0.5 * (2*c*_x1 - 2 * sqrt(fabs(c * y)))/c;
		}
	}
	else
		// die Invertierung gestaltet sich bei linearen Fuzzy-Intervallen
		// weitaus einfacher
		return (y-_b) / _m;

}

FTyp FsInt::y_wert(FTyp& x) const {

	// liefert den Zugehrigkeitsgrad (x)

	if (_x2 == _x1)
		 return 0;

	if (_SFunktion) {

		// Fuzzy-Intervall ist vom Typ S-Funktion
		FTyp Faktor;
		if (_x1<_x2) {
			if (x < (_x1+(_x2-_x1)/2)) {

				Faktor=(x-_x1)*_m;
				Faktor*=Faktor;
				Faktor*=2;
				return Faktor;
			}
			else {
				Faktor=(x-_x2)*_m;
				Faktor*=Faktor;
				Faktor*=-2;
				return Faktor+1;
			}
		}
		else {
			if (x > (_x2+(_x1-_x2)/2)) {

				Faktor=(x-_x1)*_m;
				Faktor*=Faktor;
				Faktor*=2;
				return Faktor;
			}
			else {
				Faktor=(x-_x2)*_m;
				Faktor*=Faktor;
				Faktor*=-2;
				return Faktor+1;
			}
		}
	}
	else
		// Fuzzy-Intervall ist linear
		return _m * x + _b;
}
