#include "fsset.h"

FsSet::FsSet()   {
	_Name="";
	_Anzahl_Intervalle=0;
	_fuer_Regel_verfuegbar=true;
	_SFunktion=false;
}

FsSet::FsSet(mystring& Name) {
  _Name = Name;
  _Anzahl_Intervalle=0;
  _fuer_Regel_verfuegbar=true;
  _SFunktion=false;
}

mystring FsSet::Name() {
  return _Name;
}

void FsSet::setze_SFunktion(bool S) {

	// setz den Typ der Fuzzy-Menge (S-Funktion oder linear)

	_SFunktion = S;

	if (_Anzahl_Intervalle>1) {
		_Intervall1.setze_SFunktion(S);
		_Intervall2.setze_SFunktion(S);
	}
	else
		if (_Anzahl_Intervalle==1) {
			_Intervall1.setze_SFunktion(S);
		}
};

void FsSet::setze_Grenze(FTyp x1, FTyp x2) {

	// setzt die Adaptionsgrenzen der Fuzzy-Menge

	_minv=x1;
	_maxv=x2;

	if (_Anzahl_Intervalle>1) {
			_Intervall1.setze_Grenze(x1,x2);
			_Intervall2.setze_Grenze(x1,x2);
	}
	else
		_Intervall1.setze_Grenze(x1,x2);
}

void FsSet::skalieren (FTyp& Faktor) {

	// adaptiert die Fuzzy-Menge mit dem Faktor "Faktor"

	if (_Anzahl_Intervalle>1) {
		if (_verantwortliches_Intervall==1)
			_Intervall1.skalieren(Faktor);
		else
			_Intervall2.skalieren(Faktor);
	}
	else
		_Intervall1.skalieren(Faktor);
}

int FsSet::Anzahl_Intervalle() {
  return _Anzahl_Intervalle;
}
bool FsSet::steigend() {
	return _Intervall1.steigend();
}
int FsSet::Intervall_einfuegen(FTyp x1, FTyp x2, FTyp x3,FTyp min_x,FTyp max_x) {

	// definiert eine Fuzzy-Menge mit den drei Punkten x1,x2,x3
	// und den Adaptionsgrenzen min_x und max_x

  if (_Anzahl_Intervalle!=0)
		return 1;
  else {
		if(x1 < x3) {
			_Intervall1.setze_Intervall(x1,x2,0,0);
			_Intervall2.setze_Intervall(x3,x2,0,1);
		}
		else {
			_Intervall1.setze_Intervall(x3,x2,0,0);
			_Intervall2.setze_Intervall(x1,x2,0,1);
		}
		_Anzahl_Intervalle=2;
		_minv=_min_x=min_x;
		_maxv=_max_x=max_x;

		return 0;
  }
}

int FsSet::Intervall_einfuegen(FTyp x1,FTyp x2,FTyp min_x,FTyp max_x) {

	// definiert eine Fuzzy-Menge mit den beiden Punkten x1,x2
	// und den Adaptionsgrenzen min_x und max_x

  if (_Anzahl_Intervalle==2)
	 return 1;

  if (_Anzahl_Intervalle==0)
	 _Intervall1.setze_Intervall(x1,x2,0,0);

  else {

	 _Intervall2.setze_Intervall(x1,x2,0,1);
  }
  _Anzahl_Intervalle++;

  _minv=_min_x=min_x;
  _maxv=_max_x=max_x;

  return 0;
}
////////////////////////////////////////////////////////////////////////////////
bool FsSet::verschiebbar_mit_Restriktionen(FTyp& delta_x) {

	// ermittelt, ob die Fuzzy-Menge unter Einhaltung der Restriktionen
	// verschiebbar ist (Editor)

	if (_Anzahl_Intervalle>1) {
		return (_Intervall1.verschiebbar(delta_x,_minv,_maxv) &&
					_Intervall2.verschiebbar(delta_x,_minv,_maxv));
	}
	else
		return (_Intervall1.verschiebbar(delta_x,_minv,_maxv));
}


bool FsSet::verschiebbar(FTyp delta_x) {

	// ermittelt, ob die Fuzzy-Menge unter Einhaltung der Restriktionen
	// verschiebbar ist (Lernverfahren)

	if (_Anzahl_Intervalle>1) {
		return (_Intervall1.verschiebbar(delta_x,_min_x,_max_x) &&
					_Intervall2.verschiebbar(delta_x,_min_x,_max_x));
	}
	else
		return (_Intervall1.verschiebbar(delta_x,_min_x,_max_x));
}

bool FsSet::links_verschiebbar(FTyp delta_x) {

	// ermittelt, ob die linke Grenze der Fuzzy-Menge um "delta_x"
	// verschiebbar ist (Editor)

	return (_Intervall1.links_verschiebbar(delta_x,_min_x,_max_x));
}

bool FsSet::rechts_verschiebbar(FTyp delta_x) {

	// ermittelt, ob die rechte Grenze der Fuzzy-Menge um "delta_x"
	// verschiebbar ist (Editor)

	if (_Anzahl_Intervalle==2)
		return (_Intervall2.rechts_verschiebbar(delta_x,_min_x,_max_x));
	else
		return (_Intervall1.rechts_verschiebbar(delta_x,_min_x,_max_x));
}

void FsSet::verschieben(FTyp delta_x) {

	// verschiebt die Fuzzy-Menge um "delta_x"

	if (_Anzahl_Intervalle>1) {
		_Intervall1.verschieben(delta_x);
		_Intervall2.verschieben(delta_x);
	}
	else
		_Intervall1.verschieben(delta_x);
}

void FsSet::verschiebe_links(FTyp delta_x) {

	// verschiebt den linken Eckpunkt der Fuzzy-Menge um "delta_x"
	_Intervall1.verschiebe_links(delta_x);
}
void FsSet::verschiebe_rechts(FTyp delta_x) {

	// verschiebt den rechten Eckpunkt der Fuzzy-Menge um "delta_x"
	_Intervall1.verschiebe_links(delta_x);
	if (_Anzahl_Intervalle==2)
		_Intervall2.verschiebe_rechts(delta_x);
	else
		_Intervall1.verschiebe_rechts(delta_x);
}
////////////////////////////////////////////////////////////////////////////////
FTyp FsSet::x_wert(FTyp& y) const {

	// invertiert die Fuzzy-Menge,
	// nur Fuzzy-Mengen, die aus genau einem Intervall bestehen knnen
	// invertiert werden

	return _Intervall1.x_wert(y);
}
////////////////////////////////////////////////////////////////////////////////

// die folgenden Funktionen liefern die Eckpunkte der Fuzzy-Menge

FTyp FsSet::Mitte() {
	return x2();
}
FTyp FsSet::x1() {
	return _Intervall1.x1();
}
FTyp FsSet::x2() {
	return _Intervall1.x2();
}
FTyp FsSet::x3() {
	return _Intervall2.x1();
}
////////////////////////////////////////////////////////////////////////////////
bool FsSet::enthaelt_Teilmenge_von(FTyp x1,FTyp x2) {

	// ermittelt, ob eine berschneidung der Fuzzy-Menge
	// mit dem Intervall [x1,x2] existiert (Editor)

	if (_Anzahl_Intervalle>1) {
		return (_Intervall1.enthaelt_Teilmenge_von(x1,x2) ||
					 _Intervall2.enthaelt_Teilmenge_von(x1,x2));
	}
	else
		return (_Intervall1.enthaelt_Teilmenge_von(x1,x2));
}
void  FsSet::hole_Grenzen(FTyp& x1,FTyp& x2) {

	// liefert die den Bereich, auf dem (x) > 0

	if (Anzahl_Intervalle()==2) {
		x1=_Intervall1.x1();
		x2=_Intervall2.x2();
	}
	else {
		x1=_Intervall1.x1();
		x2=_Intervall1.x2();
	}

	if (x1 > x2) {
		FTyp temp(x2);
		x2=x1;
		x1=temp;
	}
}
////////////////////////////////////////////////////////////////////////////////
void FsSet::malen(TDC& dc, TRect& viereck,FTyp& min,FTyp& max, TColor Farbe) {

	// stellt die Fuzzy-Menge in einem Fenster dar

	if (_Anzahl_Intervalle!=0) {
		dc.SetTextAlign(TA_LEFT);

		if (_Anzahl_Intervalle>1)
			_Intervall2.malen(dc,0,viereck,min,max,Farbe,!_fuer_Regel_verfuegbar);
			_Intervall1.malen(dc,_Name,viereck,min,max,Farbe,!_fuer_Regel_verfuegbar);
	}
}

ostream& operator << (ostream& os, FsSet& einFsSet) {
	os << einFsSet._Name << " "
		<< einFsSet._Anzahl_Intervalle << endl
		<< einFsSet._Intervall1;

	if (einFsSet._Anzahl_Intervalle==2)
		os << einFsSet._Intervall2;

	os << einFsSet._min_x << " " << einFsSet._max_x << " "
		<< einFsSet._minv  << " " << einFsSet._maxv << endl;

	return os;
}
istream& operator >> (istream& is, FsSet& einFsSet) {
	is >> einFsSet._Name
		>> einFsSet._Anzahl_Intervalle
		>> einFsSet._Intervall1;

	if (einFsSet._Anzahl_Intervalle==2)
		is >> einFsSet._Intervall2;

	is >> einFsSet._min_x >> einFsSet._max_x
		>> einFsSet._minv >> einFsSet._maxv;

	return is;
}

