#include "fuzzydlg.h"

DEFINE_RESPONSE_TABLE1(FuzzyDialog, TCenterDialog)
	EV_CHILD_NOTIFY(IDCANCEL, BN_CLICKED, Abbruch),
	EV_CHILD_NOTIFY(IDOK, BN_CLICKED, Uebernehmen),
	EV_CHILD_NOTIFY(IDC_TYP, CBN_SELENDOK, neue_Auswahl),
END_RESPONSE_TABLE;

FuzzyDialog::FuzzyDialog(TWindow* parent,Variable* Var,FsSet* Set,bool istEingabeVar) :
	TCenterDialog (parent,DIALOG_FUZZY),
	_Var(Var),_Set(Set), _istEingabeVar(istEingabeVar) {

	_ok=false;

	// Edit-Fenster fr die Punkte allozieren und initialisieren
	p1= new TEdit(this,IDC_P1,32);
	p2= new TEdit(this,IDC_P2,32);
	p3= new TEdit(this,IDC_P3,32);

	g1= new TEdit(this,IDC_G1,32);
	g2= new TEdit(this,IDC_G2,32);

	Name= new TEdit(this,IDC_NAME,32);

	// Combobox fr den Typ der Fuzzy-Menge
	Typ= new TComboBox(this,IDC_TYP,32);
}

void FuzzyDialog::SetupWindow() {

	TCenterDialog::SetupWindow();
	_neu=*_Set;

	// Combobox in Abhngigkeit vom Variablentyp initialisieren
	Typ->AddString("Interval");
	Typ->AddString("S-Interval");

	if (_istEingabeVar) {
		Typ->AddString("Triangle");
		Typ->AddString("S-Triangle");
	}

	// Edit-Fenster mit dem Namen der Fuzzy-Menge initialisieren
	Name->SetText(_Set->Name().c_str());

	// Punkte in Edit-Fenster schreiben
	char puffer[30];
	sprintf(puffer,"%.2f",_Set->x1());
	p1->SetText(puffer);
	sprintf(puffer,"%.2f",_Set->x2());
	p2->SetText(puffer);
	sprintf(puffer,"%.2f",_Set->x3());
	p3->SetText(puffer);

	// Typ der Fuzzy-Menge in Combobox schreiben
	if (_Set->Anzahl_Intervalle()==2) {

		if (_Set->ist_SFunktion()) {
			Typ->SetText("S-Triangle");
		}
		else {
			Typ->SetText("Triangle");
		}


		sprintf(puffer,"%.2f",_Set->minv1());
		g1->SetText(puffer);
		sprintf(puffer,"%.2f",_Set->maxv2());
		g2->SetText(puffer);

		p3->Show(SW_SHOW);
		g2->Show(SW_SHOW);

	}

	else {
		if (_Set->ist_SFunktion()) {
			Typ->SetText("S-Interval");
		}
		else {
			Typ->SetText("Interval");
		}

		if (_Set->steigend())
			sprintf(puffer,"%.2f",_Set->minv1());
		else
			sprintf(puffer,"%.2f",_Set->maxv1());

		g1->SetText(puffer);

		sprintf(puffer,"%.2f",_Var->max_holen());
		g2->SetText(puffer);

		// fr einzelne Intervalle nicht ntige Punkte ausblenden
		p3->Show(SW_HIDE);
		g2->Show(SW_HIDE);


	}


}
void FuzzyDialog::neue_Auswahl() {

	// neuer Typ der Fuzzy-Menge aus der Combobox lesen
	char Wert[30];
	Typ->GetText((char *)Wert,30);
	mystring Eingabe(Wert);

	if ((Eingabe=="Interval") || (Eingabe=="S-Interval") ) {
		// falls jetzt Intervall, unntige Punkte (x3,g2) ausblenden
		p3->Show(SW_HIDE);
		g2->Show(SW_HIDE);
	}
	else {
		// sonst x3 und zweite Begrenzung g2 zeigen
		p3->Show(SW_SHOW);
		g2->Show(SW_SHOW);
	}
}

void FuzzyDialog::Abbruch() {
	CloseWindow();
}
void FuzzyDialog::Uebernehmen() {
	_ok=true;

	char puffer[30];

	// Name holen
	Name->GetText((char *)&puffer,30);
	istrstream is(puffer);
	mystring neu(puffer),neu2;
	is >> neu2;
	// besteht Name auch aus einem einzigen Wort ?
	if (neu2!=neu) {
		MessageBox("Names mustn't contain spaces.","Error");
		_ok=false;
	}
	// neue Fuzzy-Menge konstruieren, die mit den Werten aus der
	// Dialogbox im folgenden initialissiert wird

	FsSet* temp=new FsSet(neu);

	// Typ holen
	Typ->GetText((char *)&puffer,30);
	neu=puffer;

	FTyp x1,x2,x3,Var_min,Var_max,Grenze1,Grenze2;

	// Grenzen holen
	Var_min=_Var->min_holen();
	Var_max=_Var->max_holen();

	// x1,x2 aus der Dialogbox holen
	p1->GetText((char *)&puffer,30);
	x1=atof(puffer);
	p2->GetText((char *)&puffer,30);
	x2=atof(puffer);

	// Bereichsberprfung der Punkte x1,x2
	if (x1==x2) {
		MessageBox("x1 < x2 < x3 !","Error");
		delete temp;
		return;
	}
	if ((x1<Var_min) || (x1 > Var_max)) {
		sprintf(puffer,"x1 in [%.2f,%.2f] !",Var_min,Var_max);
		MessageBox(puffer,"Error");
		delete temp;
		return;
	}
	if ((x2<Var_min) || (x2 > Var_max)) {
		sprintf(puffer,"x2 in [%.2f,%.2f] !",Var_min,Var_max);
		MessageBox(puffer,"Error");
		delete temp;
		return;
	}

	// falls Definition sinnvoll, Intervall x1,x2 einfgen
	temp->Intervall_einfuegen(x1,x2,Var_min,Var_max);

	// erste Begrenzung g1 holen
	g1->GetText((char *)&puffer,30);
	Grenze1=atof(puffer);

	if ((neu=="Interval") || (neu=="S-Interval")) {

		// Typ der Fuzzy-Menge setzen
		temp->setze_SFunktion(neu=="S-Interval");

		// Grenze berprfen
		if (x1<=x2) {
			if (Grenze1>x1) {
				MessageBox("Boundary <= x1 !","Error");
				_ok=false;
			}
			else
				temp->setze_Grenze(Grenze1,x2);
		}
		else {
			if (Grenze1<x2) {
				MessageBox("Boundary >= x2 gelten !","Error");
				_ok=false;
			}
			else
				temp->setze_Grenze(x1,Grenze1);
	  }
	}
	else {
		// dritten Punkt x3 holen
		p3->GetText((char *)&puffer,30);
		x3=atof(puffer);

		// Bereichsberprfung x3
		if (!(x1 < x2 && x2 < x3)) {
			MessageBox("x1 < x2 < x3 !","Error");
			_ok=false;
		}
		if ((x3<Var_min) || (x3 > Var_max)) {
			sprintf(puffer,"x3 in [%.2f,%.2f] !",Var_min,Var_max);
			MessageBox(puffer,"Error");
			_ok=false;
		}
		if (_ok) {
			// Intervall einfgen
			temp->Intervall_einfuegen(x3,x2,Var_min,Var_max);
			// Typ setzen
			temp->setze_SFunktion(neu=="S-Triangle");

			// zweite Grenze holen
			g2->GetText((char *)&puffer,30);
			Grenze2=atof(puffer);
			// Bereichsberprfung
			if (!(Grenze1 <= x1 && x2 <= Grenze2)) {
				MessageBox("Boundary1 <= x1 and x3 <=  boundary2 !","Error");
				_ok=false;
			}
			else
				// Grenze setzen
				temp->setze_Grenze(Grenze1,Grenze2);
		}
	}
	// Fuzzy-Menge zuweisen
	_neu=*temp;

	delete temp;

	if (_ok)
		CloseWindow();
}

FuzzyDialog::~FuzzyDialog() {

	// allozierte Elemente wieder freigeben
	delete p1,p2,p3,
				g1,g2,
				Name,
				Typ;
}
