#include "metacntl.h"

ControllerParameter::ControllerParameter() {

	// Defaultwerte der Controllerparameter
	_Verfahren=fssab;
	_RegelLernVerfahren=Nauck;

	_Lernmodus= simulieren;
	_delta=2;
	_delta_plus=1.25;
	_delta_minus=0.5;

	_Epochen=500;
	_RegelnEpochen=500;
	_RegelnEpochen2=500;

	_MindestRegelAktivierung=0.01;

	_trace=false;
	_alte_Regeln_verwerfen=true;
};


ostream& operator << (ostream& os, ControllerParameter& Par) {

	os << Par._Verfahren << " ";

	os << Par._Lernmodus << " ";

	os << Par._delta << " ";

	os << Par._delta_plus << " ";

	os << Par._delta_minus << " ";

	os << Par._Epochen << " ";

	os << Par._RegelnEpochen << " ";

	os << Par._trace << endl;


	return os;

}

istream& operator >> (istream& is, ControllerParameter& Par) {

	int temp;


	is >> temp;

	Par._Verfahren=temp;


	is >> temp;

	Par._Lernmodus=temp;


	is >> Par._delta;

	is >> Par._delta_plus;

	is >> Par._delta_minus;

	is >> Par._Epochen;

	is >> Par._RegelnEpochen;

	is >> Par._trace;


	return is;

}


MetaController::MetaController() {
}

MetaController::~MetaController() {
	alle_Eingabevariable_freigeben();
	alle_Controller_freigeben();
}
// Controller-------------------------------------------------------------------
void MetaController::Controller_hinzufuegen() {

	// ein neuer Controller wird erstellt
	FController* temp = new FController();
	_ControllerListe.append(temp);
}

void MetaController::EingabeVar_hinzufuegen(mystring Name,FTyp& min,FTyp& max) {

	// eine neue Eingabevariable wird hinzugefgt
	EingabeVariable* temp = new EingabeVariable(Name, min, max);
	_EingabeVarListe.append(temp);
}

void MetaController::EingabeVarListen_generieren() {

	// alle Controller sollen ihre Liste der Eingabevariblen erstellen
	ControllerlisteIter Iter(_ControllerListe);
	while (Iter) {
		Iter.Current()->EingabeVarListe_generieren(_EingabeVarListe);
		Iter++;
	}
}
bool MetaController::existiert_EingabeVar(mystring& Name) {

	// ermittelt, ob eine Eingabevariable mit dem gegebenen Namen existiert
	TEingabeVarlisteIter Iter(_EingabeVarListe);

	while (Iter) {
		if (Iter.Current()->Name()==Name)
			return true;
		Iter++;
	}
	return false;
}

void MetaController::EingabeVar_umbenennen(mystring& alt,mystring& neu) {

	// weist einer Eingabevariablen einen neuen Namen zu

	TEingabeVarlisteIter Iter(_EingabeVarListe);

	while (Iter) {
		if (Iter.Current()->Name()==alt)
			// falls Variable gefunden, umbenennen
			Iter.Current()->setze_Name(neu);
		Iter++;
	}

	// allen Controllern wird die Umbennennung mitgeteilt, damit sie
	// ihre Liste der Eingabevariablen-Namen aktualisieren knnen

	ControllerlisteIter Iter2(_ControllerListe);
	while (Iter2) {
		Iter2.Current()->EingabeVar_umbenennen(alt,neu);
		Iter2++;
	}
	EingabeVarListen_generieren();
}

void MetaController::alle_Eingabevariable_freigeben() {

	// die dynmisch allozierten Eingabevariablen werden freigegeben
	TEingabeVarlisteIter Iter(_EingabeVarListe);

	while (Iter) {
		delete Iter.Current();
		Iter++;
	}
}
void MetaController::alle_Controller_freigeben() {

	// die dynmisch allozierten Controller werden freigegeben
	ControllerlisteIter Iter(_ControllerListe);

	while (Iter) {
		delete Iter.Current();
		Iter++;
	}
}

void MetaController::alle_Controller_auswerten() {

	// Es wird fr jeden Controller Stellwert und Fehler berechnet
	ControllerlisteIter Iter(_ControllerListe);

	while (Iter) {
		Iter.Current()->Stellwert_berechnen();
		Iter.Current()->Fehler_berechnen();
		Iter++;
	}
}
void MetaController::alle_Controller_nur_Fehler_auswerten() {

	// es wird fr jeden Controller der Fehler berechnet
	ControllerlisteIter Iter(_ControllerListe);
	while (Iter) {
		Iter.Current()->Fehler_berechnen();
		Iter++;
	}
}
void MetaController::alle_PraemissenContainer_auswerten(bool Phase_I) {

	// wertet fr jeden Controller den Praemissencontainer aus
	// wird fr das Regellernverfahren von Nauck verwendet

	ControllerlisteIter Iter(_ControllerListe);

	while (Iter) {
		Iter.Current()->Stellwert_des_PraemissenContainers_berechnen(Phase_I);
		Iter++;
	}
}

void MetaController::alle_Controller_adaptieren() {

	// alle Controller werden adaptiert:
	// es wird der Stellwert berechnet, der Fehler berechnet und adaptiert
	ControllerlisteIter Iter(_ControllerListe);

	while (Iter) {
		Iter.Current()->Stellwert_berechnen();
		Iter.Current()->Fehler_berechnen();
		Iter.Current()->adaptieren();
		Iter++;
	}
}
void MetaController::alle_Controller_delta_ruecksetzen() {

	// die Schrittweite  wird auf den vorgegebenen Wert zurckgesetzt
	ControllerlisteIter Iter(_ControllerListe);

	while (Iter) {
		Iter.Current()->delta_ruecksetzen();
		Iter++;
	}
}
bool MetaController::Regellernen_moeglich() {
	// berprft, ob ein Lernen von Kontrollregeln mglich ist, d.h.
	// ob fr jede Variable mindestens eine Fuzzy--Menge definiert ist

	bool ok=true;

	ControllerlisteIter Iter(_ControllerListe);
	while (Iter && ok) {
		if (Iter.Current()->hol_AusgabeVar().FuzzySets_holen()->GetItemsInContainer() == 0)
			ok=false;

		if (Iter.Current()->hol_FehlerVar().FuzzySets_holen()->GetItemsInContainer() == 0)
			ok=false;

		Iter++;
	}
	if (ok) {
		TEingabeVarlisteIter Iter(_EingabeVarListe);
		while (Iter && ok) {
			if (Iter.Current()->FuzzySets_holen()->GetItemsInContainer() == 0)
				ok=false;
			Iter++;
		}
	}
	return ok;
}
void MetaController::alle_Controller_Regeln_lernen_I() {

	// fr alle Controller wird die erste Phase des Regellernens durchgefhrt
	ControllerlisteIter Iter(_ControllerListe);
	FTyp Fehler;

	while (Iter) {
		Fehler=Iter.Current()->Fehler_berechnen();
		Iter.Current()->hol_SteuerRegelbasis().Regeln_lernen_I(Fehler);
		Iter++;
	}
}

void MetaController::alle_Controller_Regeln_lernen_II() {

	// fr alle Controller wird die zweite Phase des Regellernens durchgefhrt
	ControllerlisteIter Iter(_ControllerListe);
	FTyp Fehler;

	while (Iter) {
		Fehler=Iter.Current()->Fehler_berechnen();
		Iter.Current()->hol_SteuerRegelbasis().Regeln_lernen_II(Fehler);
		Iter++;
	}
}

void MetaController::alle_neuen_Regeln_uebernehmen(FTyp& Aktivierungsschwelle) {

	// die erlernten Regelbasen der Controller sollen bernommen werden
	ControllerlisteIter Iter(_ControllerListe);
	while (Iter) {
		Iter.Current()->hol_SteuerRegelbasis().neue_Regelbasis_uebernehmen(Aktivierungsschwelle);
		Iter++;
	}
}
//------------------------------------------------------------------------------

bool MetaController::Auswertung_vorbereiten() {

	// alle Controller werden fr die Auswertung vorbereitet und mit den
	// Parametern initialisiert

	bool ok=true;
	_Fehlermeldung="";

	ControllerlisteIter Iter(_ControllerListe);
	while (Iter && ok) {
		ok=Iter.Current()->Auswertung_vorbereiten();

		if (!ok)
			_Fehlermeldung=Iter.Current()->hol_Fehlermeldung();

		else {
			Iter.Current()->setze_Parameter(_Parameter);
			Iter++;
		}
	}
	return ok;
}

void MetaController::Lernen_vorbereiten() {

	// alle Controller werden fr die Lernverfahren vorbereitet

	ControllerlisteIter Iter(_ControllerListe);
	while (Iter) {
		Iter.Current()->Lernen_vorbereiten();
		Iter++;
	}
}
void MetaController::Regellernen_vorbereiten(bool alte_verwerfen) {

	// alle Controller werden fr die Anwendung der Regellernverfahren
	// vorbereitet.

	// verfgbare Fuzzy-Mengen werden markiert und bereits definierte
	// Kontrollregeln ggf. gelscht.

	ControllerlisteIter Iter(_ControllerListe);
	if (_Parameter._RegelLernVerfahren==Nauck) {

		while (Iter) {
			Iter.Current()->hol_FehlerRegelbasis().verfuegbare_FuzzySets_markieren();
			Iter.Current()->hol_SteuerRegelbasis().PraemissenContainer_erzeugen();
			if (alte_verwerfen)
				Iter.Current()->hol_SteuerRegelbasis().alle_Regeln_loeschen();

			Iter++;
		}
	}
	else {
		while (Iter) {
			Iter.Current()->hol_FehlerRegelbasis().verfuegbare_FuzzySets_markieren();
			if (alte_verwerfen)
				Iter.Current()->hol_SteuerRegelbasis().alle_Regeln_loeschen();

			Iter++;
		}
	}
}
void MetaController::Messwerte_protokollieren() {

	// Die aktuellen Mewerte der Eingabe-,Ausgabe- und Fehlervariablen
	// werden protokolliert

	TEingabeVarlisteIter Iter(_EingabeVarListe);

	while (Iter) {
		Iter.Current()->Messwert_protokollieren();
		Iter++;
	}

	ControllerlisteIter Iter2(_ControllerListe);
	while (Iter2) {
		Iter2.Current()->_FehlerVar.Messwert_protokollieren();
		Iter2.Current()->_AusgabeVar.Messwert_protokollieren();
		Iter2++;
	}
}
void MetaController::Messwerte_loeschen() {

	// alle bereits protokollierten Mewerte werden gelscht

	TEingabeVarlisteIter Iter(_EingabeVarListe);

	while (Iter) {
		Iter.Current()->Messwerte_loeschen();
		Iter++;
	}
	ControllerlisteIter Iter2(_ControllerListe);
	while (Iter2) {
		Iter2.Current()->_FehlerVar.Messwerte_loeschen();
		Iter2.Current()->_AusgabeVar.Messwerte_loeschen();
		Iter2++;
	}
}
void MetaController::generiere(int AnzahlEingabeVar,int AnzahlAusgabeVar) {

	// ein Metacontroller mit gegebener Anzahl von Eingabe- und Ausgabe-
	// variablen wird erstellt

	// alle bisher definierten Eingabevariablen verwerfen
	alle_Eingabevariable_freigeben();
	_EingabeVarListe.Flush();

	// alle bisher definierten Controller verwerfen
	alle_Controller_freigeben();
	_ControllerListe.Flush();


	// Eingabevariable und Controller erstellen
	char puffer[30];
	int i,j;

	for (i=0; i < AnzahlEingabeVar; ++i) {
		sprintf(puffer,"Input%d",i+1);
		EingabeVar_hinzufuegen(puffer,-100,100);
	}

	for (i=0; i < AnzahlAusgabeVar; ++i)
		Controller_hinzufuegen();

	// allen Controllern die Eingabevariablen zuweisen und deren Ausgabe- und
	// Fehlervariable initialisieren

	ControllerlisteIter Iter(_ControllerListe);

	j=1;
	while (Iter) {
		for (i=0; i< AnzahlEingabeVar; ++i) {
			sprintf(puffer,"Input%d",i+1);
			Iter.Current()->EingabeVar_hinzufuegen(puffer);
		}

		Iter.Current()->EingabeVarListe_generieren(_EingabeVarListe);

		sprintf(puffer,"Output%d",j);
		Iter.Current()->AusgabeVar_setzen(puffer,-100,100);
		sprintf(puffer,"Error%d",j++);
		Iter.Current()->FehlerVar_setzen(puffer,-1,1);
		Iter++;
	}
}
// IO---------------------------------------------------------------------------
ostream& operator << (ostream& os, MetaController& einController) {
	os << einController._Parameter << endl;
	os << einController._ControllerListe.GetItemsInContainer() << endl;

	ControllerlisteIter Iter(einController._ControllerListe);

	while (Iter) {
		os << *Iter.Current() << endl;
		Iter++;
	}

	TEingabeVarlisteIter Iter2(einController._EingabeVarListe);

	os << einController._EingabeVarListe.GetItemsInContainer() << endl;

	while (Iter2) {
		os << *Iter2.Current() ;
		Iter2++;
	}
	return os;
}

istream& operator >> (istream& is, MetaController& einController) {
	is >> einController._Parameter;
	int zaehler;
	is >> zaehler;

	einController.alle_Controller_freigeben();
	einController._ControllerListe.Flush();

	while (zaehler) {
		FController* temp = new FController();
		is >> *temp;
		einController._ControllerListe.append(temp);
		zaehler--;
	}

	is >> zaehler;

	einController.alle_Eingabevariable_freigeben();
	einController._EingabeVarListe.Flush();

	while (zaehler) {
		EingabeVariable* temp = new EingabeVariable();
		is >> *temp;
		einController._EingabeVarListe.append(temp);
		zaehler--;
	}
	einController.EingabeVarListen_generieren();
	einController.Regellernen_vorbereiten(false);

	return is;
}

void MetaController::Ausgabewerte_als_String(char* Puffer)
{
	// konstruiert einen String, der alle Stellwerte enthlt
	ostrstream A;
	ControllerlisteIter Iter(_ControllerListe);

	for (; Iter; Iter++) {
		A << Iter.Current()->Stellwert_holen() << " " ;
	}

	A << ends;
	strcpy(Puffer,A.str());

	delete A.str();
}

void MetaController::Eingabewerte_als_String(char* Puffer) {

	// entnimmt Puffer alle Eingabewerte und den Anwendungsstatus des
	// zu regelnden Prozesses

	istrstream E(Puffer);
	TEingabeVarlisteIter Iter(_EingabeVarListe);

	FTyp Wert;

	while (Iter) {
		E >> Wert;
		Iter.Current()->setze_Eingabewert(Wert);
		Iter++;
	}

	E >> _Anwendungsstatus;
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

FController::FController(): _EingabeVarListe(), _AusgabeVar(), _FehlerVar(),
  _SteuerRegelbasis(&_EingabeVarListe,&_AusgabeVar),
  _FehlerRegelbasis(&_EingabeVarListe,&_FehlerVar) {
}

void FController::EingabeVar_hinzufuegen(mystring& Name) {

	// hngt den Namen einer Eingabevariablen an die Liste an
	_EingabeVarNamen.append(Name);
}

EingabeVariable* FController::Variable_von_Name(TEingabeVarliste& Liste,
																mystring& Name) {

	// liefert zu dem Namen einer Eingabevariablen das korrespondierende
	// Objekt des Metacontrollers

	TEingabeVarlisteIter Iter(Liste);
	EingabeVariable* Ergebnis=0;
	bool gefunden=false;

	while (Iter && !gefunden) {
		if (Iter.Current()->Name()==Name) {
			gefunden=true;
			Ergebnis=Iter.Current();
		}
		else
			Iter++;
	}

	return Ergebnis;
}

void FController::EingabeVarListe_generieren(TEingabeVarliste& Liste) {

	// erstellt zu jedem Namen in der Liste der Eingabevariablennamen
	// ein Objektzeiger in der Liste der Eingabevariablen, d.h.
	// die Liste von Namen wird in eine Liste von Zeigern bersetzt

	TStringlisteIter Iter(_EingabeVarNamen);

	_EingabeVarListe.Flush();

	while (Iter) {
		_EingabeVarListe.append(Variable_von_Name(Liste,Iter.Current()));
		Iter++;
	}
}

void FController::EingabeVar_umbenennen(mystring& alt,mystring& neu) {

	// sorgt fr eine Umbennenung einer Variablen in der Liste der
	// Eingabevariablennamen

	TStringlisteIter Iter(_EingabeVarNamen);
	TStringliste neueListe;


	while (Iter) {
		if (Iter.Current()==alt)
			neueListe.append(neu);
		else
			neueListe.append(Iter.Current());

		Iter++;
	}
	_EingabeVarNamen=neueListe;
}

void FController::AusgabeVar_setzen(mystring Name, FTyp& min, FTyp& max) {

	// setzt die Ausgabevariable dieses Controllers
  AusgabeVariable temp(Name, min, max);
  _AusgabeVar=temp;
}

void FController::FehlerVar_setzen(mystring Name, FTyp& min, FTyp& max) {

	// setzt die Fehlervariable dieses Controllers
  AusgabeVariable temp(Name, min, max);
  _FehlerVar=temp;
}
const mystring& FController::AusgabeVar_holen() {
	// liefert die Ausgabevariable dieses Controllers
	return _AusgabeVar.Name();
}
const mystring& FController::FehlerVar_holen() {

	// liefert die Fehlervariable dieses Controllers
	return _FehlerVar.Name();
}

void FController::FuzzySet_setze_Grenze(mystring& VarName,mystring& Name,
										FTyp x1,FTyp x2) {

	// setzt die Grenzen fr die Adaption der Fuzzy-Menge "Name"
	// der Variable "VarName"

	if (VarName==_AusgabeVar.Name())
		_AusgabeVar.FuzzySet_setze_Grenze(Name,x1,x2);
	else {
		if (VarName==_FehlerVar.Name())
			_FehlerVar.FuzzySet_setze_Grenze(Name,x1,x2);

		else {

			// in der Liste der Eingabevariablen die Variable suchen
			TEingabeVarlisteIter Iter(_EingabeVarListe);

			bool searching(true);

			while (Iter && searching) {

				searching=(Iter.Current()->Name()!=VarName);

				if (searching)
					Iter++;
			}
			if (!searching)
				(Iter.Current())->FuzzySet_setze_Grenze(Name,x1,x2);
		}
	}
}
void FController::FuzzySet_hinzufuegen(mystring& VarName,mystring& Name,
													FTyp x1,FTyp x2) {

	// fgt der Variablen "VarBame" eine neues Fuzzy-Intervall hinzu
	FuzzySet_hinzufuegen(VarName,Name,x1,x2,0,0);
}
void FController::FuzzySet_hinzufuegen(mystring& VarName,mystring& Name,
													FTyp x1,FTyp x2, FTyp x3) {

	// fgt der Variablen "VarnName" eine neue Fuzzy-Menge hinzu
	FuzzySet_hinzufuegen(VarName,Name,x1,x2,x3,0,0);
}
void FController::FuzzySet_hinzufuegen(mystring& VarName,mystring& Name,
													FTyp x1,FTyp x2,
													FTyp min_x, FTyp max_x) {

	// fgt der Variablen "VarnName" eine neue Fuzzy-Menge hinzu
	// sucht dazu nach der Variablen mit diesem Namen

  if (VarName==_AusgabeVar.Name())
	 _AusgabeVar.FuzzySet_einfuegen(Name,x1,x2,min_x,max_x);
  else
	 if (VarName==_FehlerVar.Name())
		_FehlerVar.FuzzySet_einfuegen(Name,x1,x2,min_x,max_x);

	 else {

		// Eingabevariablen durchsuchen

		TEingabeVarlisteIter Iter(_EingabeVarListe);

		bool searching(true);

		while (Iter && searching) {

			searching=(Iter.Current()->Name()!=VarName);

			if (searching)
				Iter++;
		}
		if (!searching)
			(Iter.Current())->FuzzySet_einfuegen(Name,x1,x2,min_x,max_x);
	 }
}
bool FController::Auswertung_vorbereiten() {

	// bereitet die Fehler- und Steuerregelbasis fr die Auswertung vor

	bool ok=true;
	_Fehlermeldung="";

	ok=_SteuerRegelbasis.Auswertung_vorbereiten();

	if (!ok) {
		_Fehlermeldung=_SteuerRegelbasis.hol_Fehlermeldung();
	}
	else {
		ok=_FehlerRegelbasis.Auswertung_vorbereiten();
		if (!ok) {
			_Fehlermeldung=_FehlerRegelbasis.hol_Fehlermeldung();
		}
	}
	return ok;

}
void FController::Lernen_vorbereiten() {

		// initialisert die Steuerregelbasis mit den aktuellen
		// Lernparametern

		_SteuerRegelbasis.Lernen_vorbereiten(_Parameter._Verfahren,
														_Parameter._RegelLernVerfahren,
														_Parameter._Epochen,
														_Parameter._delta,
														_Parameter._delta_plus,
														_Parameter._delta_minus);
}
void FController::delta_ruecksetzen() {

	// setzt die Schrittweite zurck, falls z.B. die Regelung versagte
	_SteuerRegelbasis.delta_ruecksetzen();
}
void FController::FuzzySet_hinzufuegen(mystring& VarName,mystring& Name,
													FTyp x1, FTyp x2, FTyp x3,
													FTyp min_x, FTyp max_x) {

	// fgt eine Fuzzy-Menge ein

	if (VarName==_AusgabeVar.Name())
		_AusgabeVar.FuzzySet_einfuegen(Name,x1,x2,x3,min_x,max_x);
	else {
		TEingabeVarlisteIter Iter(_EingabeVarListe);

		bool searching(true);
		while (Iter && searching) {

			searching=(Iter.Current()->Name()!=VarName);

			if (searching)
				Iter++;
		}

		if (!searching)
			(Iter.Current())->FuzzySet_einfuegen(Name,x1,x2,x3,min_x,max_x);
	}
}
void FController::SteuerRegel_hinzufuegen(list<mystring>& Praemisse,mystring& Konklusion) {

	// eine Steuerregel wird dem Controller hinzugefgt,
	// dazu werden die Stringlisten kopiert

	Regel temp(&_EingabeVarListe,&_AusgabeVar);

	listIter<mystring> Iter(Praemisse);

	while(Iter) {
		temp.Praemisse_hinzufuegen(Iter.Current());
		Iter++;
	}

	temp.Konklusion_setzen(Konklusion);

  _SteuerRegelbasis.Regel_hinzufuegen(temp);
}
void FController::FehlerRegel_hinzufuegen(list<mystring>& Praemisse,mystring& Konklusion) {

	// eine Fehlerregel wird dem Controller hinzugefgt,
	// s.o.

  Regel temp(&_EingabeVarListe,&_FehlerVar);
  listIter<mystring> Iter(Praemisse);


  while(Iter) {
	temp.Praemisse_hinzufuegen(Iter.Current());
	Iter++;
  }

  temp.Konklusion_setzen(Konklusion);

  _FehlerRegelbasis.Regel_hinzufuegen(temp);
}

void FController::EingabeVar_setzeWert(mystring& VarName,FTyp& Wert) {

	// setzt die Eingabevariable mit dem Namen "VarName" auf den Wert "Wert"
	// sucht dazu das korrespondierende Objekt in der Liste der Eingabevar.

	TEingabeVarlisteIter Iter(_EingabeVarListe);

	bool searching(true);
	while (Iter && searching) {

		searching=(Iter.Current()->Name()!=VarName);

		if (searching)
			Iter++;
	}

	if (!searching)
		(Iter.Current())->setze_Eingabewert(Wert);
}
EingabeVariable* FController::EingabeVar_holen(mystring& VarName) {

	// liefert einen Zeiger auf das Objekt der Eingabevariablen mit dem
	// Namen "VarName"

	TEingabeVarlisteIter Iter(_EingabeVarListe);

	bool searching(true);
	while (Iter && searching) {

		searching=(Iter.Current()->Name()!=VarName);

		if (searching)
			Iter++;
	}

	if (!searching)
		return Iter.Current();
	else
		return 0;
}
void FController::Stellwert_berechnen() {

	// berechnet den Stellwert der Steuerregelbasis
	_Stellwert =  _SteuerRegelbasis.Regeln_auswerten();
}
void FController::Stellwert_des_PraemissenContainers_berechnen(bool Phase_I) {

	// berechnet den Stellwert des Praemissencontainers
	_Stellwert =
		_SteuerRegelbasis.Stellwert_des_PraemissenContainers_berechnen(Phase_I);
}
FTyp& FController::Fehler_berechnen() {

	// berechnet den Fehler
	_Fehler =  _FehlerRegelbasis.Regeln_auswerten();
	return _Fehler;
}
FController::~FController() {
	// die Freigabe der allozierten Objekte erfolgt durch den Metacontroller
}

void FController::adaptieren() {

	// adaptiert den Controller durch die Vernderung der Breite der
	// Fuzzy-Mengen

  _SteuerRegelbasis.adaptieren(_Fehler);
}
ostream& operator << (ostream& os, FController& einController) {

	os << einController._AusgabeVar ;
	os << einController._FehlerVar ;

	TStringlisteIter Iter(einController._EingabeVarNamen);

	os << einController._EingabeVarNamen.GetItemsInContainer() << endl;

	while (Iter) {
		os << Iter.Current() << " ";
		Iter++;
	}

	os << einController._SteuerRegelbasis << einController._FehlerRegelbasis;
	return os;
}
istream& operator >> (istream& is, FController& einController) {

	is >> einController._AusgabeVar ;
	is >> einController._FehlerVar ;

	int zaehler;
	is >> zaehler;

	einController._EingabeVarListe.Flush();
	einController._EingabeVarNamen.Flush();

	while (zaehler) {
		mystring temp;
		is >> temp;
		einController._EingabeVarNamen.append(temp);
		zaehler--;
	}

	is >> einController._SteuerRegelbasis >> einController._FehlerRegelbasis;

	return is;
}
//################# D D E - K L I E N T ########################################
	MetaController* DDEClient::_Meta;

	char DDEClient::SendePuffer[80], DDEClient::EmpfangsPuffer[80];
	HDDEDATA       DDEClient::hData;
	DWORD          DDEClient::dwResult;
	WORD           DDEClient::wFmt = CF_TEXT;
	HINSTANCE DDEClient::hInst;
	FARPROC DDEClient::lpDdeProc;
	DWORD DDEClient::InstId;
	DWORD DDEClient::aCbFilter;

	HSZ DDEClient::hszService,DDEClient::hszTopic,DDEClient::hszItem;
	HCONV DDEClient::hConv;

	HSZ DDEClient::hszInitialisieren,
		DDEClient::hszEingabe, DDEClient::hszAusgabe,
		DDEClient::hszIdentifizieren;

DDEClient::DDEClient(MetaController* Meta,HINSTANCE hInstance) {

	_Meta=Meta;

	// initialisiert die DDE-Verbindung
	InstId=0;
	hInst=hInstance;
	hConv	= (HCONV)NULL;
	wFmt = CF_TEXT;
	lpDdeProc = MakeProcInstance((FARPROC)&DDEClient::DDECallback,hInst);
	aCbFilter = 0;
	DdeInitialize((LPDWORD)&InstId,(PFNCALLBACK)lpDdeProc,APPCMD_CLIENTONLY,0L);

	// die ntigen Stringhandles erstellen
	hszService = 	DdeCreateStringHandle(InstId,"NEFCON-WIN",CP_WINANSI);
	hszTopic = 		DdeCreateStringHandle(InstId,"DATA-TRANSFER",CP_WINANSI);
	hszItem = 		DdeCreateStringHandle (InstId, "Data", CP_WINANSI );

	hszInitialisieren = DdeCreateStringHandle (InstId, "Init", CP_WINANSI );
	hszEingabe = DdeCreateStringHandle (InstId, "I", CP_WINANSI );
	hszAusgabe = DdeCreateStringHandle (InstId, "O", CP_WINANSI );
	hszIdentifizieren = DdeCreateStringHandle (InstId, "Ident", CP_WINANSI );
}
DDEClient::~DDEClient() {

	// Verbindung abbauen, falls sie noch besteht
	if ( hConv != (HCONV)NULL ) {
		DdeDisconnect ( hConv );
		hConv = (HCONV)NULL;
	}

	FreeProcInstance ( lpDdeProc );

	// Stringhandles wieder lschen
	DdeFreeStringHandle ( InstId, hszService );
	DdeFreeStringHandle ( InstId, hszTopic );
	DdeFreeStringHandle ( InstId, hszItem );

	DdeFreeStringHandle ( InstId, hszInitialisieren );
	DdeFreeStringHandle ( InstId, hszEingabe );
	DdeFreeStringHandle ( InstId, hszAusgabe );
	DdeFreeStringHandle ( InstId, hszIdentifizieren );

	// Server abmelden
	DdeUninitialize(InstId);
}
HDDEDATA EXPENTRY DDEClient::DDECallback ( WORD wType, WORD wFmt, HCONV hConvX, HSZ hsz1,
			  HSZ hsz2, HDDEDATA hData, DWORD dwData1, DWORD dwData2 ) {

	// callback-Funktion des Klienten

	switch ( wType ) {
		case XTYP_DISCONNECT:
			// Verbindung wurde aufgelst
			hConv = (HCONV)NULL;
			return ( (HDDEDATA) NULL );

		case XTYP_ERROR:
			// ein Fehler ist aufgetreten
			return ( (HDDEDATA) NULL );

		case XTYP_XACT_COMPLETE:
			// Transaktion komplettiert
			break;
	}
}
bool DDEClient::Anwendung_initialisieren() {

	// fordert beim Prozeserver die Initialisierung des Prozesse an
	if ( hConv != (HCONV)NULL ) {

		hData = DdeClientTransaction ( NULL, 0, hConv,
											hszInitialisieren, wFmt, XTYP_REQUEST, 3000, &dwResult );

		// trat Fehler auf ?
		if (DdeGetLastError(InstId)!=DMLERR_NO_ERROR)
			return false;
		else {
			// Data-Handle auslesen und freigeben
			DdeGetData ( hData, (LPBYTE) EmpfangsPuffer, 80L, 0L );
			DdeFreeDataHandle(hData);

			return true;
		}
	}
	else
		return false;
}
bool DDEClient::Anwendung_identifizieren() {

	// fordert die Identifikation des Prozeservers an

	if ( hConv != (HCONV)NULL ) {


		hData = DdeClientTransaction ( NULL, 0, hConv,
											hszIdentifizieren, wFmt, XTYP_REQUEST, 3000, &dwResult );

		// trat Fehler auf ?
		if (DdeGetLastError(InstId)!=DMLERR_NO_ERROR)
			return false;
		else {
			// Data-Handle auslesen und freigeben
			DdeGetData ( hData, (LPBYTE) EmpfangsPuffer, 80L, 0L );
			DdeFreeDataHandle(hData);
			return true;
		}
	}
	else
		return false;
}
bool DDEClient::Eingabewerte_holen() {
	// fordert neue Eingabewerte vom Prozeserver an und bermittelt sie
	// dem Metacontroller

	if ( hConv != (HCONV)NULL ) {

		hData = DdeClientTransaction ( NULL, 0, hConv,
											hszEingabe, wFmt, XTYP_REQUEST, 3000, &dwResult );

		// trat Fehler auf ?
		if (DdeGetLastError(InstId)!=DMLERR_NO_ERROR)
			return false;
		else {
			// Data-Handle auslesen und freigeben
			DdeGetData ( hData, (LPBYTE) EmpfangsPuffer, 80L, 0L );
			DdeFreeDataHandle(hData);
			// Eingabewerte im Empfangspuffer dem Metacontroller bermitteln
			_Meta->Eingabewerte_als_String(EmpfangsPuffer);

			return true;
		}
	}
	else
		return false;
}
bool DDEClient::Stellwerte_senden() {

	// sendet die Stellwerte des Metacontrollers an den Prozeserver

	if ( hConv != (HCONV)NULL ) {

		// Ausgabewerte vom Metacontroller als String besorgen
		_Meta->Ausgabewerte_als_String(SendePuffer);

		// Sonderfall Win32

		#if defined(__WIN32__)
			hData = DdeCreateDataHandle ( InstId, (LPBYTE) SendePuffer,
		#else
			hData = DdeCreateDataHandle ( InstId, SendePuffer,
		#endif
				sizeof ( SendePuffer ), 0L, hszItem, wFmt, 0 );

		// bertragung vornehmen
		if ( hData != (HDDEDATA)NULL ) {
			hData = DdeClientTransaction ( (LPBYTE)hData, -1, hConv,
						hszAusgabe, wFmt, XTYP_POKE, 1000, &dwResult );
			return true;
		}
	}
	return false;
}
bool DDEClient::DDE_init() {

	// initialisiert die DDE-Verbindung

	#if defined(__WIN32__)
		hData = DdeCreateDataHandle ( InstId, (LPBYTE) SendePuffer,
	#else
		hData = DdeCreateDataHandle ( InstId, SendePuffer,
	#endif
		sizeof ( SendePuffer ), 0L, hszItem, wFmt, 0 );

	if ( hData != (HDDEDATA)NULL )
		hData = DdeClientTransaction ( (LPBYTE)hData, -1, hConv,
					hszItem, wFmt, XTYP_POKE, 1000, &dwResult );

	return true;
}
bool DDEClient::Connect() {

	// erstellt eine DDE-Verbindung mit dem Prozeserver
	hConv = DdeConnect ( InstId, hszService, hszTopic,
					(PCONVCONTEXT) NULL );
	// Fehler ?
	if (!hConv) {
		return false;
	}

	return true;
}
bool DDEClient::Unconnect() {

	// baut die Verbindung mit dem Prozeserver ab, falls sie noch besteht

	if ( hConv != (HCONV)NULL ) {
		DdeDisconnect ( hConv );
		hConv = (HCONV)NULL;
	}
	return true;
}

