#ifndef _METACONTROLLER_
#define _METACONTROLLER_

#include "regelbas.h"
#include <stdio.h>

class MetaController;
class FController;

typedef  list<FController*> Controllerliste ;
typedef  listIter<FController*> ControllerlisteIter ;

enum Lernmodus {simulieren=0, Fuzzy_adaptieren, Regeln_lernen};

class ControllerParameter {

	public:

		ControllerParameter();

		Lernmodus _Lernmodus;
		Verfahren _Verfahren;
		RegelLernVerfahren _RegelLernVerfahren;

		FTyp _delta,_delta_plus,_delta_minus;
		long int _Epochen, _RegelnEpochen, _RegelnEpochen2;
		FTyp _MindestRegelAktivierung;

		bool _trace;
		bool _alte_Regeln_verwerfen;

		friend ostream& operator << (ostream& os, ControllerParameter& Par);
		friend istream& operator >> (istream& is, ControllerParameter& Par);

};


class MetaController {

	Controllerliste _ControllerListe;
	TEingabeVarliste _EingabeVarListe;

	ControllerParameter _Parameter;
	mystring _Fehlermeldung;

	public:

		MetaController();
		~MetaController();

		void generiere(int AnzahlEingabeVar,int AnzahlAusgabeVar);

		void Controller_hinzufuegen();
		Controllerliste& ControllerListe_holen(){return _ControllerListe;};

		void EingabeVar_hinzufuegen(mystring Name, FTyp& min, FTyp& max);

		void alle_Eingabevariable_freigeben();
		void alle_Controller_freigeben();

		void EingabeVarListen_generieren();
		TEingabeVarliste& EingabeVarListe_holen(){return _EingabeVarListe;};

		bool existiert_EingabeVar(mystring& Name);
		void EingabeVar_umbenennen(mystring& alt,mystring& neu);

		void setze_Parameter(ControllerParameter& Parameter) {_Parameter=Parameter;};
		ControllerParameter& Parameter_holen() { return _Parameter;};

		int Anzahl_Controller() { return _ControllerListe.GetItemsInContainer();};
		int Anzahl_EingabeVariable() { return _EingabeVarListe.GetItemsInContainer();};

		bool Auswertung_vorbereiten();
		mystring& hol_Fehlermeldung() {return _Fehlermeldung;};

		void Lernen_vorbereiten();

		void Regellernen_vorbereiten(bool alte_verwerfen);
		bool Regellernen_moeglich();

		void alle_Controller_auswerten();
		void alle_Controller_nur_Fehler_auswerten();
		void alle_Controller_adaptieren();
		void alle_Controller_delta_ruecksetzen();

		void alle_PraemissenContainer_auswerten(bool Phase_I);
		void alle_Controller_Regeln_lernen_I();
		void alle_Controller_Regeln_lernen_II();

		void alle_neuen_Regeln_uebernehmen(FTyp& Aktivierungsschwelle);

		void Messwerte_protokollieren();
		void Messwerte_loeschen();

		friend ostream& operator << (ostream& os, MetaController& einController);
		friend istream& operator >> (istream& is, MetaController& einController);

		void Ausgabewerte_als_String(char* Puffer);
		void Eingabewerte_als_String(char* Puffer);

		int _Anwendungsstatus;
};
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
class FController {

	FTyp _Fehler,_Stellwert;
	ControllerParameter _Parameter;

	Regelbasis _SteuerRegelbasis,_FehlerRegelbasis;

	EingabeVariable* Variable_von_Name(TEingabeVarliste& Liste,
													mystring& Name);
	TStringliste _EingabeVarNamen;
	mystring _Fehlermeldung;

public:

	TEingabeVarliste _EingabeVarListe;
	AusgabeVariable _AusgabeVar,_FehlerVar;

	AusgabeVariable& hol_AusgabeVar() { return _AusgabeVar;};
	AusgabeVariable& hol_FehlerVar() { return _FehlerVar;};

	FController();
	~FController();

	void setze_Parameter(ControllerParameter& Parameter) {_Parameter=Parameter;};

	Regelbasis& hol_SteuerRegelbasis() { return _SteuerRegelbasis;};
	Regelbasis& hol_FehlerRegelbasis() { return _FehlerRegelbasis;};

	FTyp& Stellwert_holen() { return _Stellwert;};

	void EingabeVar_hinzufuegen(mystring& Name);
	void EingabeVarListe_generieren(	TEingabeVarliste& Liste);
	TStringliste& EingabeVarNamen_holen() { return _EingabeVarNamen;};
	void EingabeVar_umbenennen(mystring& alt,mystring& neu);
	TEingabeVarliste& EingabeVarListe_holen() { return _EingabeVarListe ;};

	EingabeVariable* EingabeVar_holen(mystring& VarName);
	void EingabeVar_setzeWert(mystring& VarName,FTyp& Wert);


	void AusgabeVar_setzen(mystring Name, FTyp& min, FTyp& max);
	void FehlerVar_setzen(mystring Name, FTyp& min, FTyp& max);

	const mystring& AusgabeVar_holen();
	const mystring& FehlerVar_holen();

	void FuzzySet_hinzufuegen(mystring& VarName,mystring& Name,
									FTyp x1,FTyp x2, FTyp x3);

	void FuzzySet_setze_Grenze(mystring& VarName,mystring& Name,
										FTyp x1,FTyp x2=0);

	void FuzzySet_hinzufuegen(mystring& VarName,mystring& Name,
									FTyp x1,FTyp x2);

	void FuzzySet_hinzufuegen(mystring& VarName,mystring& Name,
									FTyp x1,FTyp x2,
									FTyp min_x, FTyp max_x);
	void FuzzySet_hinzufuegen(mystring& VarName,mystring& Name,
									FTyp x1,FTyp x2, FTyp x3,
									FTyp min_x, FTyp max_x);

	void SteuerRegel_hinzufuegen(list<mystring>& Praemisse, mystring& Konklusion);
	void FehlerRegel_hinzufuegen(list<mystring>& Praemisse, mystring& Konklusion);

	bool Auswertung_vorbereiten();
	mystring& hol_Fehlermeldung() {return _Fehlermeldung;};

	void Lernen_vorbereiten();
	void delta_ruecksetzen();


	void Stellwert_berechnen();
	void Stellwert_des_PraemissenContainers_berechnen(bool Phase_I);
	FTyp& Fehler_berechnen();

	void adaptieren();

	friend ostream& operator << (ostream& os, FController& einController);
	friend istream& operator >> (istream& is, FController& einController);

	};

	#include <owl/window.h>
	#include <ddeml.h>
	#include <dde.h>

	class DDEClient {

		static MetaController* _Meta;
		static ofstream of;
		static char SendePuffer[80], EmpfangsPuffer[80];
		static HDDEDATA  hData;
		static DWORD     dwResult;
		static WORD      wFmt;
		static HINSTANCE hInst;
		static FARPROC lpDdeProc;
		static DWORD InstId;
		static DWORD aCbFilter;

		static HSZ hszService,hszTopic,hszItem;
		static HCONV hConv;

		static HSZ hszInitialisieren,
						hszEingabe, hszAusgabe,
						hszIdentifizieren;

		public:

			DDEClient(MetaController* Meta, HINSTANCE hInstance);
			~DDEClient();

			static char* Ergebnis() { return EmpfangsPuffer;};

			static  HDDEDATA EXPENTRY DDECallback
					( WORD wType, WORD wFmt, HCONV hConvX, HSZ hsz1, HSZ hsz2,
						HDDEDATA hData, DWORD dwData1, DWORD dwData2 );

			static bool DDE_init();

			static bool Connect();
			static bool Unconnect();

			static bool Anwendung_initialisieren();
			static bool Anwendung_identifizieren();

			static bool Eingabewerte_holen();
			static bool Stellwerte_senden();
	};
#endif
