#ifndef _MYMDIC_
#define _MYMDIC_

#include "stdlib.h" // fr max()
#include "global.h"
#include "metacntl.h"

#include <owl/mdi.h>
#include <owl/opensave.h>

#include "cntrdlg.h"

#include "aboutdlg.h"
#include "abortdlg.h"
#include "contldlg.h"
#include "vardlg.h"
#include "neudlg.h"

// MDI-Children:
#include "surface.h"
#include "activity.h"
#include "maxact.h"

#include "setedit.h"
#include "regeledt.h"
#include "graphchi.h"
#include "setedit.h"
#include "3dmess.h"

#define FARBEN_PRO_SCHEMA 25

#define Schemata 7
#define ANZAHL_PALETTENFARBEN FARBEN_PRO_SCHEMA * Schemata


typedef  list<GraphMDIChild*> Fensterliste ;
typedef  listIter<GraphMDIChild*> FensterlisteIter ;

typedef  list<TSetEdit*> SetEditliste ;
typedef  listIter<TSetEdit*> SetEditlisteIter ;

typedef  list<TRegelEdit*> RegelEditliste ;
typedef  listIter<TRegelEdit*> RegelEditlisteIter ;

class TControlChild;

class TMyMDIClient : public TMDIClient {


	FTyp FaktorRot[Schemata],FaktorGruen[Schemata],FaktorBlau[Schemata];
	FTyp BiasRot[Schemata],BiasGruen[Schemata],BiasBlau[Schemata];
	LOGPALETTE* _palette;
	TPalette* _Palette_fuer_alle;

	Fensterliste _alleGraphen;
	SetEditliste _alleSetEdits;

	RegelEditliste _alleRegelEdits;

	TWindow* _EditController;

	HINSTANCE _hInstance;
	DDEClient IOManager;

	public:
		TMyMDIClient(HINSTANCE hInstance,
							MetaController* Meta);

		void neues_Messwerte_Fenster(MesswerteListe* Liste,mystring& titel);
		void neues_SetEdit_Fenster(Variable* Var,
											bool istAusgabeVar,
											bool istFehlerVar);
		void neues_Regelbasis_Fenster(Regelbasis& Basis,mystring& titel);
		void neues_Kennfeld_Fenster(FController* Controller,bool Fehler=false);
		void neues_Regelaktivitaeten_Fenster(FController* Controller, bool Fehler);
		void neues_MaxRegelaktivitaeten_Fenster(FController* Controller, bool Fehler);

		void neues_3DMess_Fenster(FController* Controller,bool Fehler=false);

		void alleGraphenupdaten();
		void alleSetEditsupdaten();
		void alleRegelEditsupdaten();

		bool _Abbruch;
		void schliesseController();

		~TMyMDIClient();


	protected:
		bool Verbindung_aufbauen();

		void SetupWindow();
		bool EvEraseBkgnd(HDC hDC);
		bool CanClose();

		void CmEditController();

		void CmFileOpen();
		void CmFileSaveAs();
		void CmAbout();
		void CmNeu();

		void CmTileChildrenHoriz();
		void CmTileChildrenVert();
		void CmCascadeChildren();

		void CmStarteController();

		void schliesseGraph(GraphMDIChild* Graph);
		void schliesseSetedit(TSetEdit* Editor);
		void schliesseRegeledit(TRegelEdit* Editor);

		void EvSetFocus(HWND);

		MetaController* _Meta;

		TBitmap* _root;

		LRESULT Mailbox(WPARAM, LPARAM);
		DECLARE_RESPONSE_TABLE(TMyMDIClient);
};


#include <owl/mdichild.h>
#include "main.rc"

#define CM_PMESSWERTE 			5000
#define CM_PFUZZY 				5001
#define CM_PSTEUERREGELBASIS 	5002
#define CM_PFEHLERREGELBASIS 	5003
#define CM_PKENNFELD_STEUER	5004
#define CM_PKENNFELD_FEHLER	5005
#define CM_PMESS3D_STEUER		5006
#define CM_PMESS3D_FEHLER		5007
#define CM_PVAR 					5008
#define CM_PCCLIPBOARD 			5009
#define CM_PAKTIVITAETEN_STEUER	5010
#define CM_PAKTIVITAETEN_FEHLER	5011
#define CM_PMAXAKTIVITAETEN_STEUER	5012
#define CM_PMAXAKTIVITAETEN_FEHLER	5013

enum SelTyp {kein=0, EingabeVar, Controller, AusgabeVar, FehlerVar};

class TControlChild : public TMDIChild{

	TMyMDIClient& _Parent;
	MetaController* _Meta;

	TBitmap *_VarBitmap,*_ControllerBitmap;
	TFont* _Schrift;

	int xpos_EingabeVar[MAX_EINGABEVARS];
	int ypos_EingabeVar;

	int xpos_Controller[MAX_CONTROLLER];
	int ypos_Controller;

	int xpos_AusgabeVar[MAX_CONTROLLER];
	int xpos_FehlerVar[MAX_CONTROLLER];
	int ypos_Ausgabe;

	int dx_Ausgabe;

	SelTyp Punkt_checken(TPoint& point);

	bool innerhalb_Y_Bereich(TPoint& point, int ypos);
	bool innerhalb_X_Bereich(TPoint& point, int xpos);
	bool innerhalb(TPoint& point, int xpos, int ypos);

	EingabeVariable* EingabeVariable_holen(TPoint& point);
	FController* Controller_holen(TPoint& point);
	AusgabeVariable* AusgabeVariable_holen(TPoint& point);
	AusgabeVariable* FehlerVariable_holen(TPoint& point);

	EingabeVariable* selektierte_EingabeVariable;
	AusgabeVariable* selektierte_AusgabeVariable, *selektierte_FehlerVariable;
	FController* selektierter_Controller;

	void Pfeil(TDC& dc,FTyp& x1,FTyp& y1,FTyp& x2,FTyp& y2);

	public:
		 TControlChild(TMyMDIClient& parent,MetaController* Meta);
		 ~TControlChild();

		 void SetupWindow();
		 void Paint(TDC&, bool, TRect& );
		 void EvSize(UINT sizeType, TSize &size);

		 void CmPMesswerte();
		 void CmPFuzzy();
		 void CmPVar();

		 void CmPSteuerRegelbasis();
		 void CmPFehlerRegelbasis();

		 void CmPKennfeldSteuer();
		 void CmPKennfeldFehler();
		 void CmPAktivitaetenSteuer();
		 void CmPAktivitaetenFehler();
		 void CmPMaxAktivitaetenSteuer();
		 void CmPMaxAktivitaetenFehler();

		 void CmPMess3DSteuer();
		 void CmPMess3DFehler();
		 void CmPCClipboard();

		 void EvLButtonUp(UINT, TPoint& point);
		 void EvGetMinMaxInfo(MINMAXINFO  & minmaxinfo);
		 DECLARE_RESPONSE_TABLE(TControlChild);

};
#endif
