#include "pendldlg.h"

PendelParameter::PendelParameter() {

	// initialisiert die Vorgabewerte der Pendelparameter

	_stabiler_Winkel=0;
	_stabile_Epochen=20;

	_instabiler_Winkel=90;
	_Startwinkel=0;
	_InitBereich=45;

	_reinit=_zufaellig_initialisieren=true;
}

DEFINE_RESPONSE_TABLE1(PendelDialog, TDialog)
	EV_CHILD_NOTIFY(IDOK, BN_CLICKED, HandleOK),
	EV_CHILD_NOTIFY(IDCANCEL, BN_CLICKED, HandleAbbruch),
END_RESPONSE_TABLE;

PendelDialog::PendelDialog(TFrameWindow* parent, PendelParameter& Parameter) : TDialog (parent,DIALOG_PENDEL) {

	// Parameter speichern
	_Parameter=Parameter;

	// Dialogelemente erstellen
	stabiler_Winkel=new TEdit(this,IDC_STABILER_WINKEL,6);
	stabile_Epochen=new TEdit(this,IDC_STABILE_EPOCHEN,6);

	instabiler_Winkel=new TEdit(this,IDC_INSTABILER_WINKEL,6);
	Startwinkel=new TEdit(this,IDC_STARTWINKEL,6);
	InitBereich=new TEdit(this,IDC_ZUFAELLIGER_BEREICH,6);

	zufaellig=new TRadioButton(this,IDC_ZUFAELLIG);
	fest=new TRadioButton(this,IDC_FEST);

	reinit= new TCheckBox(this,IDC_REINIT);
}

void PendelDialog::SetupWindow() {

	TDialog::SetupWindow();
	char Puffer[10];

	// Dialogelemente mit den Parameterwerten besetzen

	sprintf(Puffer,"%.1f",_Parameter._stabiler_Winkel);
	stabiler_Winkel->SetText((char *)Puffer);

	sprintf(Puffer,"%d",_Parameter._stabile_Epochen);
	stabile_Epochen->SetText(Puffer);

	sprintf(Puffer,"%.1f",_Parameter._instabiler_Winkel);
	instabiler_Winkel->SetText(Puffer);

	sprintf(Puffer,"%.1f",_Parameter._Startwinkel);
	Startwinkel->SetText(Puffer);


	sprintf(Puffer,"%.1f",_Parameter._InitBereich);
	InitBereich->SetText(Puffer);

	// Checkboxen initialisieren

	if (_Parameter._zufaellig_initialisieren)
		zufaellig->SetCheck(true);
	else
		fest->SetCheck(true);

	reinit->SetCheck(_Parameter._reinit);
}

PendelDialog::~PendelDialog() {

	// Dialogelmente freigeben

	delete stabiler_Winkel;
	delete stabile_Epochen;
	delete instabiler_Winkel;
	delete Startwinkel;
	delete InitBereich;

	delete zufaellig;
	delete fest;

	delete reinit;
}

void PendelDialog::HandleOK() {

	// bertrgt die genderten Parameter aus der Dialogbox in das Parameter-
	// objekt

	bool ok=true;
	char Wert[6];
	double Ergebnis;

	// stabiler Winkel holen, Bereichsberprfung, speichern
	stabiler_Winkel->GetText((char *)&Wert,6);
	Ergebnis=fabs(atof(Wert));

	if (Ergebnis >= 90) {
		MessageBox("|stable angle| <90","Range error");
		ok=false;
	}
	else
		_Parameter._stabiler_Winkel=Ergebnis;

	// instabiler Winkel holen, Bereichsberprfung, speichern
	instabiler_Winkel->GetText((char *)&Wert,6);
	Ergebnis=fabs(atof(Wert));

	if (Ergebnis > 90) {
		MessageBox("|instable angle| <=90","Range error");
		ok=false;
	}
	else
		_Parameter._instabiler_Winkel=Ergebnis;


	// Bereich aus dem der Winklen initialisiert wird holen,
	// Bereichsberprfung, speichern

	InitBereich->GetText((char *)&Wert,6);
	Ergebnis=fabs(atof(Wert));

	if (Ergebnis >= 90) {
		MessageBox("0< initial angle range <90","Range error");
		ok=false;
	}
	else
		_Parameter._InitBereich=Ergebnis;

	// Startwinkel holen, Bereichsberprfung, speichern
	Startwinkel->GetText((char *)&Wert,6);
	Ergebnis=atof(Wert);

	if (Ergebnis >= 90) {
		MessageBox("|initial angle| <90","Range error");
		ok=false;
	}
	else
		_Parameter._Startwinkel=Ergebnis;

	// stabile Zyklen holen, Bereichsberprfung, speichern
	stabile_Epochen->GetText((char *)&Wert,6);
	Ergebnis=fabs(atof(Wert));

	if (Ergebnis >= 32000) {
		MessageBox("|stable epochs| <32000","Range error");
		ok=false;
	}
	else
		_Parameter._stabile_Epochen=Ergebnis;

	// Winkel "zufllig" initialisieren ?
	_Parameter._zufaellig_initialisieren=zufaellig->GetCheck();

	// falls Winkel konstant, neu initialissieren ?
	_Parameter._reinit=reinit->GetCheck();

	if (ok) {
		_uebernommen=true;
		TDialog::CloseWindow();
	}
}

void PendelDialog::HandleAbbruch() {

	// falls nicht mit "OK" beendet, dies speichern

	_uebernommen=false;
	TDialog::CloseWindow();
}
