#include "pendlwin.h"

DEFINE_RESPONSE_TABLE1(PendelWin,TFrameWindow)
	EV_WM_LBUTTONUP,
	EV_WM_PAINT,
	EV_WM_SIZE,
END_RESPONSE_TABLE;

PendelWin::PendelWin(TWindow* parent,HINSTANCE hInstance)
	: TFrameWindow(parent,"Inverted pendulum") ,Server(this,hInstance) {
	_letzter_Winkel=0;

	// Gre des Fensters geeignet whlen
	Attr.W=185;
	Attr.H=160;

	// Parameter bernehmen
	Server.setze_Parameter(_Parameter);

	// Tabelle mit Sinus- und Cosinuswerten aus Geschwindigkeitsgrnden
	// vorberechnen

	for (int i=-90; i<=90; ++i) {
		FTyp Winkel= i* M_PI / 180 ;
		sintab[i+90]=sin(Winkel);
		costab[i+90]=cos(Winkel);
	}
}

void PendelWin::berechne_Konstante(TRect& groesse) {

	// berechnet Konstante fr die Darstellung, die sich aus der Fenster-
	// gre ableiten

	_Laenge = min(groesse.Width(),groesse.Height()) / 2;
	_Radius = _Laenge / 10;
	_Mitte = groesse.left + groesse.Width() / 2;
	_Wagenhoehe = _Laenge / 5;
	_Wagenbreite = 2.5* _Wagenhoehe;

}
void PendelWin::Wagen_malen() {

	// malt den Wagen, auf dem das inverse Pendel montiert ist
	// wird nur gemalt, wenn sich die Fenstergre gendert hat

	TClientDC mydc(*this);
	TRect groesse=GetClientRect();

	mydc.SelectObject(TPen(TColor::Black, 1));

	// Wagen malen...
	mydc.SelectObject(TBrush(TColor::LtGray));
	mydc.Rectangle(_Mitte-_Wagenbreite,groesse.bottom,
						_Mitte+_Wagenbreite,groesse.bottom-_Wagenhoehe);


	mydc.SelectObject(TBrush(TColor::Gray));
	mydc.Ellipse(_Mitte-_Wagenbreite*4/5,groesse.bottom+_Wagenhoehe/2,
					_Mitte-_Wagenbreite*1/5,groesse.bottom-_Wagenhoehe/2);
	mydc.Ellipse(_Mitte+_Wagenbreite*4/5,groesse.bottom+_Wagenhoehe/2,
					_Mitte+_Wagenbreite*1/5,groesse.bottom-_Wagenhoehe/2);

	mydc.SelectObject(TBrush(TColor::Black));
	mydc.Ellipse(_Mitte-_Wagenbreite*2/3,groesse.bottom+_Wagenhoehe/3,
					_Mitte-_Wagenbreite*1/3,groesse.bottom-_Wagenhoehe/3);
	mydc.Ellipse(_Mitte+_Wagenbreite*2/3,groesse.bottom+_Wagenhoehe/3,
					_Mitte+_Wagenbreite*1/3,groesse.bottom-_Wagenhoehe/3);

	mydc.SelectObject(TBrush(TColor::White));
}

void PendelWin::Malen(TDC& mydc,FTyp& Winkel, TRect& groesse, bool loeschen ) {

	// malt das inverse Pendel

	// falls loeschen=true, dann wird die Zeichnung des Pendels gelscht

	if (loeschen) {
		mydc.SelectObject(TPen(TColor::White,2));
	}
	else {
		mydc.SelectObject(TPen(TColor::Black,2));
	}
	int iWinkel=Winkel+90;

	// Position der ausgelenkten Spitze berechnen

	FTyp top_x = _Mitte + sintab[iWinkel] * _Laenge;
	FTyp top_y = groesse.bottom - _Wagenhoehe -  2 - costab[iWinkel] * _Laenge;

	// Stab zeichnen

	mydc.MoveTo(_Mitte,groesse.bottom-_Wagenhoehe-1);
	mydc.LineTo(top_x,top_y);

	if (loeschen)
		mydc.Ellipse(top_x-_Radius,top_y-_Radius,top_x+_Radius,top_y+_Radius);
	else {

		// Kugel auf der Spitze zeichnen

		mydc.SelectStockObject(NULL_PEN);
		mydc.SelectObject(TBrush(TColor::LtBlue));
		mydc.Ellipse(top_x-_Radius,top_y-_Radius,top_x+_Radius,top_y+_Radius);

		mydc.SelectObject(TBrush(TColor::LtCyan));
		mydc.Ellipse(top_x-_Radius*0.5,top_y-_Radius*0.5,top_x+_Radius*0.25,top_y+_Radius*0.25);
		mydc.SelectStockObject(WHITE_BRUSH);
		mydc.SelectStockObject(BLACK_PEN);
	}

}
void PendelWin::Paint(TDC&, bool, TRect& ) {

	// Darstellung des Pendels lschen

	// Fenster in Symbolgre ?
	if (!IsIconic()) {

		TClientDC mydc(*this);
		TRect groesse=GetClientRect();
		mydc.SelectStockObject(NULL_PEN);
		mydc.SelectStockObject(WHITE_BRUSH);
		mydc.Rectangle(groesse);

		Wagen_malen();

		Malen(mydc,(int)_letzter_Winkel,groesse);
	}
}
void PendelWin::Update() {

	// Darstellung des Pendels erneuern

	// Fenster in Symbolgre ?
	if (!IsIconic()) {

		TClientDC mydc(*this);
		TRect groesse=GetClientRect();

		// Winkel des Pendels vom Server ermitteln
		FTyp Winkel  = Server._Pendel.winkel();

		// Pendelauslenkung notfalls clippen
		if (Winkel > 90)
			Winkel=90;
		else
			if (Winkel < -90)
				Winkel=-90;

		// nur wenn sich der Winkel gendert hat, Pendel neuzeichnen
		if ((int) _letzter_Winkel != (int) Winkel) {
			Malen(mydc,(int)_letzter_Winkel,groesse,true);
			Malen(mydc,(int)Winkel,groesse);
			_letzter_Winkel=Winkel;
		}
	}
}
void PendelWin::SetupWindow() {
	TFrameWindow::SetupWindow();
	berechne_Konstante(GetClientRect());
}

void PendelWin::EvSize(UINT sizeType, TSize &size) {
	TFrameWindow::EvSize(sizeType,size);

	// falls sich die Gre des Fensters gendert hat, neue Gre der
	// Darstellung berechnen
	berechne_Konstante(GetClientRect());

	// ... und neuzeichnen
	Invalidate(true);
}

void PendelWin::EvLButtonUp(UINT, TPoint&) {

	// zeigt den Kontexdialog des inversen Pendels an und holt die neuen Parameter

	PendelDialog PendelDlg(this,_Parameter);
	PendelDlg.Execute();

	if (PendelDlg.uebernommen()) {
		_Parameter=PendelDlg.Parameter_holen();
		Server.setze_Parameter(_Parameter);
		Server._Pendel.winkel(_Parameter._Startwinkel);
		Update();
	}
}
//----------------------------- DDE-Server -------------------------------------
#include "ddeserv.h"

	// Pendelfenster
	PendelWin* DDEServer::_parent;

	// Pendel
	Pendel DDEServer::_Pendel;
	PendelParameter DDEServer::_Parameter;
	int DDEServer::_stabileEpochen;
	int DDEServer::_letzterWinkel;
	bool DDEServer::_first,DDEServer::_very_first;

	// Anwendungsparameter
	int DDEServer::_Anwendungsstatus;
	char DDEServer::AnwendungsName[20];
	int DDEServer::Anzahl_EingabeVariable,DDEServer::Anzahl_AusgabeVariable;
	double DDEServer::Eingabewerte[MAX_EINGABEVARS];
	double DDEServer::Ausgabewerte[MAX_CONTROLLER];

	// DDE
	char DDEServer::SendePuffer[80], DDEServer::EmpfangsPuffer[80];

	HINSTANCE DDEServer::hInst;
	FARPROC DDEServer::lpDdeProc;
	DWORD DDEServer::InstId;
	DWORD DDEServer::aCbFilter;
	HCONV DDEServer::hConvApp;

	// Stringhandles
	HSZ DDEServer::hszService,DDEServer::hszTopic,DDEServer::hszItem,
			DDEServer::hszInitialisieren,
			DDEServer::hszEingabe, DDEServer::hszAusgabe,
			DDEServer::hszIdentifizieren;

DDEServer::DDEServer(PendelWin* parent,HINSTANCE hInstance) {

	_stabileEpochen=0;
	_parent=parent;

	// Anwendungsname und Anzahl der Ein- und Ausgabevariablen definieren
	strcpy(AnwendungsName,"Pendel");
	Anzahl_EingabeVariable=2;
	Anzahl_AusgabeVariable=1;

	hInst=hInstance;

	hConvApp	= (HCONV)NULL;
	lpDdeProc = MakeProcInstance((FARPROC)&DDEServer::DDECallback,hInst);
	InstId=0;

	// DDE initialisieren
	DdeInitialize((LPDWORD)&InstId,(PFNCALLBACK)lpDdeProc,APPCLASS_STANDARD,0L);

	// Stringhandles generieren
	hszService = 	DdeCreateStringHandle(InstId,"NEFCON-WIN",CP_WINANSI);
	hszTopic = 		DdeCreateStringHandle(InstId,"DATA-TRANSFER",CP_WINANSI);
	hszItem = 		DdeCreateStringHandle (InstId, "Data", CP_WINANSI );
	hszInitialisieren = DdeCreateStringHandle (InstId, "Init", CP_WINANSI );
	hszEingabe = DdeCreateStringHandle (InstId, "I", CP_WINANSI );
	hszAusgabe = DdeCreateStringHandle (InstId, "O", CP_WINANSI );
	hszIdentifizieren = DdeCreateStringHandle (InstId, "Ident", CP_WINANSI );

	// Name des Services initialisieren
	DdeNameService(InstId,hszService,(HSZ) NULL, DNS_REGISTER);

	_Anwendungsstatus=1; // = aktiv
}


DDEServer::~DDEServer() {

	// Service nicht mehr registrieren
	DdeNameService(InstId,hszService,NULL, DNS_UNREGISTER);

	FreeProcInstance ( lpDdeProc );

	// Stringhandles freigeben
	DdeFreeStringHandle ( InstId, hszService );
	DdeFreeStringHandle ( InstId, hszTopic );
	DdeFreeStringHandle ( InstId, hszItem );

	DdeFreeStringHandle ( InstId, hszInitialisieren );
	DdeFreeStringHandle ( InstId, hszEingabe );
	DdeFreeStringHandle ( InstId, hszAusgabe );
	DdeFreeStringHandle ( InstId, hszIdentifizieren );

	// Verbindung abmelden
	DdeUninitialize(InstId);
}



void DDEServer::Ausgabe() {

	// Stellwerte der Entwicklungsumgebung lesen

	istrstream A(EmpfangsPuffer);

	for (int i=0; i<Anzahl_AusgabeVariable; ++i) {
		A >>  Ausgabewerte[i] ;
	}
}

void DDEServer::Eingabe() {

	// Mewerte des Prozesses fr die Entwicklungsumgebung verpacken

	ostrstream E;
	for (int i=0; i<Anzahl_EingabeVariable; ++i) {
		E <<  Eingabewerte[i] << " " ;
	}
	E << _Anwendungsstatus;
	E << endl << ends;

	strcpy (SendePuffer,E.str());
	delete E.str();
}

HDDEDATA EXPENTRY DDEServer::DDECallback ( WORD wType, WORD wFmt, HCONV hConv, HSZ hsz1,
															HSZ hsz2, HDDEDATA hData, DWORD dwData1,
														DWORD dwData2 ) {

	// DDE-Callbackfunktion fr den Prozeserver
	switch ( wType ) {

		case XTYP_REQUEST:

			// NEFCON-WIN fordert vom Prozeserver Informationen an

			// was ist gewnscht ?
			if (hsz2 == hszEingabe) {

				// NEFCON-WIN wnscht neue Mewerte

				// neue Mewerte besorgen
				neue_Eingabewerte();

				// ... und in den Sendepuffer schreiben
				Eingabe();

				// Datahandle fr den Sendepuffer erstellen
				hData = DdeCreateDataHandle ( InstId, (LPBYTE) DDEServer::SendePuffer,
														80,
														0L, hszItem, wFmt, 0 );

				// konnte Datahandle erstellt werden ?
				if ( hData != (HDDEDATA)NULL ) {
					// Handle an DDEML bergeben, fertig
					return ( hData );
				}
				else {
					// nein, Nullzeiger zurckliefern
					return ( (HDDEDATA)NULL );
				}
			}
			if (hsz2 == hszInitialisieren) {

				// Anwendung soll initialisiert werden
				Anwendung_initialisieren();

				// "o.k." zurckliefern
				strcpy(SendePuffer,"o.k.");

				// Datahandle fr den Sendepuffer erstellen
				hData = DdeCreateDataHandle ( InstId, (LPBYTE) DDEServer::SendePuffer,
														80,
														0L, hszItem, wFmt, 0 );
				// konnte Datahandle erstellt werden ?
				if ( hData != (HDDEDATA)NULL ) {
					// Handle an DDEML bergeben, fertig
					return ( hData );
				}
				else {
					// nein, Nullzeiger zurckliefern
					return ( (HDDEDATA)NULL );
				}

			}

			if (hsz2 == hszIdentifizieren) {

				// Prozeserver soll sich identifizieren

				// Prozeeigenschaften in den Sendepuffer schreiben
				sprintf(SendePuffer,"%s %d %d",AnwendungsName,
														Anzahl_EingabeVariable,
														Anzahl_AusgabeVariable);

				// Datahandle fr den Sendepuffer erstellen
				hData = DdeCreateDataHandle ( InstId, (LPBYTE) DDEServer::SendePuffer,
														80,
														0L, hszItem, wFmt, 0 );

				// konnte Datahandle erstellt werden ?
				if ( hData != (HDDEDATA)NULL ) {
					// Handle an DDEML bergeben, fertig
					return ( hData );
				}
				else {
					// nein, Nullzeiger zurckliefern
					return ( (HDDEDATA)NULL );
				}
			}
		break;


		case XTYP_POKE:

			if ( hsz1 == hszTopic ) {
				if (hsz2 == hszAusgabe) {

					// NEFCON-WIN hat neue Stellwerte bertragen

					// Data-Handle auslesen und Inhalt in den Empfangspuffer kopieren
					DdeGetData ( hData, (LPBYTE) EmpfangsPuffer, 80L, 0L );

					// Data-Handle freigeben
					DdeFreeDataHandle(hData);

					// Stellwerte aus dem Empfangspuffer lesen
					Ausgabe();

					// ... und ausgeben
					neue_Ausgabewerte();

				}
				else {

					// was auch immer im Empfangspuffer gelandet ist, ignorieren

					DdeGetData ( hData, (LPBYTE) EmpfangsPuffer, 80L, 0L );
					DdeFreeDataHandle(hData);
				}

				if ( EmpfangsPuffer != NULL ) {

					// Empfangsbesttigung senden
					return ( (HDDEDATA) DDE_FACK );
				}
			}
			else
				// sonst Nullpointer senden
				return ( (HDDEDATA) NULL );
		break;

		case XTYP_CONNECT: if ( hsz2 == hszService ) {
										// Verbindung steht !
										_very_first=true;
										return ( (HDDEDATA) TRUE );
								}
								else {
									// Fehler
									return ( (HDDEDATA) FALSE );
								}
		break;

		case XTYP_CONNECT_CONFIRM:
			// zugewiesenes Handle speichern
			hConvApp = hConv;
		break;

		case XTYP_DISCONNECT:
			// Auflsung der Verbindung mit Nullpointer quittieren
			hConvApp = (HCONV)NULL;
		break;

		// alles andere ignorieren
		case XTYP_EXECUTE:
			break;
		case XTYP_ERROR:
			break;
	}

	// sonst Nullpointer zurckgeben
	return ( (HDDEDATA) NULL );
}

void DDEServer::neue_Eingabewerte() {

	// neue Eingabewerte durch Simulation ermitteln

	if (!_first) {
		_Pendel.simulation(Ausgabewerte[0]);

		// neue Pendelauslenkung zeichnen
		_parent->Update();
	}
	else {
		_first=false;
	}

	if (_very_first) {

		// falls erster Zyklus, Fenster des Prozeservers in den Vordergrund
		// bringen

		_very_first=false;
		_parent->SetFocus();
	}

	// neue Eingabewerte schreiben

	Eingabewerte[0]=_Pendel.winkel();
	Eingabewerte[1]=_Pendel.wi_geschw();

	double Winkel=Eingabewerte[0];

	// Anwendungsstatus ermitteln

	// stabile Epochen zhlen
	if ((int)Winkel==_letzterWinkel)
		++_stabileEpochen;
	else {
		_stabileEpochen=0;
		_letzterWinkel=Winkel;
	}

	// gengend stabile Epochen ?

	if (_stabileEpochen==_Parameter._stabile_Epochen) {

		// ja, jetzt berprfen ob die Abweichung vom Nullpunkt tolerabel

		if ((int)fabs(Winkel) <= _Parameter._stabiler_Winkel)
			_Anwendungsstatus=2; // Pendel konnte stabilisiert werden
		else
			_Anwendungsstatus=3; // Pendel stabil, aber nichjt optimal

		// soll reinitialisiert werden ?
		if (_Parameter._reinit)
			Anwendung_initialisieren();
	}
	else {
		// nein, Pendel umgefallen ?

		double absWinkel=fabs(Winkel);

		if (absWinkel>_Parameter._instabiler_Winkel)
			_Anwendungsstatus=0; // ja, also Regelversagen
		else
			_Anwendungsstatus=1; // nein, als Anwendungsstatus = aktiv
	}
}

void DDEServer::neue_Ausgabewerte() {

	// ist hier redundant
}

void DDEServer::Anwendung_initialisieren() {

	// zufllige Initialisierung des Pendelwinkels gewnscht ?
	if (_Parameter._zufaellig_initialisieren)
		// ja, also aus dem Bereich "zufllig" initialisieren
		_Pendel.winkel(_Parameter._InitBereich - 2* (rand() % (int)_Parameter._InitBereich));
	else
		// nein, Startwinkel festlegen
		_Pendel.winkel(_Parameter._Startwinkel);

	_letzterWinkel=_Pendel.winkel();

	// sonstige Parameter rcksetzen
	_stabileEpochen=0;
	_Pendel.wi_geschw(0);
	_Pendel.wa_geschw(0);
	_Pendel.position(0);

	_first=true;
}



