#include "Regel.h"

Regel::Regel(TEingabeVarliste* EingabeVarListe,  AusgabeVariable* AusgabeVar) :
	 _EingabeVarListe(EingabeVarListe), _AusgabeVar(AusgabeVar),
	 _Praemisse() , _Konklusion(""), _letzterErfuelltheitsgrad(0) {
	 _used=_Cr=_Zr=0;
	 _ausgewaehlt=false;
	 _verwendbar=true;
	 _Op=Minimum;
}

void Regel::Praemisse_hinzufuegen(mystring& Name) {
	// hngt einen Namen an die Liste der Namen der Fuzzy-Mengen in der Prmisse
	// an
	_Praemisse.append(Name);
}
void Regel::Konklusion_setzen(mystring& Name) {
	// setzt den Namen der Fuzzy-Menge in der Konklusion
  _Konklusion=Name;
}
void Regel::Operator_setzen(Operator Op) {
	// setzt die zu verwendende t-Norm
  _Op=Op;
}

bool Regel::Auswertung_vorbereiten() {

	// bersetzt die Namen der Fuzzy-Mengen in Zeiger auf die diesen Namen
	// zugeordneten Objekten

	bool ok=true;
	_Fehlermeldung="";

	// bisherige Pointerliste verwerfen
	_Pointer.clear();

	// Konklusion bersetzen

	// gibt es die Fuzzy-Menge mit dem Namen in der Konklusion in der
	// Ausgabevariablen ?

	if (!_AusgabeVar->ex_Set(_Konklusion)) {
		// nein, meckern
		_Fehlermeldung = _Konklusion + " used in rule, but doesn't exist in variable " + _AusgabeVar->Name() + ".\n" ;
		ok=false;
	}
	else
		// Zeiger holen
		_AusgabeSet = _AusgabeVar->hol_Set(_Konklusion);

	FsSet* result;

	// Prmisse bersetzen

	TStringlisteIter PraemisseIter(_Praemisse);
	TEingabeVarlisteIter EingabeVarIter(*_EingabeVarListe);

	while (EingabeVarIter && PraemisseIter) {

		// berpfen, ob die Fuzzy-Menge in der jeweiligen Eingabevariablen ex.
		if (!EingabeVarIter.Current()->ex_Set(PraemisseIter.Current())) {
			// nein, meckern
			_Fehlermeldung += PraemisseIter.Current() + " used in rule, but doesn't exist in variable "
									+ EingabeVarIter.Current()->Name() + ".\n" ;
			ok=false;
		}
		else {
			// Zeiger holen
			result=(EingabeVarIter.Current())->hol_Set(PraemisseIter.Current());
			// Zeiger in die Liste der Zeiger auf die Fuzzy-Mengen der
			// Prmisse aufnehmen
			_Pointer.append(result);
		}
		// nchste Eingabevariable, nchster Eintrag in der Prmisse
		EingabeVarIter++;
		PraemisseIter++;
  }

  return ok;
}

FTyp& Regel::Regel_auswerten(FTyp& Erfuelltheitsgrad) {

	// wertet die Regel aus, liefert Erflltheitsgrad und Stellwert

	FTyp result;
	FTyp min(1);

	// durch die Pointerliste laufen, Aktivierung ermitteln und mit t-Norm
	// verknpfen

	TSetlisteIter FastIter(_Pointer);

	switch (_Op) {

			// t-Norm = min
			case Minimum: {

				while (FastIter && min>0) {

					result=(FastIter.Current())->Aktivierung();

					if (result<min)
						min=result;

					FastIter++;
				}

				break;
			}
			// t-Norm = harmonisches Mittel
			case HarmonicMean: {

				while (FastIter && min>0) {

					result=(FastIter.Current())->Aktivierung();
					min = 2*min*result / (min + result);

					FastIter++;
				}

				break;
			}

			// t-Norm = Produkt
			case Product: {

				while (FastIter && min>0) {

					result=(FastIter.Current())->Aktivierung();
					min = min*result ;
					FastIter++;
				}

				break;
			}
		}

	// Ausgabemenge invertieren
	_Stellwert=_AusgabeSet->x_wert(min);
	// Erflltheitsgrad speichern
	_letzterErfuelltheitsgrad=Erfuelltheitsgrad=min;
	// Stellwert an Regelbasis zurckliefern
	return _Stellwert;
}

void Regel::delta_ruecksetzen() {
	// setzt die Schrittweite zurck (fr SuperSAB, Delta-Bar-Delta)
	_delta=_delta0;
}

int Regel::operator==(const Regel& mc) const
{
	// vergleicht zwei Regeln,
	// liefert wahr, wenn Prmissen und Konklusionen identisch sind
	 return ((_Praemisse == mc._Praemisse) && (_Konklusion== mc._Konklusion));
}


mystring Regel::Regel_als_String() {

	// liefert die Regel im
	// 		If ... is ... and ... is ... then ... is ... - Format
	// als String z.B. fr den Regelbasen-Editor

	#define FBREITE 3

	TStringlisteIter PraemisseIter(_Praemisse);
	TEingabeVarlisteIter EingabeVarIter(*_EingabeVarListe);

	ostrstream os;

	os << "If " ;

	bool first(true);
	os.setf(ios::left);

	// Prmisse der Regel ausgeben

	while (PraemisseIter) {

		os.width(0);

		if (!first)
			os << " and ";
		else {
			first=false;
		}

		os << EingabeVarIter.Current()->Name() ;

		os.width(FBREITE);
		os << " is " << PraemisseIter.Current() ;

		PraemisseIter++;
		EingabeVarIter++;
	}

	// Konklusion der Regel ausgeben

	os.width(0);
	os << " then ";
	os << _AusgabeVar->Name();
	os	<< " is " << _Konklusion << "." << ends;

	mystring Ausgabe(os.str());
	delete os.str();

	return Ausgabe;
}

void Regel::Regel_adaptieren(FTyp& Fehler) {

	// adaptiert die Regel durch Vernderung der Trger der Fuzzy-Mengen

	FTyp absFehler=fabs(Fehler);
	FTyp Fehler_dieser_Regel;

	// Fehler dieser Regel berechnen
	if ((Fehler * _Stellwert) > 0.0)
		Fehler_dieser_Regel = absFehler *_letzterErfuelltheitsgrad;
	else
		Fehler_dieser_Regel = -absFehler * _letzterErfuelltheitsgrad;

	// SuperSAB und Delta-Bar-Delta fhren fr jede Regel eine eigene
	// Schrittweite ein, die in Abhngigkeit der Fehler verndert wird

	if ((_Verfahren==fssab) || (_Verfahren == fssabv)) {

		// SuperSAB
		// hat Fehler gleiche Tendenz behalten ?
		FTyp test=(_letzter_Fehler * Fehler_dieser_Regel);
		if (test > 0.0)
			_delta*=_delta_plus;
		else
			if (test < 0.0)
				_delta*=_delta_minus;
	}
	else {
		if ((_Verfahren==fdbd) || (_Verfahren==fdbdv)) {

			// DELTA-BAR-DELTA
			// hat Fehler gleiche Tendenz behalten ?
			FTyp test=(_letzter_Fehler * Fehler_dieser_Regel);
			if (test > 0.0)
				_delta+=_delta_plus;
			else
				if (test < 0.0)
					_delta*=_delta_minus;
		}
	}
	if (Fehler_dieser_Regel!=0.0) {

		// Adaptionsfaktor berechnen
		FTyp Faktor = _delta * Fehler_dieser_Regel;

		// jede Fuzzy-Menge in der Prmisse skalieren
		TSetlisteIter FastIter(_Pointer);
		while (FastIter) {
			FastIter.Current()->skalieren(Faktor);
			FastIter++;
		}

		// soll die Fuzzy-Menge in der Konklusion verschoben werden ?
		if ((_Verfahren==fbackpropv) || (_Verfahren==fssabv) || (_Verfahren==fdbdv)){

			// ja, Betrag der Verschiebung berechen
			FTyp dx=(_AusgabeSet->x2()-_AusgabeSet->x1())*fabs(Faktor);

			// Richtung der Verschiebung berechnen
			if ((_letzter_Fehler*Fehler_dieser_Regel) > 0.0) {
				if (fabs(_letzter_Fehler) < fabs(Fehler_dieser_Regel)) {
					if (_AusgabeSet->verschiebbar_mit_Restriktionen(dx))
						_AusgabeSet->verschieben(dx);
				}
			}
			else {
				if ((_letzter_Fehler*Fehler_dieser_Regel) < 0.0)
					if (_AusgabeSet->verschiebbar_mit_Restriktionen(-dx))
						_AusgabeSet->verschieben(-dx);
			}
		}
		else {
			// nein, also Fuzzy-Menge in der Konklusion skalieren
			_AusgabeSet->skalieren(-Faktor);
		}
	}

	// den aktuellen Fehler der Regel merken
	_letzter_Fehler=Fehler_dieser_Regel;

}
void  Regel::Lernen_vorbereiten(Verfahren Verf, int Epochen,
									FTyp& delta,FTyp& delta_plus,FTyp& delta_minus) {


	// initialisiert die Lernparameter


	_Epochen=Epochen;

	_Verfahren=Verf;
	_delta0=_delta=delta;
	_delta_plus=delta_plus;
	_delta_minus=delta_minus;

	_letzter_Fehler=0;
	_bisher=0;
}

int operator > (const Regel& Regel1, const Regel& Regel2) {

	// wird zur alphabetischen Sortierung verwendet

	return (Regel1.Regel_als_String() > Regel2.Regel_als_String());
}

ostream& operator << (ostream& os, Regel& eineRegel) {

	TStringlisteIter Iter(eineRegel.Praemisse_holen());

	os << eineRegel._Praemisse.GetItemsInContainer() << " " ;

	while (Iter) {
		os << Iter.Current() << " ";
		Iter++;
	}
	os << eineRegel.Konklusion_holen();
	os << endl ;

	return os;
}
istream& operator >> (istream& is, Regel& eineRegel) {
	eineRegel.Praemisse_holen().Flush();

	int Zaehler;
	is >> Zaehler;

	mystring temp;

	while (Zaehler) {
		is >> temp;
		eineRegel.Praemisse_hinzufuegen(temp);
		--Zaehler;
	}

	is >> temp;
	eineRegel.Konklusion_setzen(temp);

	return is;
}

