#include "Regelbas.h"

// #define DEBUG_CONTAINER
// #define DEBUG_NAUCK_PHASE_1
// #define DEBUG_NAUCK_PHASE_2

Regelbasis::Regelbasis(TEingabeVarliste* EingabeVarListe,
							 AusgabeVariable* AusgabeVar,
							 Operator Op):
  _EingabeVarListe(EingabeVarListe), _AusgabeVar(AusgabeVar), _Regelliste(),
  _Op(Op) {
  _EditFenster=0;
}

void Regelbasis::Regel_hinzufuegen(Regel& eineRegel) {

	// fgt eine Regel in die Regelliste ein

	// neue Regel mit den Eigenschaften der bergebenen Regel konstruieren
	Regel* temp=new Regel(eineRegel);
	// t-Norm setzen
	temp->Operator_setzen(_Op);

	// in die Regelbasis aufnehmen
  _Regelliste.append(temp);
}

bool  Regelbasis::Auswertung_vorbereiten() {

	// bereitet jede Regel auf die Auswertung vor, d.h. es werden
	// die Namen der Fuzzy-Mengen in Zeiger umgewandelt

	TRegellisteIter Iter(_Regelliste);
	bool ok=true;
	_Fehlermeldung="";

	while (Iter) {
		ok = ok && Iter.Current()->Auswertung_vorbereiten();
		if (!ok) {
			_Fehlermeldung+=Iter.Current()->hol_Fehlermeldung();
		}
		Iter++;
	}
	return ok;
}

void  Regelbasis::Lernen_vorbereiten(Verfahren Verf,RegelLernVerfahren RegelVerfahren,
												 int Epochen,
												 FTyp& delta,FTyp& delta_plus,FTyp& delta_minus) {

	// jede Regel wird mit bergebenen Lernparametern initialisiert

	TRegellisteIter Iter(_Regelliste);
	_Verfahren=Verf;
	_RegelLernVerfahren=RegelVerfahren;

	while (Iter) {
		Iter.Current()->Lernen_vorbereiten(Verf,
														Epochen,
														delta,
														delta_plus,
														delta_minus);
		Iter++;
	}
}
void  Regelbasis::Operator_setzen(Operator Op) {

	// die bergebene t-Norm wird allen Regeln mitgeteilt

	TRegellisteIter Iter(_Regelliste);
	_Op=Op;

	while (Iter) {
		Iter.Current()->Operator_setzen(Op);
		Iter++;
	}
}
int Regelbasis::Regelaktivitaet_bestimmen() {

	// bestimmt die Anzahl der aktiven Regeln in der Regelbasis

	int count=0;
	TRegellisteIter Iter(_Regelliste);
	while (Iter) {

		Iter.Current()->Regel_auswerten(_ErgebnisGrad);
		if (_ErgebnisGrad>0)
			++count;

		Iter++;
	}

  return count;
}

FTyp Regelbasis::MaxAktivitaet_bestimmen() {

	// bestimmt den maximalen Erflltheitsgrad ber alle Regeln dieser
	// Regelbasis

	FTyp Max=0;
	TRegellisteIter Iter(_Regelliste);
	while (Iter) {
		Iter.Current()->Regel_auswerten(_ErgebnisGrad);
		Max=max(_ErgebnisGrad,Max);
		Iter++;
	}
	return Max;
}

FTyp& Regelbasis::Regeln_auswerten(){

	// wertet die Regelbasis aus, ermittelt den Stellwert

  _Ergebnis=_ErgebnisGrad=_Ausgabe=_alleGrade=0;

  TRegellisteIter Iter(_Regelliste);
  while (Iter) {

		_Ergebnis=Iter.Current()->Regel_auswerten(_ErgebnisGrad);

		// fr Defuzzyfizierung :
		_Ausgabe+=_ErgebnisGrad*_Ergebnis;
		_alleGrade+=_ErgebnisGrad;

		Iter++;
	}

  // alle Regeln sind jetzt ausgewertet, Stellwert berechnen

  if (_alleGrade!=0)
	 _Steuerwert=_Ausgabe/_alleGrade;
  else {
	 _Steuerwert=0;
  }

  // Stellwert der Ausgabevariablen aktualisieren
  _AusgabeVar->_Ausgabewert=_Steuerwert;

  return _Steuerwert;
}

void Regelbasis::adaptieren(FTyp& Fehler) {

	// ruft die Adaption der Fuzzy-Mengen jeder Regel in dieser Regelbasis auf

	TRegellisteIter Iter(_Regelliste);

	while (Iter) {
		Iter.Current()->Regel_adaptieren(Fehler);
		Iter++;
	}
}

void Regelbasis::delta_ruecksetzen() {

	// setzt die Schrittweite jeder Regel zurck (fr SuperSAB, Delta-Bar-Delta)

	TRegellisteIter Iter(_Regelliste);

	while (Iter) {
		Iter.Current()->delta_ruecksetzen();
		Iter++;
	}
}
bool Regelbasis::enthaelt_Regel(Regel& test) {

	// ermittelt, ob die bergebene Regel bereits in der Regelbasis definiert ist

	bool gefunden=false;
	TRegellisteIter Iter(_Regelliste);
	while (Iter && (!gefunden)) {

		if (*Iter.Current()==test)
			gefunden=true;

		Iter++;
	}
	return gefunden;
}

void Regelbasis::Regeln_lernen_Stellmach(FTyp& Fehler) {

	// bastelt aus dem aktuellen erweiterten Fehler und den Werten der
	//	Eingabevariablen einen Vorschlag fr eine Kontrollregel

	Regel tempRegel(_EingabeVarListe,_AusgabeVar);
	TEingabeVarlisteIter Iter(*_EingabeVarListe);
	bool ok=true;
	FsSet* bestes;

	// Praemisse kontruieren (ber alle Eingabevariablen)
	while (Iter && ok) {

		// hol Fuzzy-Menge mit maximalem Treffergrad (x)
		bestes=Iter.Current()->hol_bestes_FuzzySet();

		// falls es eine gibt, deren Name in die neue Regel aufnehmen
		if (bestes==0)
			ok=false;
		else
			tempRegel.Praemisse_hinzufuegen(bestes->Name());

		Iter++;
	}

	// Konklusion konstruieren
	if (ok) {

		// Vorschlag fr den Stellwert gem. Stellmach berechnen
		FTyp Stellwert;
		if (Fehler >=0)
			Stellwert=Fehler*_AusgabeVar->max_holen();
		else
			Stellwert=-Fehler*_AusgabeVar->min_holen();

		// optimale Fuzzy-Menge beschaffen
		bestes=_AusgabeVar->hol_bestes_FuzzySet(Stellwert);
		if (bestes!=0) {
			tempRegel.Konklusion_setzen(bestes->Name());

		}
		else
			ok=false;
	}

	// exisitert bereits eine Regel mit dieser Prmisse in der Regelbasis ?
	if (ok)
		if(!enthaelt_Regel_mit_gleicher_Praemisse(tempRegel)) {
			Regel_hinzufuegen(tempRegel);
		}
}


void Regelbasis::verfuegbare_FuzzySets_markieren() {

	// falls diese Regelbasis fr den erweiterten Fehler verwendet wird :

	// alle verwendeten Fuzzy-Mengen fr das Lernen von Kontrollregeln als
	// nicht-verfgbar markieren

	// erstmal alle erlauben
	TEingabeVarlisteIter EIter1(*_EingabeVarListe);
	while (EIter1) {
		EIter1.Current()->alle_FuzzySets_fuer_Regeln_erlauben();
		EIter1++;
	}

	// jede Regel betrachten, verwendete Fuzzy-Mengen in der Prmisse verbieten
	TRegellisteIter Iter(_Regelliste);

	while (Iter) {

		TStringliste& Praemisse=Iter.Current()->Praemisse_holen();

		TEingabeVarlisteIter EIter(*_EingabeVarListe);
		TStringlisteIter PIter(Praemisse);

		while (EIter && PIter) {
			EIter.Current()->hol_Set(PIter.Current())->_fuer_Regel_verfuegbar=false;
			EIter++;
			PIter++;
		}
		Iter++;
	}
}

void Regelbasis::entferne_Regel(Regel* eineRegel) {

	// entfernt die Regel aus der Regelbasis und gibt sie frei
	_Regelliste.Detach(eineRegel);
	delete eineRegel;
}

bool Regelbasis::enthaelt_Regel_mit_gleicher_Praemisse(Regel& eineRegel) {

	// berprfen, ob in der Regelliste eine Regel mit gleicher Prmisse
	// existiert

	TRegellisteIter Iter(_Regelliste);
	bool gefunden(false);

	while (Iter && !gefunden) {
		// Prmissen der beiden Regeln vergleichen
		if (Iter.Current()->Praemisse_holen()==eineRegel.Praemisse_holen())
				gefunden=true;
		Iter++;
	}
	return gefunden;
}

ostream& operator << (ostream& os, Regelbasis& eineRegelbasis) {

	os << eineRegelbasis._Op << " " ;

	TRegellisteIter Iter(eineRegelbasis._Regelliste);
	os << eineRegelbasis._Regelliste.GetItemsInContainer() << endl ;

	while (Iter) {
		os << *Iter.Current();
		Iter++;
	}

	return os;
}

istream& operator >> (istream& is, Regelbasis& eineRegelbasis) {
	eineRegelbasis.Speicher_freigeben();
	eineRegelbasis._Regelliste.Flush();

	int Op;
	is >> Op;
	eineRegelbasis._Op=Op;

	int Zaehler;
	is >> Zaehler;

	while (Zaehler) {
		Regel temp(	eineRegelbasis._EingabeVarListe,
						eineRegelbasis._AusgabeVar);
		is >> temp;
		eineRegelbasis.Regel_hinzufuegen(temp);
		--Zaehler;
	}

	return is;
}


void Regelbasis::PraemissenContainer_freigeben() {

	// gibt jede Regel der Prmissencontainers frei und lscht diese
	// abschlieend

	// Prmissencontainer = Listen von Regellisten

	TPraemissenContainerIter ContainerIter(_PraemissenContainer);

	// alle Container betrachten
	while (ContainerIter) {

		// Container holen
		TRegellisteIter Iter2(*ContainerIter.Current());

		// alle Regeln (=Elemente) dieses Containers lschen
		while (Iter2) {
			Regel* zuLoeschen=Iter2.Current();
			Iter2++;

			delete zuLoeschen;
		}
		// abschlieend diesen Container (=Regelliste) lschen
		ContainerIter.Current()->Flush();
		TRegelliste* zuLoeschen=ContainerIter.Current();
		ContainerIter++;
		delete zuLoeschen;

	}
	_PraemissenContainer.Flush();
}

void Regelbasis::alle_Regeln_loeschen() {

	// lscht alle Regeln der Regelbasis und gibt sie frei

	TRegellisteIter Iter(_Regelliste);

	while (Iter) {

		delete Iter.Current();
		Iter++;
	}
	_Regelliste.Flush();
}
void Regelbasis::Speicher_freigeben() {

	// gibt den gesamten Speicher der Regelbasis frei
	alle_Regeln_loeschen();
	PraemissenContainer_freigeben();

}

Regelbasis::~Regelbasis() {
	Speicher_freigeben();
}


void Regelbasis::Backtrack(TEingabeVarlisteIter& Iter,Regel& tempRegel) {

	#ifdef DEBUG_NAUCK_PHASE_1
		ofstream of("phase1.dmp");
	#endif

	// Naucks Regellernen :

	// erzeugt mittels Backtracking alle mglichen Permutationen der
	// Fuzzy-Mengen und fgt diese in Prmissencontainer mit gleichen
	// Prmissen ein

	if (Iter) { // es ex. noch weitere EingabeVariable

		IteratorType EingabeSetsIter(*Iter.Current()->FuzzySets_holen());
		TEingabeVarlisteIter Iter2=Iter;

		// alle verwendbaren Fuzzy-Mengen zur Konstruktion der Prmissen heranziehen,
		// und mit nchster Eingabevariable weiter backtracken
		Iter2++;
		while (EingabeSetsIter) {

			if (EingabeSetsIter.Current().Value()->_fuer_Regel_verfuegbar) {
				Regel neueRegel=tempRegel;
				neueRegel.Praemisse_hinzufuegen(EingabeSetsIter.Current().Value()->Name());

				Backtrack(Iter2,neueRegel);
			}
			EingabeSetsIter++;
		}
	}
	else { // Praemissse komplett -> alle mgl. Konklusionen erzeugen
		IteratorType AusgabeSetsIter(*_AusgabeVar->FuzzySets_holen());

		// neuen Prmissencontainer anlegen, da die Prmisse jetzt komplett ist
		TRegelliste* neueListe=new TRegelliste();
		_PraemissenContainer.append(neueListe);

		// alle Fuzzy-Mengen der Ausgabevariablen betrachten
		while (AusgabeSetsIter) {

			// neue Regel erzeugen
			Regel* neueRegel = new Regel(_EingabeVarListe,_AusgabeVar);
			*neueRegel=tempRegel;

			// Konklusion setzen
			neueRegel->Konklusion_setzen(AusgabeSetsIter.Current().Value()->Name());

			neueRegel->Auswertung_vorbereiten();
			// und in den Prmissencontainer einfgen
			neueListe->append(neueRegel);

			AusgabeSetsIter++;
		}
	}
}

void Regelbasis::PraemissenContainer_erzeugen() {

	// initiiert die Erstellung der Prmissencontainer

	if (_RegelLernVerfahren==Nauck) {

		Regel tempRegel(_EingabeVarListe,_AusgabeVar);

		TEingabeVarlisteIter Iter(*_EingabeVarListe);
		Backtrack(Iter,tempRegel);

		#ifdef DEBUG_CONTAINER

			// DEBUG: alle erstellten Prmissencontainer in eine Datei ausgeben
			ofstream of("rules.dmp");
			TPraemissenContainerIter Iter2(_PraemissenContainer);
			int count=1;
			while (Iter2) {
				of << "Container " << count++ << endl;
				TRegellisteIter Iter3(*Iter2.Current());
				while (Iter3) {
					of << Iter3.Current()->Regel_als_String() << endl;
					Iter3++;
				}
				Iter2++;
				of << endl;
			}
			of.close();
		#endif
	}
}

void Regelbasis::Regeln_lernen_I(FTyp& Fehler) {

	// koordiniert die Anwendung der 1. Phase des Regellernens

	// Verfahren von Stellmach verwenden ?
	if (_RegelLernVerfahren==Stellmach) {
		Regeln_lernen_Stellmach(Fehler);
		return;
	}
	else {

		// Nauck Phase I
		#ifdef DEBUG_NAUCK_PHASE_1
			ofstream of("phase1.dmp", ios::app);
			of.setf(ios::left);
			of.setf(ios::fixed);
			of.precision(10);
			of << endl << "NEUE EPOCHE :" << endl;
			of << "Fehler: Regel:" << endl;
		#endif

		// alle Prmissencontainer betrachten
		FTyp Test;
		TPraemissenContainerIter Iter(_PraemissenContainer);
		while (Iter) {
			//of << "----------" << endl;
			TRegellisteIter Iter2(*Iter.Current());

			while (Iter2) {
				// Regel des aktuellen Containers holen
				Regel* aktRegel=Iter2.Current();

				// falls letzter Erflltheitsgrad > 0, Cr hochsetzen
				if (aktRegel->hol_letzten_Erfuelltheitsgrad()>0.0) {
					aktRegel->_Cr+=1;

					// produziert sie einen sinnvollen Stellwert ?
					Test=(Fehler * (aktRegel->hol_Stellwert()));

					FTyp Fehler_der_Regel = fabs(Fehler) * Iter2.Current()->hol_letzten_Erfuelltheitsgrad();
					if (Test < 0)
						Fehler_der_Regel *= -1;


					#ifdef DEBUG_NAUCK_PHASE_1
						of << Fehler_der_Regel
							<< " " << Iter2.Current()->Regel_als_String();
					#endif

					if (Test<0) {
						// nein, also verwerfen
						#ifdef DEBUG_NAUCK_PHASE_1
							of << " -> entfernen" << endl;
						#endif
						Iter2++;
						Iter.Current()->Detach(aktRegel);
						delete aktRegel;

						// nchste Regel

					}
					else
						#ifdef DEBUG_NAUCK_PHASE_1
							of << endl ;
						#endif
						// ja, nchste Regel
						Iter2++;

				}
				else
					Iter2++;
			}

			Iter++;
		}
		#ifdef DEBUG_NAUCK_PHASE_1
			of.close();
		#endif
	}

}

void Regelbasis::Regeln_lernen_II(FTyp& Fehler) {

	// Anwendung der 2. Phase des Regellernens (Nauck)

	// alle Prmissencontainer betrachten
	TPraemissenContainerIter Iter(_PraemissenContainer);
	while (Iter) {

		TRegellisteIter Iter2(*Iter.Current());

		// Index der Regel aus dem Prmissencontainer auswhlen, die im nchsten
		// Zyklus zum Stellwert beitragen darf

		int Anzahl_Regeln=Iter.Current()->GetItemsInContainer();
		int index=random(Anzahl_Regeln),count=0;

		while (Iter2) {

			// Regel holen
			Regel* aktRegel=Iter2.Current();

			// war sie ausgewhlt ?
			if (aktRegel->_ausgewaehlt) {

				// hatte sie einen Erflltheitsgrad > 0.0 ?
				if (aktRegel->hol_letzten_Erfuelltheitsgrad()>0.0) {
					++aktRegel->_Cr;

					// Fehlersignal addieren zum Zhler Zr addieren
					aktRegel->_Zr+=(fabs(Fehler) * aktRegel->hol_letzten_Erfuelltheitsgrad());

				}
			}

			// bestimmen, ob sie im nchsten Zyklus ausgewhlt ist
			aktRegel->_ausgewaehlt=(index==count);

			++count;
			Iter2++;
		}

		Iter++;
	}
}

FTyp& Regelbasis::Stellwert_des_PraemissenContainers_berechnen(bool Phase_I) {

	// berechnet den Stellwert der Prmissencontainer (Nauck)

	_Ergebnis=_ErgebnisGrad=_Ausgabe=_alleGrade=0;

	// alle Prmissencontainer betrachten
	TPraemissenContainerIter Iter(_PraemissenContainer);

	if (Phase_I) {

		// bei der 1. Phase alle Regeln dieses Containers auswerten

		while (Iter) {
			TRegellisteIter Iter2(*Iter.Current());

			while (Iter2) {
				_Ergebnis=Iter2.Current()->Regel_auswerten(_ErgebnisGrad);
				// fr Defuzzyfizierung
				_Ausgabe+=_ErgebnisGrad*_Ergebnis;
				_alleGrade+=_ErgebnisGrad;
				Iter2++;
			}
			Iter++;
		}
	}
	else {

		// bei der 2. Phase nur ausgewhlte Regeln auswerten

		Regel* aktRegel;

		while (Iter) {
			TRegellisteIter Iter2(*Iter.Current());

			while (Iter2) {

				aktRegel=Iter2.Current();
				if (aktRegel->_ausgewaehlt) {

					_Ergebnis=aktRegel->Regel_auswerten(_ErgebnisGrad);

					// fr Defuzzyfizierung
					_Ausgabe+=_ErgebnisGrad*_Ergebnis;
					_alleGrade+=_ErgebnisGrad;

				}
				Iter2++;
			}
			Iter++;
		}
	}

	// Stellwert berechnen
	if (_alleGrade!=0)
		_Steuerwert=_Ausgabe/_alleGrade;
	else
		 _Steuerwert=0;

	// Ausgabevariable mit dem neuen Stellwert initialisieren
	_AusgabeVar->_Ausgabewert=_Steuerwert;

	return _Steuerwert;
}

void Regelbasis::neue_Regelbasis_uebernehmen(FTyp& AktivierungsSchwelle) {

	if (_RegelLernVerfahren==Nauck) {

		#ifdef DEBUG_NAUCK_PHASE_2
			ofstream of("phase2.dmp");
			of.setf(ios::left);
			of.setf(ios::fixed);
			of.precision(10);

		#endif

		// bei Naucks Lernverfahren wird nach dem Ende des Lernverfahrens
		// dann jeweils eine geeignete Regel aus einem Praemissencontainer
		// in die Regelbasis kopiert, wenn sie die Mindesaktivierung
		// berschreitet


		// alle Prmissencontainer betrachten
		TPraemissenContainerIter Iter(_PraemissenContainer);

		while (Iter) {
			#ifdef DEBUG_NAUCK_PHASE_2
				of << "Cr: Zr: Regel:" << endl;
			#endif
			TRegellisteIter Iter2(*Iter.Current());
			Regel* beste=0; // bisher "beste" Regel in diesem Container

			while (Iter2) {

				if (beste==0) {

					// falls noch keine geeignete Regel gefunden, nur
					// Aktivierungsschwelle vergleichen

					if (Iter2.Current()->_Cr > AktivierungsSchwelle)
						// gengend gro, Regel ist geeignet
						beste=Iter2.Current();
				}
				else {
					// falls summierter Fehler Zr dieser Regel kleiner ist
					// und Aktivierungsschwelle erreicht ist, dann ist diese
					// Regel geeigneter

					if ( (Iter2.Current()->_Zr < beste->_Zr) &&
						(Iter2.Current()->_Cr > AktivierungsSchwelle))
							beste=Iter2.Current();
				}
				#ifdef DEBUG_NAUCK_PHASE_2
					of.width(4);
					of << Iter2.Current()->_Cr ;
					of.width(0);
					of << " " ;
					if (Iter2.Current()->_Zr >= 0.0)
						of << " " ;
					of.width(12);
					of << Iter2.Current()->_Zr ;
					of.width(0);
					of << " " << Iter2.Current()->Regel_als_String() << endl;
				#endif

				// nchste Regel des Prmissencontainers betrachten
				Iter2++;
			}
			#ifdef DEBUG_NAUCK_PHASE_2
				of << endl;
				of << "ausgewhlt wurde : " ;
			#endif

			// falls geeignete Regel gefunden, deren Prmisse nicht schon in
			// der Regelbasis definiert wurde, diese in die Regelbasis aufnehmen

			if (beste) {
				if (!enthaelt_Regel_mit_gleicher_Praemisse(*beste))
					Regel_hinzufuegen(*beste);

				#ifdef DEBUG_NAUCK_PHASE_2
					of << beste->Regel_als_String() ;
				#endif
			}
			else {
				#ifdef DEBUG_NAUCK_PHASE_2
					of << "keine.";
				#endif
			}
			#ifdef DEBUG_NAUCK_PHASE_2
				of << endl << endl;
			#endif

			// nchsten Prmissencontainer betrachten
			Iter++;
		}
		#ifdef DEBUG_NAUCK_PHASE_2
			of.close();
		#endif

		// alle Prmissencontainer wurden ausgewertet, also lschen
		PraemissenContainer_freigeben();
	}
}


void Regelbasis::sortieren() {

	// sortiert die Regelbasis alphabetisch fr die Darstellung im Listenfenster

	bool sortiert=false;

	while (!sortiert) {

		sortiert=true;

		// durch die Regelliste laufen und vergleichen
		TRegellisteIter Iter(_Regelliste);
		while(Iter) {
			TRegellisteIter Iter2=Iter;
			Iter2++;
			if (Iter2) {
				// falls ntig, zwei Eintrge vertauschen
				if (*Iter.Current() > *Iter2.Current()) {
					Regel tempRegel=*Iter.Current();
					*Iter.Current()=*Iter2.Current();
					*Iter2.Current()=tempRegel;
					sortiert=false;
				}
			}
			Iter++;
		}
	}
}
