#include "regeldlg.h"

DEFINE_RESPONSE_TABLE1(TRegelDialog, TDialog)
	EV_CHILD_NOTIFY(IDCANCEL, BN_CLICKED, Abbruch),
	EV_CHILD_NOTIFY(IDOK, BN_CLICKED, Uebernehmen),
END_RESPONSE_TABLE;

TRegelDialog::TRegelDialog(TDialog* parent,Regelbasis& Basis,Regel* eineRegel)
	: TDialog (parent,DIALOG_REGEL) , _Basis(Basis), _Regel(eineRegel) {

	// Anzahl der Eingabevariablen besorgen
	_Anzahl_Eingabevar=Basis.hol_EingabeVarListe()->GetItemsInContainer();

	// Dialogelemente initialisieren
	for (int i=0; i < 6; ++i) {
		_Titel[i]= new TEdit(this,i+100,16);
		_VarName[i]= new TEdit(this,i+200,16);
		_Is[i]= new TEdit(this,i+300,3);
		_Set[i]= new TComboBox(this,i+400,16);
	}
}
TRegelDialog::~TRegelDialog() {

	// Dialogelemente freigeben

	for (int i=0; i < 6; ++i) {
		delete _Titel[i];
		delete _VarName[i];
		delete _Is[i];
		delete _Set[i];
	}
}
void TRegelDialog::ComboBox_initialisieren(Variable& Var,TComboBox& Box){

	// initialisiert den Inhalt einer Combobox mit den Namen
	// aller Fuzzy-Mengen der Variable

	IteratorType Iter(*Var.FuzzySets_holen());
	while (Iter) {
		Box.AddString(Iter.Current().Key()->c_str());
		Iter++;
	}
}
bool TRegelDialog::ueberpruefe_FuzzySets() {

	// bergibt true, falls fr jede Variable mind. eine Fuzzy-Menge
	// definiert ist.

	TEingabeVarlisteIter CheckIter(*_Basis.hol_EingabeVarListe());

	bool ok(true);

	while (CheckIter && ok) {
		if ((CheckIter.Current()->FuzzySets_holen())->GetItemsInContainer()==0)
			ok=false;
		else
			CheckIter++;
	}
	if (ok) {
		if (_Basis.hol_AusgabeVariable()->FuzzySets_holen()->GetItemsInContainer()==0)
			ok=false;
	}
	return ok;
}

void TRegelDialog::initialisiere_Sets() {

	// initialisiert die Dialogelemente mit dem Inhalt einer Regel

	// ist die Regel bereits definiert ?

	if (_Regel) {
			// ja,
			TStringlisteIter Iter(_Regel->Praemisse_holen());
			int i=0;

			while (Iter) {

				// Verknpfung schreiben
				if (i)
					_Titel[i]->SetText("and");
				else
					_Titel[i]->SetText("If");

				// Name der Fuzzy-Menge schreiben
				_Set[i]->SetText(Iter.Current().c_str());
				Iter++;
				++i;
			}
			_Titel[i]->SetText("then");

			// Name der Fuzzy-Menge in der Konklusion schreiben
			_Set[i]->SetText(_Regel->Konklusion_holen().c_str());
			SetCaption("Rule edit");
		}

	else {
		// nein, also einen sinnvollen Vorschlag fr eine Regel basteln
		TEingabeVarlisteIter Iter(*_Basis.hol_EingabeVarListe());
		int i=0;
		while (Iter) {

			// s.o.
			if (i)
				_Titel[i]->SetText("and");
			else
				_Titel[i]->SetText("If");

			IteratorType Iter1(*Iter.Current()->FuzzySets_holen());

			// Namen der ersten verfgbaren Fuzzy-Menge der aktuellen
			// Eingabevariablen nehmen
			_Set[i]->SetText(Iter1.Current().Key()->c_str());

			Iter++;
			++i;
		}

		_Titel[i]->SetText("then");

		// erstbeste Fuzzy-Menge der Ausgabevariablen nehmen
		IteratorType Iter3(*_Basis.hol_AusgabeVariable()->FuzzySets_holen());
		_Set[i]->SetText(Iter3.Current().Key()->c_str());
		SetCaption("New rule");
	}
}

void TRegelDialog::initialisiere_VarNamen() {

	// initialisiert die Dialogelemente mit den Namen  der Variablen
	// und speichert deren verfgbare Fuzzy-Mengen in der Combobox

	TEingabeVarlisteIter Iter(*_Basis.hol_EingabeVarListe());

	int i=0;
	while (Iter) {
		_VarName[i]->SetText(Iter.Current()->Name().c_str());
		ComboBox_initialisieren(*Iter.Current(),*_Set[i]);
		++i;
		Iter++;
	}
	_VarName[i]->SetText(_Basis.hol_AusgabeVariable()->Name().c_str());
	ComboBox_initialisieren(*_Basis.hol_AusgabeVariable(),*_Set[i]);

}
void TRegelDialog::SetupWindow() {


	TDialog::SetupWindow();
	TRect dlgRect=GetWindowRect();

	// geeignete Gre des Fensters whlen
	MoveWindow(dlgRect.left,dlgRect.right,
				max(250,(_Anzahl_Eingabevar)*80+100),dlgRect.Height(),TRUE);

	// ist auch fr jede Variable mindestens eine Fuzzy-Menge definiert ?
	if (!ueberpruefe_FuzzySets()) {
		MessageBox("Error: Rules can't be defined, if there are no fuzzy-sets in one variable.",
						"Rule edit",MB_ICONSTOP|MB_OK);
		CloseWindow();
	}
	else {

		// nicht verwendete Dialogelemente ausblenden
		for (int i=_Anzahl_Eingabevar+1; i<6 ; ++i) {
			_Titel[i]->Show(SW_HIDE);
			_VarName[i]->Show(SW_HIDE);
			_Is[i]->Show(SW_HIDE);
			_Set[i]->Show(SW_HIDE);
		}

		initialisiere_VarNamen();
		initialisiere_Sets();

		// Dialog zentrieren
		TRect parentRect;
		Parent->GetWindowRect(parentRect);

		TRect dlgRect;
		GetWindowRect(dlgRect);

		int xPos = parentRect.Width() / 2- dlgRect.Width() / 2 + parentRect.left;
		int yPos = parentRect.Height() / 2- dlgRect.Height() / 2 + parentRect.top;

		MoveWindow(xPos,yPos,dlgRect.Width(), dlgRect.Height());

	}
}

void TRegelDialog::Abbruch() {
	CloseWindow();
}
void TRegelDialog::Uebernehmen() {

	// um die Regel aus der Dialogbox zu bernehmen, wir eine neue Regel
	// erstellt, die mit deren Fuzzy-Mengen initialisiert wird.


	// neue temporre Regel
	Regel* tempRegel=new Regel(_Basis.hol_EingabeVarListe(),
									  _Basis.hol_AusgabeVariable());
	char puffer[30];
	mystring einString;
	int i=0;

	// Prmisse auslesen
	while (i<_Anzahl_Eingabevar) {
		_Set[i]->GetText((char *)&puffer,30);
		einString=puffer;
		tempRegel->Praemisse_hinzufuegen(einString);
		++i;
	}

	// Konklusion setzen
	_Set[i]->GetText((char *)&puffer,30);
	einString=puffer;
	tempRegel->Konklusion_setzen(einString);

	// falls eine bestehende Regel mit der Dialogbox editiert wurde, wird
	// diese aus der Regelbasis gelscht und statt dessem die neue eingefgt
	if (_Regel) {
			CloseWindow();
			_Basis.entferne_Regel(_Regel);
			_Basis.Regel_hinzufuegen(*tempRegel);
	}
	else {

		// existiert bereits eine Regel mit identischem Inhalt ?
		if (_Basis.enthaelt_Regel(*tempRegel)) {
			MessageBox("Error: This rule is already existing.",
							"New rule",MB_ICONSTOP|MB_OK);
			delete tempRegel;
		}
		else {
			// ansonsten regulr einfgen
			CloseWindow();
			_Basis.Regel_hinzufuegen(*tempRegel);
		}
	}
}
