#include "regeledt.h"

DEFINE_RESPONSE_TABLE1(TRegelEdit,TMDIChild)
	EV_WM_SIZE,
	EV_WM_GETMINMAXINFO,
END_RESPONSE_TABLE;

DEFINE_RESPONSE_TABLE1(BasisDialog,TDialog)
	EV_CHILD_NOTIFY(IDC_NEU, BN_CLICKED, Regel_neu),
	EV_CHILD_NOTIFY(IDC_LOESCHEN, BN_CLICKED, Regel_loeschen),
	EV_CHILD_NOTIFY(IDC_AND, CBN_SELENDOK, Auswahl),
	EV_CHILD_NOTIFY(IDC_REGELBOX, LBN_DBLCLK, RegelAuswahl),
	EV_WM_SIZE,
	EV_WM_GETMINMAXINFO,
END_RESPONSE_TABLE;
//------------------------------------------------------------------------------

// Basisdialog enthlt die eigentlichen Dialogelemente

//------------------------------------------------------------------------------
BasisDialog::BasisDialog(TMDIChild* parent, TResId resId,  Regelbasis& Basis) :
	TDialog(parent,resId), _Basis(Basis), _Liste(Basis.hol_Regelliste()) {

	// Dialogelemente initialisieren
	Liste = new TListBox(this, IDC_REGELBOX);
	OpBox = new TComboBox(this, IDC_AND,16);
}
BasisDialog::~BasisDialog() {
	delete Liste;
	delete OpBox;
}
void BasisDialog::Regel_neu() {
	// Dialog zum editieren einer einzelnen Regel aufrufen
	TRegelDialog(this,_Basis,0).Execute();

	// Liste der definierten Regeln im Fenster erneuern
	Liste_erneuern();
}
Regel* BasisDialog::Regel_von_Index(int i) {

	// holt die Regel mit dem Index i aus der Liste
	TRegellisteIter Iter(_Liste);

	while (Iter && i) {
		--i;
		Iter++;
	}
	if (Iter)
		return (Iter.Current());
	else
		return 0;

}
void BasisDialog::Regel_loeschen() {

	// lscht die markierte Regel in der Liste

	// welcher Eintrag ist markiert ?
	int Auswahl=Liste->GetSelIndex();

	// keiner ?
	if (Auswahl<0)
		MessageBox("Error: Choose rule before deleting!","Delete a rule",
						MB_ICONSTOP|MB_OK);
	else {
		// Regel aus der Regelbasis entfernen und Liste erneuern
		_Basis.entferne_Regel(Regel_von_Index(Auswahl));
		Liste_erneuern();
	}
}
void BasisDialog::Liste_erneuern() {

	// Liste sortiert halten (ist bersichtlicher)
	_Basis.sortieren();

	// alte Liste verwerfen
	Liste->ClearList();
	TRegellisteIter Iter(_Liste);

	// alle Regeln als Strings in die Liste einfgen
	while (Iter) {
		Liste->AddString(Iter.Current()->Regel_als_String().c_str());
		Iter++;
	}

	// verwendete t-Norm schreiben
	switch (_Basis.Operator_holen())  {
		case (Minimum) : {
			OpBox->SetText("Minimum");
			break;
		}
		case (HarmonicMean) : {
			OpBox->SetText("harm. Mean");
			break;
		}
		case (Product) : {
			OpBox->SetText("Product");
			break;
		}
	}
}
void BasisDialog::SetupWindow() {
	TDialog::SetupWindow();

	// Auswahlmglichkeiten fr die t-Norm initialisieren
	OpBox->AddString("Minimum");
	OpBox->AddString("harm. Mean");
	OpBox->AddString("Product");

	// Liste der Regeln im Fenster erstellen
	Liste_erneuern();
}

void BasisDialog::Auswahl() {

	// aktualisiert die Auswahl der t-Norm, falls diese vom Benutzer gendert
	// wurde

	char Wert[30];
	OpBox->GetText((char *)Wert,30);

	mystring Eingabe(Wert);

	if (Eingabe=="Minimum") {
		_Basis.Operator_setzen(Minimum);
	}
	if (Eingabe=="harm. Mean") {
		_Basis.Operator_setzen(HarmonicMean);
	}
	if (Eingabe=="Product") {
		_Basis.Operator_setzen(Product);
	}
}

void BasisDialog::RegelAuswahl() {
	// eine Regel wurde ausgewhlt, also Regeleditor fr diese anzeigen
	TRegelDialog(this,_Basis,Regel_von_Index(Liste->GetSelIndex())).Execute();
	Liste_erneuern();
}
//------------------------------------------------------------------------------

// Regeledit ist ein bergeordnetes Fenster das die eigentliche Dialogbox
// enthlt. Es sorgt dafr, da bei der nderung der Fenstergre die Gre
// des eingebetteten Listenfensters sinnvoll angepat wird.

//------------------------------------------------------------------------------
TRegelEdit::TRegelEdit(TMDIClient& parent, BasisDialog* DialogBox,
					Regelbasis& Basis,
					mystring Name) :
	TMDIChild(parent,"",DialogBox, TRUE), _Basis(Basis) {
	mystring Titel = "Rule-base " + Name;
	SetCaption(Titel.c_str());
}
void TRegelEdit::aktualisieren() {
	_Dialog->Liste_erneuern();
}
void TRegelEdit::SetupWindow() {

	TMDIChild::SetupWindow();

	// eigebettetes Dialogfenster erzeugen
	_Dialog = new BasisDialog(this ,DIALOG_REGELBASIS, _Basis);
	SetClientWindow(_Dialog);

	// minimale Hhe des Fensters ermitteln
	TRect rect = GetWindowRect();
	minHoehe = rect.Height();

	SetIcon(GetApplication(),ICON_SCHREIBE);
}

void TRegelEdit::EvSize(UINT sizeType, TSize &size) {

	// Gre des Fensters wurde gendert

	// bisherige Gre der Dialogbox aufheben
	BasisDialog* dlg= TYPESAFE_DOWNCAST(GetClientWindow(), BasisDialog);
	TRect oldRect = dlg->GetWindowRect();

	// Grennderung durchfhren
	TMDIChild::EvSize(sizeType, size);

	// wurde das Fenster auf Symbolgre verkleinert ?
	if (sizeType != SIZE_MINIMIZED) {

		// nein, also Gre des Listenfensters anpassen
		TRect neu_rect = dlg->GetWindowRect();

		// nderungen ermitteln
		int xChange = neu_rect.Width() - oldRect.Width();
		int yChange = neu_rect.Height() - oldRect.Height();
		TRect rect = dlg->Liste->GetWindowRect();
		rect.right+=xChange;
		rect.bottom+=yChange;

		// Grennderung der Liste durchfhren
		dlg->Liste->SetWindowPos(0,
										rect,
										SWP_NOZORDER | SWP_NOMOVE);
	}
}
void TRegelEdit::EvGetMinMaxInfo(MINMAXINFO  &minmaxinfo) {
	TMDIChild::EvGetMinMaxInfo(minmaxinfo);
}
bool TRegelEdit::CanClose() {
	// teilt dem MDI-Klient das Schlieen des Regeleditors mit.
	Parent->SendMessage(WM_USER,CLOSE_REGELEDIT ,(long)this);
	return true;
}

TRegelEdit::~TRegelEdit() {
	// falls Dialog vorhanden, diesen freigeben
	if (_Dialog)
		delete _Dialog;
}

