#include "setedit.h"

DEFINE_RESPONSE_TABLE1(TSetEdit,TMDIChild)
	EV_WM_RBUTTONDBLCLK,
	EV_WM_RBUTTONDOWN,
	EV_WM_LBUTTONUP,
	EV_WM_RBUTTONUP,
	EV_WM_MOUSEMOVE,
	EV_WM_SIZE,
	EV_COMMAND(CM_PFEIGENSCHAFTEN, CmPFEigenschaften),

	EV_COMMAND(CM_PFLOESCHEN, CmPFLoeschen),
	EV_COMMAND(CM_PFHINZUFUEGEN, CmPFHinzufuegen),
	EV_COMMAND(CM_PFCLIPBOARD, CmPFClipboard),

	EV_COMMAND(CM_PPARTITION, CmPPartition),
END_RESPONSE_TABLE;

TSetEdit::TSetEdit(TMDIClient& parent,
	FController* Controller, mystring& Name, bool ist_AusgabeVar,
	bool ist_FehlerVar) :
		TMDIChild(parent,""), _Controller(Controller), _Name(Name),
		_ist_AusgabeVar(ist_AusgabeVar),	_ist_FehlerVar(ist_FehlerVar) {

	initialisieren();
}

TSetEdit::TSetEdit(TMDIClient& parent,
					Variable* Var,
					bool ist_AusgabeVar,bool ist_FehlerVar) :

	TMDIChild(parent,""), _Var(Var),
	_ist_AusgabeVar(ist_AusgabeVar),	_ist_FehlerVar(ist_FehlerVar) {

	// Fuzzy-Mengen holen
	_FuzzySets = _Var->FuzzySets_holen();

	// keine Fuzzy-Menge ausgewhlt
	_InfoSet=_Set=0;

	// Definitionsbereich holen
	_min=_Var->min_holen();
	_max=_Var->max_holen();

}


void TSetEdit::SetupWindow() {

	TMDIChild::SetupWindow();
	TRect parentRect;
	Parent->GetWindowRect(parentRect);
	Titel();
	_Schrift= new TFont("Arial",14);
	SetIcon(GetApplication(),ICON_MOUSE);

	MoveWindow(30,30,400,150);

}

void TSetEdit::initialisieren() {

	// ermittelt die zu editierende Variable ("Name") des Controllers

	bool gefunden(false);

	// soll die Ausgabevariable editiert werden ?
	if (_ist_AusgabeVar){
		if (_Controller->AusgabeVar_holen()==_Name) {
			_Var=&_Controller->_AusgabeVar;
			_FuzzySets = _Var->FuzzySets_holen();
		}
	}
	else {
		// soll eine Eingabevariable editiert werden ?
		if (!_ist_FehlerVar) {

			// in der Liste der Eingabevariablen nach dem Namen suchen
			TEingabeVarliste& Eingaben= _Controller->_EingabeVarListe;
			TEingabeVarlisteIter Iter(Eingaben);
			while (Iter && !gefunden) {

				_Var=Iter.Current();

				if (_Var->Name()==_Name) {
					gefunden=true;
					_FuzzySets = _Var->FuzzySets_holen();
				}
				if (!gefunden)
				Iter++;
			}
		}
		else {
			// also Fehlervariable editieren
			if (_Controller->FehlerVar_holen()==_Name) {
				_Var=&_Controller->_FehlerVar;
				_FuzzySets = _Var->FuzzySets_holen();
			}
		}
	}
	// keine Fuzzy-Menge ausgewhlt bisher
	_InfoSet=_Set=0;

	// Definitionsbereich
	_min=_Var->min_holen();
	_max=_Var->max_holen();
}

bool TSetEdit::CanClose() {
	// dem MDI-Clienten mitteilen, da das Fenster geschlossen wird
	Parent->SendMessage(WM_USER,CLOSE_SETEDIT ,(long)this);
	return true;
}

TRect TSetEdit::berechne_Bildviereck(TRect& groesse) {

	// Rnder bestimmen

	TRect bild=groesse;

	bild.left+=RAND_LINKS;
	bild.right-=RAND_RECHTS;
	bild.top+=RAND_OBEN;
	bild.bottom-=RAND_UNTEN;

	return bild;
}

int TSetEdit::ist_innerhalb(TRect& bild, TPoint punkt) {

	// ist der Punkt innerhalb des Bildrechteckts + Fangradius ?

	return (punkt.x  >= (bild.left -FANGRADIUS) &&
				punkt.x <= (bild.right+FANGRADIUS) &&
				punkt.y >= (bild.top-FANG_Y) &&
				punkt.y <= (bild.bottom+FANG_Y));
}
int TSetEdit::ist_oben(TRect& bild, TPoint& punkt) {

	// ist der Punkt am oberen Fensterrand ?
	return ((punkt.y - bild.top) < FANGRADIUS);
}

int TSetEdit::ist_unten(TRect& bild, TPoint& punkt) {

	// ist der Punkt am unteren Fensterrand ?

	return ((bild.bottom - punkt.y) < FANGRADIUS);
}
void  TSetEdit::ueberpruefe_Ecke(const FsSet* Set ,FTyp& x,FTyp& y,
										FTyp& beste_Distanz, FTyp& x1,FTyp& x2) {

	// ermittelt die Distanz Punktes zur Abbildung der Fuzzy-Menge

	FTyp Distanz;

	if (x> x1 && x< x2) {
		Distanz=berechne_Distanz(Set,x,y);
		if (Distanz < beste_Distanz || beste_Distanz== -1)
			beste_Distanz=Distanz;
	}

}
FTyp TSetEdit::berechne_Distanz(const FsSet* Set ,FTyp& x,FTyp& y) {

	// ermittelt die Distanz des Punktes zum y-Wert der Fuzzy-Menge beim x-Wert

	if (Set->y_wert(x)>y)
		return Set->y_wert(x)-y;
	else
		return y-Set->y_wert(x);
}

FTyp TSetEdit::Distanz(TDC& dc,TRect& bild, TPoint punkt,const FsSet* Set) {

	FTyp Distanz,beste_Distanz(-1);
	bool gefunden(false);

	// die dem Bildpunkt im Fenster entsprechende Koordinate im
	// Definitionsbereich bestimmen + Toleranz

	FTyp x1= _min + (punkt.x - bild.left - FANGRADIUS)
				* (_max-_min)/(bild.right - bild.left);
	FTyp x2= _min + (punkt.x - bild.left + FANGRADIUS)
				* (_max-_min)/(bild.right - bild.left);

	FTyp y= (FTyp) (bild.bottom - punkt.y) / (FTyp) (bild.bottom - bild.top);


	// berschneidet sich der Trger der betrachtete Fuzzy-Menge mit
	// dem Intervall [x1,x2] ?

	if (Set->enthaelt_Teilmenge_von(x1,x2)) {

		// falls ja, minimaler Abstand innerhalb des Fangradius zur Fuzzy-Menge
		// bestimmen

		for (int xp=-FANGRADIUS; xp <=FANGRADIUS; xp++) {

			FTyp x= _min + (xp+ punkt.x - bild.left) * (_max-_min)/(bild.right - bild.left);

			if (Set->enthalten(x)) {

				if (!gefunden) {
					gefunden=true;
					beste_Distanz=berechne_Distanz(Set,x,y);
				}
				else {
					Distanz=berechne_Distanz(Set,x,y);
					if (Distanz < beste_Distanz)
						beste_Distanz=Distanz;
				}
			}
		}
	}

	// ueberpruefe Ecken gesondert

	if (Set->Anzahl_Intervalle()==2) {
		ueberpruefe_Ecke(Set,Set->x1(),y,beste_Distanz,x1,x2);
		ueberpruefe_Ecke(Set,Set->x2(),y,beste_Distanz,x1,x2);
		ueberpruefe_Ecke(Set,Set->x3(),y,beste_Distanz,x1,x2);
	}
	else {
		ueberpruefe_Ecke(Set,Set->x1(),y,beste_Distanz,x1,x2);
		ueberpruefe_Ecke(Set,Set->x2(),y,beste_Distanz,x1,x2);
	}

	if (beste_Distanz>0) {
		beste_Distanz*= (bild.bottom - bild.top);
	}

	return beste_Distanz;
}

const FsSet* TSetEdit::hol_selektiertes_Set(TDC& dc,TRect& bild, TPoint& punkt) {

	// holt die Fuzzy-Menge mit der geringsten Entfernung zum Punkt "punkt"

	IteratorType Iter(*_FuzzySets);
	const FsSet* bestes_bisher(0);
	FTyp  beste_Entfernung(0),Entfernung;
	bool erstes(true);

	// geringste Entfernung durch Betrachtung aller Fuzzy-Mengen ermitteln
	while (Iter) {

		const FsSet* value=Iter.Current().Value();

		Entfernung=Distanz(dc,bild,punkt,value);

		// Entfernung geringer als bisher beste ?
		if (Entfernung >=0 )
			if ((Entfernung < beste_Entfernung) || (bestes_bisher==0)) {
					beste_Entfernung=Entfernung;
					bestes_bisher=value;
			}

		Iter++;
	}
	// falls Entfernung innerhalb des Toleranzintervalls, dann Fuzzy-Menge
	// liefern
	if (beste_Entfernung <= FANG_Y && beste_Entfernung >0)
		return bestes_bisher;
	else
		// sonst keine Fuzzy-Menge zurckliefern
		return 0;
}

void TSetEdit::male_X_Achse(TDC& dc,TRect& bild, bool alles) {

	// malt die x-Achse unter Verwendung der Lineal-Klasse
	Lineal XAchse;
	XAchse.Grenzen_x(_min,_max);
	XAchse.Grenzen_y(0,1);
	XAchse.zeichne(dc,bild,true,alles);

}

void TSetEdit::male_Y_Achse(TDC& dc,TRect& bild,bool alles) {

	// malt die y-Achse unter Verwendung der Lineal-Klasse
	Lineal YAchse;
	YAchse.Grenzen_x(_min,_max);
	YAchse.Grenzen_y(0,1);
	YAchse.zeichne(dc,bild,false,alles,true);
}

void TSetEdit::EvSize(UINT sizeType, TSize& Size) {

	// bei nderungen der Fenstergre, neuzeichnen

	TMDIChild::EvSize(sizeType,Size);
	Invalidate(false);
}

void TSetEdit::Paint(TDC&, bool, TRect& ) {

	// zeichnet den Fuzzy-Mengen - Editor mit Koordinatenkreuzen und allen Mengen

	_min=_Var->min_holen();
	_max=_Var->max_holen();
	Titel();

	TRect groesse=GetClientRect();

	TClientDC mydc(*this);
	mydc.SelectObject(*_Schrift);

	mydc.SelectStockObject(WHITE_PEN);
	mydc.SelectStockObject(WHITE_BRUSH);
	mydc.Rectangle(groesse);
	mydc.SelectStockObject(BLACK_PEN);

	IteratorType Iter(*_FuzzySets);

	// Koordinatenkreuz
	TRect bild=berechne_Bildviereck(groesse);

	_Pixel_x= (long double) (_max-_min) / (long double) (bild.right - bild.left);

	mydc.SetTextColor(TColor::Black);
	male_X_Achse(mydc,bild);
	male_Y_Achse(mydc,bild);

	// alle Fuzzy-Mengen zeichen
	while (Iter) {
		const FsSet* value=Iter.Current().Value();
		value->malen(mydc,bild,_min,_max, TColor::Black);
		Iter++;
	}

	// falls eine Fuzzy-Menge selektiert, diese hellgrn zeichnen
	if (_InfoSet)
		_InfoSet->malen(mydc,bild,_min,_max, TColor::LtGreen);
}



TPoint TSetEdit::projiziere(TRect& bild, FTyp& x ,FTyp& y) {

	// projiziert den Punkt (x,y) in das Fenster

	TPoint punkt;
	punkt.x=bild.left+(x-_min)/(_max-_min)*(bild.right-bild.left);
	punkt.y=bild.bottom-(bild.bottom-bild.top) * y;
	return punkt;
}
void TSetEdit::Titel() {

	// der Standardtitel fr den Fuzzy-Mengen - Editor wird erzeugt

	mystring Titel="Fuzzy-sets " + _Var->Name();
	SetCaption(Titel.c_str());
}
void TSetEdit::Info(FsSet* Set) {

	// falls eine Fuzzy-Menge ausgewhlt wurde, dann zustzliche Informationen
	// im Titel des Fensters anzeigen

	mystring titel = Set->Name() + " [";
	char puffer[50];

	// Eckpunkte der Fuzzy-Menge anzeigen

	if (Set->Anzahl_Intervalle()==2) {
		sprintf((char*)puffer,"%.2f %.2f %.2f]",Set->x1(),Set->x2(),Set->x3());
	}
	else {
		sprintf((char*)puffer,"%.2f %.2f]",Set->x1(),Set->x2());
	}
	titel += puffer;
	SetCaption(titel.c_str());
}
void TSetEdit::EvRButtonDown(UINT, TPoint& punkt) {

	// mit dem rechten Mausknopf kann die Breite der Fuzzy-Menge verndert
	// oder die Menge verschoben werden

	TRect bild=berechne_Bildviereck(GetClientRect());
	TClientDC mydc(*this);
	mydc.SelectObject(*_Schrift);

	// ist der angeklickte Punkt berhaupt innerhalb des Fensterbereichs mit
	// den Fuzzy-Mengen ?

	if (ist_innerhalb(bild,punkt))
		// falls ja, gewhlte Fuzzy-Menge besorgen
		_Set= (FsSet*) hol_selektiertes_Set(mydc,bild,punkt);

	if (_Set) {
		// Information im Titel anzeigen
		Info(_Set);
		SetCapture();
		_neues_Set=*_Set;

		// Punkt merken, wo geklickt wurde
		_letzter_gueltiger_Punkt=_Startpunkt=punkt;


		if (_Set->Anzahl_Intervalle()==2) {

			// Typ Dreieck

			// Eckpunkt der Fuzzy-Menge bestimmen, der angeklickt wurde
			if (ist_unten(bild,punkt)) {
				if (projiziere(bild,_Set->Mitte(),0).x > punkt.x)
					_Modus=links;
				else
					_Modus=rechts;
			}
			else
				_Modus=mitte;
		}
		else {

			// Typ einzelnes Intervall

			// Eckpunkt der Fuzzy-Menge bestimmen, der angeklickt wurde
			if (ist_unten(bild,punkt)) {
				if (_Set->steigend())
					_Modus=links;
				else
					_Modus=rechts;
			}
			else {
				if (ist_oben(bild,punkt)) {
					if (_Set->steigend())
						_Modus=rechts;
					else
						_Modus=links;
				}
				else {
					_Modus=mitte;
				}
			}
		}

		if (_Modus==mitte)
			// hellrot fr Verschiebemodus zeichen
			_Set->malen(mydc,bild,_min,_max, TColor::LtRed);
		else
			// hellblau fr Verbreiterung zeichnen
			_Set->malen(mydc,bild,_min,_max, TColor::LtBlue);
	}
}

void TSetEdit::wiederherstellen(const FsSet* QuellSet,FsSet& Set,TDC& dc,
											TRect& bild) {

	// malt Achsen und Fuzzy-Mengen neu, die sich mit der Fuzzy-Menge "Set"
	// berschneiden

	dc.SelectObject(TPen(TColor::Black));
	dc.SetTextColor(TColor::Black);

	male_X_Achse(dc,bild,false);
	male_Y_Achse(dc,bild,false);

	IteratorType Iter(*_FuzzySets);

	FTyp x1,x2,x3,x4;

	Set.hole_Grenzen(x1,x2);

	// alle Fuzzy-Mengen betrachten
	while (Iter) {

		const FsSet* value=Iter.Current().Value();

		if (value!=QuellSet)

			// berscheidungen untereinander berpfen
			if (value->enthaelt_Teilmenge_von(x1,x2))
				value->malen(dc,bild,_min,_max, TColor::Black);
			else {
				value->hole_Grenzen(x3,x4);
				if (Set.enthaelt_Teilmenge_von(x3,x4))
					value->malen(dc,bild,_min,_max, TColor::Black);
			}
		Iter++;
	}
}

void TSetEdit::EvMouseMove(UINT, TPoint& punkt) {

	// der Mauszeiger wurde innerhalb des Fensters bewegt

	TClientDC mydc(*this);
	mydc.SelectObject(*_Schrift);
	TRect bild=berechne_Bildviereck(GetClientRect());

	// existiert eine angeklickte Fuzzy-Menge ?
	if (_Set) {

		// existiert ein Info-Set (hellgrn) ?
		if (_InfoSet) {
			// falls ja, dann diese schwarz zeichnen
			_InfoSet->malen(mydc,bild,_min,_max, TColor::Black);
			Titel();
			// diese Fuzzy-Menge knftig nicht mehr hellgrn anzeigen
			_InfoSet=0;
		}

		// wie weit wurde die Maus gegenber dem Startpunkt bewegt ?
		// daraus Verschiebungsbetrag der Fuzzy-Menge im Definitionsbereich
		// der Variablen ableiten
		FTyp delta_x= (punkt.x-_Startpunkt.x)* _Pixel_x;

		// falls auf den Eckpunkt x mit dem Zugehrigkeitsgrad (x)=1
		// geklickt wurde, dann wird die Fuzzy-Menge verschoben

		if (_Modus==mitte) {
			// verschiebbar ?
			if (_Set->verschiebbar(delta_x)) {

				// ja, bisher verndertes Set lschen
				_neues_Set.malen(mydc,bild,_min,_max, TColor::White);

				// ursprngliches Set grau malen
				_Set->malen(mydc,bild,_min,_max, TColor::Gray);

				// alle anderen Fuzzy-Mengen wiederherstellen
				wiederherstellen(_Set,_neues_Set,mydc,bild);

				// neues Set verschieben und neuzeichnen
				_neues_Set=*_Set;
				_neues_Set.verschieben(delta_x);
				_neues_Set.malen(mydc,bild,_min,_max, TColor::LtRed);

				_letzter_gueltiger_Punkt=punkt;
			}
			else
				// falls nicht verschiebbar, akustisches Signal
				MessageBeep(0);
		}
		else {

			if (_Modus==links) {

				// kann der linke Eckpunkt verschoben werden ?
				if (_Set->links_verschiebbar(delta_x)) {

					// ja, bisher verndertes Set lschen
					_neues_Set.malen(mydc,bild,_min,_max, TColor::White);

					// ursprngliches Set grau malen
					_Set->malen(mydc,bild,_min,_max, TColor::Gray);

					// alle anderen Fuzzy-Mengen wiederherstellen
					wiederherstellen(_Set,_neues_Set,mydc,bild);

					// Eckpunkt verschieben und neuzeichnen
					_neues_Set=*_Set;
					_neues_Set.verschiebe_links(delta_x);
					_neues_Set.malen(mydc,bild,_min,_max, TColor::LtBlue);
					_letzter_gueltiger_Punkt=punkt;
				}
				else
					// falls nicht verschiebbar, akustisches Signal
					MessageBeep(0);
			}
			else
				// kann der linke Eckpunkt verschoben werden ?
				if (_Set->rechts_verschiebbar(delta_x)) {

					// ja, bisher verndertes Set lschen
					_neues_Set.malen(mydc,bild,_min,_max, TColor::White);

					// ursprngliches Set grau malen
					_Set->malen(mydc,bild,_min,_max, TColor::Gray);

					// alle anderen Fuzzy-Mengen wiederherstellen
					wiederherstellen(_Set,_neues_Set,mydc,bild);

					// Eckpunkt verschieben und neuzeichnen
					_neues_Set=*_Set;
					_neues_Set.verschiebe_rechts(delta_x);
					_neues_Set.malen(mydc,bild,_min,_max, TColor::LtBlue);
					_letzter_gueltiger_Punkt=punkt;
				}
				else
					// falls nicht verschiebbar, akustisches Signal
					MessageBeep(0);
		}
		// neue Daten der vernderten Fuzzy-Menge im Titel anzeigen
		Info(&_neues_Set);
	}
	else {
		// es existiert kein angeklicktes Set

		// zeigt der Mauszeiger jetzt auf eine andere Fuzzy-Menge ?
		FsSet* Info_neu((FsSet*)hol_selektiertes_Set(mydc,bild,punkt));
		if (Info_neu!=_InfoSet) {

			if (_InfoSet)
				// ja, altes InfoSet wieder schwarz zeichnen
				_InfoSet->malen(mydc,bild,_min,_max, TColor::Black);
			else
				SetCapture();

			_InfoSet=Info_neu;

			if (_InfoSet) {
				// neues InfoSet hellgrn zeichnen und Information im Titel
				// erneuern
				 Info(_InfoSet);
				_InfoSet->malen(mydc,bild,_min,_max,TColor::LtGreen);
			}
			else {
				Titel();
				ReleaseCapture();
			}
		}
	}
}

void TSetEdit::EvRButtonUp(UINT, TPoint&) {

	// der links Mausknopf wurde wieder losgelassen,
	// ein Verschiebevorgang wird jetzt beendet

	TRect bild=berechne_Bildviereck(GetClientRect());
	TClientDC mydc(*this);
	mydc.SelectObject(*_Schrift);

	// aktuelle Position
	FTyp delta_x= (_letzter_gueltiger_Punkt.x-_Startpunkt.x)* _Pixel_x;

	// ist eine Fuzzy-Menge ausgewhlt ?
	if (_Set) {
		ReleaseCapture();

		// bisherige Darstellung lschen
		_Set->malen(mydc,bild,_min,_max, TColor::White);

		// all anderen Fuzzy-Mengen wiederherstellen
		wiederherstellen(_Set,*_Set,mydc,bild);

		// Verschiebungen fixieren
		if (_Modus==mitte)
			_Set->verschieben(delta_x);
		else {
			if (_Modus==links)
				_Set->verschiebe_links(delta_x);
			else
				_Set->verschiebe_rechts(delta_x);
		}

		// modifiziertes Set neuzeichnen
		_Set->malen(mydc,bild,_min,_max, TColor::Black);

		// wieder Standardtitel
		Titel();

		// ab jetzt wieder kein Set ausgewhlt
		_Set=0;
	}
}


void TSetEdit::EvLButtonUp(UINT, TPoint& point) {

	// das Kontexmen wird erstellt und angezeigt

	TPopupMenu PopupMenu;
	TRect rect;
	GetWindowRect(rect);

	PopupMenu.AppendMenu(MF_STRING, CM_PFCLIPBOARD, "Copy into clipboard");
	PopupMenu.AppendMenu(MF_SEPARATOR);
	PopupMenu.AppendMenu(MF_STRING, CM_PFHINZUFUEGEN, "Add fuzzy-set...");
	PopupMenu.AppendMenu(MF_STRING, CM_PPARTITION, "Create default partition...");

	// falls gerade auf eine Fuzzy-Menge gezeigt wird, Eigenschaften und
	// Lschen anbieten

	if (_InfoSet) {
		PopupMenu.AppendMenu(MF_STRING, CM_PFLOESCHEN, "Delete fuzzy-set");
		PopupMenu.AppendMenu(MF_SEPARATOR);
		PopupMenu.AppendMenu(MF_STRING, CM_PFEIGENSCHAFTEN, "Properties...");
	}

	// Men anzeigen
	PopupMenu.TrackPopupMenu(TPM_LEFTBUTTON,
										point.x+rect.left, point.y+rect.top,
										0,HWindow,0);
}
void TSetEdit::EvRButtonDblClk(UINT, TPoint&) {

	// falls eine Fuzzy-Menge selektiert ist und ein Doppelklick nit dem
	// rechten Mausknopf erfolgte, Eigenschaften anzeigen

	if (_InfoSet) {
		CmPFEigenschaften();
	}
	else
		MessageBeep(0);
}
void TSetEdit::CmPFEigenschaften() {

	// Kontextdialog der Fuzzy-Menge erstellen und ausfhren

	FuzzyDialog Dialog(this,_Var,_InfoSet,!(_ist_AusgabeVar || _ist_FehlerVar));
	Dialog.Execute();

	if (Dialog.war_ok()) {

		// neue Fuzzy-Menge aus dem Kontextdialog holen
		FsSet& neu=Dialog.Ergebnis_holen();

		// alte Association Name->Fuzzy-Set aus Hash-table lschen.
		FsSet *altes_Set;
		mystring *alter_Name;
		AssociationType assoc(&_InfoSet->Name(),0);
		alter_Name=(mystring *)_FuzzySets->Find(assoc)->Key();
		altes_Set=(FsSet *)_FuzzySets->Find(assoc)->Value();
		AssociationType assoc2(alter_Name,altes_Set);
		_FuzzySets->Detach(assoc2,0);
		delete alter_Name,altes_Set;

		if (_Var->ex_Set(neu.Name())) {

			AssociationType assoc(&neu.Name(),0);

			alter_Name=(mystring *)_FuzzySets->Find(assoc)->Key();
			altes_Set=(FsSet *)_FuzzySets->Find(assoc)->Value();

			AssociationType assoc2(alter_Name,altes_Set);
			_FuzzySets->Detach(assoc2,0);

			delete alter_Name,altes_Set;
		}


		// neue Association mit den aktuellen Eigenschaften aus dem
		// Kontexdialog einfgen

		FsSet *neues_Set;
		neues_Set=new FsSet(neu.Name());
		*neues_Set=neu;
		mystring* neuer_Name;
		neuer_Name=new mystring(neu.Name());
		AssociationType assoc3(neuer_Name,neues_Set);
		_FuzzySets->Add(assoc3);

		// neuzeichnen
		Invalidate(true);
		_InfoSet=0;
	}
}



void TSetEdit::CmPFLoeschen() {

	// die ausgewhlte Fuzzy-Menge wird nach einer Sicherheitsabfrage gelscht

	mystring Frage = "Delete fuzzy-set " + _InfoSet->Name() + " ?";

	if (MessageBox(Frage.c_str(),"Delete", MB_YESNO | MB_ICONQUESTION)==IDYES) {

		// alte Association aus dem Hash-table lschen
		FsSet *altes_Set;
		mystring *alter_Name;

		AssociationType assoc(&_InfoSet->Name(),0);

		alter_Name=(mystring *)_FuzzySets->Find(assoc)->Key();
		altes_Set=(FsSet *)_FuzzySets->Find(assoc)->Value();

		AssociationType assoc2(alter_Name,altes_Set);
		_FuzzySets->Detach(assoc2,0);

		delete alter_Name,altes_Set;
		_InfoSet=0;

		// neuzeichnen
		Invalidate(true);
	}


}

void TSetEdit::CmPFHinzufuegen() {

	// eine neue Fuzzy-Menge soll der Variablen hinzugefgt werden

	// Name vorschlagen
	FsSet* neues_Set= new FsSet("new");

	// Eckpunkte vorschlagen
	FTyp x1,x2;
	x1=_Var->min_holen()+(_Var->max_holen()-_Var->min_holen())*0.5;
	x2=_Var->min_holen()+(_Var->max_holen()-_Var->min_holen())*0.75;

	// Intervall einfgen
	neues_Set->Intervall_einfuegen(x1,x2,_Var->min_holen(),_Var->max_holen());
	neues_Set->setze_Grenze(x1,_Var->max_holen());

	// Kontextdialog dieser Fuzzy-Menge aufrufen
	FuzzyDialog Dialog(this,_Var,neues_Set,!(_ist_AusgabeVar || _ist_FehlerVar));

	Dialog.Execute();

	if (Dialog.war_ok()) {

		FsSet& neu=Dialog.Ergebnis_holen();

		// alte Association Name<->Fuzzy-Set aus Hash-table lschen, falls es
		// sie gibt
		if (_Var->ex_Set(neu.Name())) {

			FsSet *altes_Set;
			mystring *alter_Name;

			AssociationType assoc(&neu.Name(),0);

			alter_Name=(mystring *)_FuzzySets->Find(assoc)->Key();
			altes_Set=(FsSet *)_FuzzySets->Find(assoc)->Value();

			AssociationType assoc2(alter_Name,altes_Set);
			_FuzzySets->Detach(assoc2,0);

			delete alter_Name,altes_Set;
		}

		// neue Association mit der Fuzzy-Menge aus dem Kontextdialog
		// einfgen

		FsSet *neues_Set2;
		neues_Set2=new FsSet(neu.Name());
		*neues_Set2=neu;
		mystring* neuer_Name;

		neuer_Name=new mystring(neu.Name());

		AssociationType assoc3(neuer_Name,neues_Set2);
		_FuzzySets->Add(assoc3);

		// neuzeichnen
		Invalidate(true);
	}

	delete neues_Set;

}

void TSetEdit::CmPFClipboard() {

	// kopiert die Darstellung in die Zwischenablage

	Invalidate(false);
	TClientDC clientDC(*this);
	TMemoryDC memDC(clientDC);
	TRect groesse=GetClientRect();

	// Bitmap erstellen

	TBitmap* bitmap=new TBitmap(clientDC,groesse.Width(),groesse.Height(),FALSE);
	memDC.SelectObject(*bitmap);
	memDC.BitBlt(0,0,groesse.Width(),groesse.Height(),clientDC,0,0,SRCCOPY);

	// in die Zwischenablage kopieren

	TClipboard &clipboard = OpenClipboard();
	clipboard.EmptyClipboard();
	bitmap->ToClipboard(clipboard);
	clipboard.CloseClipboard();

	delete bitmap;

}
void TSetEdit::CmPPartition() {

	// partitioniert die Variable

	// Dialog aurufen
	PartDialog Dialog(this);
	Dialog.Execute();

	if (Dialog.war_ok()) {

		// alle Fuzzy-Mengen werden gelscht !!!

		_Set=_InfoSet=0;
		_Var->alle_FuzzySets_loeschen();

		// Partitionierungsfunktion aufrufen
		partitionieren(!(_ist_AusgabeVar || _ist_FehlerVar),
								Dialog.Anzahl_FuzzySets_holen());

		// neuzeichnen
		Invalidate(true);
	}
}

void TSetEdit::partitionieren(bool Dreiecke_erlaubt, int Anzahl) {

	// koordiniert die automatische Partitionierung

	einfuegen(true,Anzahl,Dreiecke_erlaubt);
	einfuegen(false,Anzahl,Dreiecke_erlaubt);

}
mystring TSetEdit::Name_bestimmen(bool positiv,int Nr) {

	// erstellt einen Namen fr eine Fuzzy-Menge bei der automatischen
	// Partitionierung

	if (positiv) {
		switch (Nr) {
			case 0:
				return "pz";
			case 1:
				return "ps";
			case 2:
				return "pb";
			default: {
				char puffer[20];
				sprintf(puffer,"pb%d",Nr-2);
				return puffer;
			}
		}
	}
	else {
		switch (Nr) {
			case 0:
				return "nz";
			case 1:
				return "ng";
			case 2:
				return "nb";
			default: {
				char puffer[20];
				sprintf(puffer,"nb%d",Nr-2);
				return puffer;
			}
		}
	}
}
void TSetEdit::einfuegen_positiv(FTyp x0,FTyp& Intervalllaenge,int Anzahl,bool Dreiecke_erlaubt) {

	// partitioniert den positiven Wertebereich der Variablen

	for (int i=0; i < Anzahl;++i) {

				// Fuzzy-Menge basteln
				mystring* neuer_Name=new mystring(Name_bestimmen(true,i));
				FsSet* neues_Set=new FsSet(*neuer_Name);

				if (i==(Anzahl-1)) {				// letztes Fuzzy-Set ?
					  neues_Set->Intervall_einfuegen(x0+(i-1)*Intervalllaenge,
																x0+i*Intervalllaenge,
																_min,
																_max);

					  neues_Set->setze_Grenze(x0,_max);
				}
				else {

					switch (i) {

						case 0:              // nulltes ? -> gespiegelt "\"
							neues_Set->Intervall_einfuegen(x0+Intervalllaenge,
																	 x0,
																	_min,
																	_max);

							neues_Set->setze_Grenze(x0,_max);
							break;

						default:             // sonst -> "/" bzw. "/\"

							neues_Set->Intervall_einfuegen(x0+(i-1)*Intervalllaenge,
																		x0+i*Intervalllaenge,
																		_min,
																		_max);
							if (Dreiecke_erlaubt)
								neues_Set->Intervall_einfuegen(x0+(i+1)*Intervalllaenge,
																		x0+i*Intervalllaenge,
																		_min,
																		_max);

							neues_Set->setze_Grenze(x0,_max);
							break;

						}
				}

				// Association "Name->Fuzzy-Menge" in die Hash-Tabelle einfgen
				AssociationType assoc(neuer_Name,neues_Set);
				_FuzzySets->Add(assoc);
	}
}

void TSetEdit::einfuegen_negativ(FTyp x0,FTyp& Intervalllaenge,int Anzahl,bool Dreiecke_erlaubt) {

	// partitioniert den negativen Wertebereich der Variablen

	for (int i=0; i < Anzahl;++i) {

		// Fuzzy-Menge basteln
		mystring* neuer_Name=new mystring(Name_bestimmen(false,i));
		FsSet* neues_Set=new FsSet(*neuer_Name);

		if (i==(Anzahl-1)) {				// letztes Fuzzy-Set ?
			neues_Set->Intervall_einfuegen(x0  -(i-1)*Intervalllaenge,
													x0  -i*Intervalllaenge,
													_min,
													_max);

			neues_Set->setze_Grenze(_min,x0);
		}
		else {

			switch (i) {

				case 0:              // nulltes ? -> gespiegelt "/
					neues_Set->Intervall_einfuegen(x0 -Intervalllaenge,
															x0,
															_Var->min_holen(),
															_Var->max_holen());

					neues_Set->setze_Grenze(_min,x0);
					break;

				default:             // sonst -> "\" bzw. "/\"

					if (Dreiecke_erlaubt) {
						neues_Set->Intervall_einfuegen(x0-(i+1)*Intervalllaenge,
																			x0-i*Intervalllaenge,
																			_Var->min_holen(),
																			_Var->max_holen());
						neues_Set->Intervall_einfuegen(x0-(i-1)*Intervalllaenge,
																		-i*Intervalllaenge,
																		_Var->min_holen(),
																		_Var->max_holen());
					}
					else {
						neues_Set->Intervall_einfuegen(x0-(i-1)*Intervalllaenge,
																		-i*Intervalllaenge,
																		_Var->min_holen(),
																		_Var->max_holen());
					}

					neues_Set->setze_Grenze(_min,x0);
					break;

			}
		}

		// Association "Name->Fuzzy-Menge" in die Hash-Tabelle einfgen
		AssociationType assoc(neuer_Name,neues_Set);
		_FuzzySets->Add(assoc);

	}
}

void TSetEdit::einfuegen(bool positiv,int Anzahl,bool Dreiecke_erlaubt) {

	// berechnet die Intervalllngen bei der automatischen Partitionierung

	FTyp Intervalllaenge;

	// Definitionsbereich holen
	_min=_Var->min_holen();
	_max=_Var->max_holen();

	// ist eine Grenze des Definitionsbereichs = 0, oder sind beide > 0 ?
	if ((_min * _max) >=0 ) {

		// ja,

		// Intervalllnge bestimmen
		if (Anzahl<=2)
			Intervalllaenge=fabs(fabs(_max)-fabs(_min));
		else
			Intervalllaenge=fabs(fabs(_max)-fabs(_min))/(Anzahl-1);

		// Einfgen aufrufen
		if (_min < 0) {
			einfuegen_negativ(_max,Intervalllaenge,Anzahl,Dreiecke_erlaubt);
		}
		else {
			einfuegen_positiv(_min,Intervalllaenge,Anzahl,Dreiecke_erlaubt);
		}
	}
	else {

		// nein,
		bool min_ist_kleiner=(fabs(_min) < fabs(_max));
		bool gleich=(fabs(_min) == fabs(_max));

		// Intervalllnge bestimmen
		if (Anzahl<=2)
			Intervalllaenge=max(fabs(_min),fabs(_max));
		else
			Intervalllaenge=max(fabs(_min),fabs(_max))/(Anzahl-1);

		if (positiv) {
			// postiven Definitionsbereich partitionieren
			if (min_ist_kleiner || gleich) {
					einfuegen_positiv(0,Intervalllaenge,Anzahl,Dreiecke_erlaubt);
			}
			else {
				FTyp Anzahl2 = _max/Intervalllaenge +1 ;
				einfuegen_positiv(0,Intervalllaenge,Anzahl2,Dreiecke_erlaubt);
			}
		}
		else {
			// negativen Definitionsbereich partitionieren
			if (min_ist_kleiner && !gleich) {
				FTyp Anzahl2 = -(_min/Intervalllaenge) +1;
				einfuegen_negativ(0,Intervalllaenge,Anzahl2,Dreiecke_erlaubt);
			}
			else {
				einfuegen_negativ(0,Intervalllaenge,Anzahl,Dreiecke_erlaubt);
			}
		}
	}
}

TSetEdit::~TSetEdit() {
	// Schrift freigeben
	delete _Schrift;
}
