#include "Variable.h"

Variable::Variable():
	_FuzzySets(50), _min(-100), _max(100) {
	_EditFenster=0;
}

Variable::Variable(mystring& Name, FTyp& min, FTyp& max):
	_min(min), _max(max), _FuzzySets()  {
	_Name=Name;
	_EditFenster=0;
}

ContainerType* Variable::FuzzySets_holen() {

	// Zugriffsfunktion fr den Container mit den Fuzzy-Mengen

	return &_FuzzySets;
}

void Variable::Speicher_freigeben() {

	// lscht alle Namen und Fuzzy-Mengen dieser Variable

	// alle Eintrge der Hash-Tabelle betrachten
	IteratorType Iter(_FuzzySets);
	while (Iter) {
		delete Iter.Current().Key();
		delete Iter.Current().Value();
		Iter++;
	}
}
void Variable::hol_definierte_Grenzen(FTyp& min,FTyp& max) {

	// ermittelt das Intervall, auf dem die Fuzzy-Mengen definiert sind

	FTyp tempmin,tempmax;

	// alle Fuzzy-Mengen betrachten
	IteratorType Iter(_FuzzySets);
	if (Iter) {

		// erste Fuzzy-Menge holen und deren Trger ermitteln
		Iter.Current().Value()->hole_Grenzen(min,max);
		tempmin=min;
		tempmax=max;
		Iter++;

		while (Iter) {

			// nchste Fuzzy-Menge holen und deren Trger damit vergleichen
			Iter.Current().Value()->hole_Grenzen(tempmin,tempmax);
			if (tempmin<min)
				min=tempmin;
			if (tempmax>max)
				max=tempmax;
			Iter++;

		}
	}
	else {
		min=0;
		max=0;
	}
}

void Variable::alle_FuzzySets_loeschen() {

	// lscht alle Fuzzy-Mengen und dann den Container

	Speicher_freigeben();
	_FuzzySets.Flush(0);
}

Variable::~Variable() {

	// belegten Speicher freigeben
	Speicher_freigeben();

}
void Variable::FuzzySet_einfuegen(mystring& Name, FTyp x1, FTyp x2,
											FTyp min_x, FTyp max_x) {

	// fgt ein Fuzzy-Intervall in den Container (=Hash-Tabelle) der Fuzzy-Mengen
	// ein

	mystring* key= new mystring(Name);
	FsSet* temp= new FsSet(Name);

	if (min_x==0 && max_x==0) {
		min_x=_min;
		max_x=_max;
	}
	temp->Intervall_einfuegen(x1,x2,min_x,max_x);
	// association Name->Fuzzy-Menge erstellen
	AssociationType assoc(key,temp);

	// ...und in den Container einfgen
	_FuzzySets.Add(assoc);
}

void Variable::FuzzySet_einfuegen(mystring& Name, FTyp x1, FTyp x2, FTyp x3,
											FTyp min_x, FTyp max_x) {

	// fgt eine Fuzzy-Menge vom Typ Dreieck in den Container (=Hash-Tabelle)
	// der Fuzzy-Mengen ein

	mystring* key= new mystring(Name);
	FsSet* temp= new FsSet(Name);

	if (min_x==0 && max_x==0) {
		min_x=_min;
		max_x=_max;
	}

	temp->Intervall_einfuegen(x1,x2,x3,min_x,max_x);
	// association Name->Fuzzy-Menge erstellen

	AssociationType assoc(key,temp);

	// ...und in den Container einfgen
  _FuzzySets.Add(assoc);
}

mystring& Variable::Name() {

	// Zugriffsfunktion fr den Namen der Variablen

	return _Name;
}
bool Variable::ex_Set(mystring& Name) {

	// enthlt diese Variable eine Fuzzy-Menge mit dem Namen "Name" ?

	IteratorType Iter(_FuzzySets);
	bool found=false;

	while (Iter && !found) {
		if (*Iter.Current().Key()==Name)
			found=true;
		else
			Iter++;
	}
	return found;
}


FsSet* Variable::hol_Set(mystring& Name) {

	// liefert die Fuzzy-Menge mit dem Namen "Name" aus dem Container

	AssociationType assoc(&Name,0);
	return ((FsSet *)_FuzzySets.Find(assoc)->Value());
}

void Variable::FuzzySet_setze_Grenze(mystring& Name, FTyp x1, FTyp x2) {

	// setzt die Grenzen der Fuzzy-Menge mit dem Namen "Name"

	hol_Set(Name)->setze_Grenze(x1,x2);
}

ostream& operator << (ostream& os, Variable& aVar) {

	os << aVar._Name << " " << aVar._min << " " << aVar._max << " ";
	os << aVar._FuzzySets.GetItemsInContainer() << endl;

	IteratorType Iter(aVar._FuzzySets);

	while (Iter) {
		os << *Iter.Current().Value() << endl;
		Iter++;

	}

	os << endl;

	return os;
}

istream& operator >> (istream& is, Variable& aVar) {

	aVar.alle_FuzzySets_loeschen();
	int Anzahl_FuzzySets;

	is >> aVar._Name >> aVar._min >> aVar._max;
	is >> Anzahl_FuzzySets;

	while (Anzahl_FuzzySets) {

		FsSet* temp= new FsSet();

		is >> *temp;
		mystring* key= new mystring(temp->Name());

		AssociationType assoc(key,temp);
		aVar._FuzzySets.Add(assoc);

		Anzahl_FuzzySets--;
	}

	return is;
}

